#include "../df_access_control_lists_v1.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN void DF_API_CALL DFCClearEntranceListBuffer( int connectionId, int number )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, number);

    // zulaessige Bereiche pruefen
    if (!(((number >= 0) && (number <= 3)) || (number == -1)) )
    {
        pChannel->logEnd("");
        return;
    }

    pChannel->ClearEntranceListBuffer(number);

    pChannel->logEnd("");
}

DF_API_EXTERN int DF_API_CALL DFCMakeEntranceList( int connectionId, int number, int recordCount, int size, unsigned char *buffer, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i", __func__, connectionId, number, recordCount, size);

    // zulaessige Bereiche pruefen
    if (!((number >= 0) && (number <= 3)) ||
        (buffer == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // ListNum ist:
    // 0: Zeitzonen     (22 Byte) zu komprimierten  (6 Byte) max.   16 DS moeglich
    // 1: Zeitmodelle   (35 Byte) zu komprimierten  (3 Byte) max.   32 DS moeglich
    // 2: Personalstamm (19 Byte) zu komprimierten (16 Byte) max. 5120 DS moeglich
    // 3: Feiertage     (16 Byte) zu komprimierten  (6 Byte) max.   64 DS moeglich

    // Bereich der gueltigen Nummer pruefen.
    switch ( number )
    {
    case 0:
        if ((recordCount < 1) || (recordCount > 16))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }

        if (size != (recordCount * ENTRANCE_RECORD_SIZE_0))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }
        break;

    case 1:
        if ((recordCount < 1) || (recordCount > 32))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }

        if (size != (recordCount * ENTRANCE_RECORD_SIZE_1))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }
        break;

    case 2:
        if ((recordCount < 1) || (recordCount > 5120))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }

        if (size != (recordCount * ENTRANCE_RECORD_SIZE_2))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }
        break;

    case 3:
        if ((recordCount < 1) || (recordCount > 64))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }

        if (size != (recordCount * ENTRANCE_RECORD_SIZE_3))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }
        break;

    default:
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }


    int iRes = pChannel->MakeEntranceList(number, recordCount, size, buffer);

    pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_SUCCESS, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCLoadEntranceList( int connectionId, int deviceAddress, int number, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        !(((number >= 0) && (number <= 3)) || (number == -1)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes;
    // Kommunikationsversionsnummer
    int commVersion = 0;

    if (pChannel->getProtocolVersion(deviceAddress) == 1)
    {
        // Bei AEIII+ zusaetzliche Pruefung wegen Befehlsunterstuetzung notwendig
        commVersion = pChannel->getProtocolVersion2(deviceAddress, TRUE, errorNumber);
        switch (commVersion)
        {
            //case 0x0400: Unterstuetzt kein Zutritt
        case 0x0401:
        case 0x0402:
            break;

        default:
            *errorNumber = IV_ERROR_VERSION_FIRMWARE;
            pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }
    }

    if (commVersion != 0x0400 && commVersion != 0x0401)
    {
        char version[32];
        memset(version, 0, sizeof(version));
        // Firmwareversion lesen
        if (!(iRes = pChannel->getVersionFirmware(deviceAddress, version, errorNumber)))
        {
            pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }

        // Firmwareversion pruefen
        if (!theApp.m_util.CheckVersionFirmwareZKOld(version))
        {
            *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
            return 0;
        }
    }

    iRes = pChannel->writeEntranceListData(deviceAddress, number, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}
