/**
* @file  AECom.cpp
* @brief Definitionen der Exportroutinen.
*/
//////////////////////////////////////////////////////////////////////

#include "../df_api.h"
#include "../../DatafoxLibraryIV.h"
#include "../../libraryversion.h"

#if defined( _MSC_VER )
#define snprintf	    _snprintf
#endif

#include <boost/property_tree/ptree.hpp>
#include <boost/property_tree/ini_parser.hpp>
#include <boost/foreach.hpp>
#include <string>
#include <set>
#include <exception>
#include <sstream>
#include <regex>

/*
"device.serial";
"device.type";
// Ermitteln der Hardware
"modul.board");
"modul.rs485";
"modul.tcpip";
"modul.modemMC";
"modul.rs485ZK";
"modul.transponder";
"modul.display";
"modul.keypad";
"modul.mmc";
"modul.gps";
"modul.usb";
"modul.radio";
"modul.dcf77";
"modul.inAnalog";
"modul.inDigital";
"modul.outDigital";
"modul.fingerprint";
"modul.externalReader";
"option.ioModule";
"option.tbAdaption");
"option.comServer";
"option.tpWrite";
"option.http";
"option.language";
"option.biokeyUpdate";
"option.setup";
"modul.touch";
"modul.sensor";
"modul.battery";
"modul.charge";	
*/

struct ini_struct
{
public:
    ini_struct()
    {
        // Alle Keys in Map halten
        setting.insert("serial");
        //    setting.insert("version");
        setting.insert("type");
        // Ermitteln der Hardware
        setting.insert("board");
        //    setting.insert("rs485");
        //    setting.insert("tcpip");
        //    setting.insert("modemMC");
        setting.insert("rs485ZK");
        setting.insert("transponder");
        //    setting.insert("display");
        //    setting.insert("keypad");
        //    setting.insert("mmc");
        //    setting.insert("gps");
        //    setting.insert("usb");
        //    setting.insert("radio");
        //    setting.insert("dcf77");
        //    setting.insert("inAnalog");
        //    setting.insert("inDigital");
        //    setting.insert("outDigital");
        //    setting.insert("fingerprint");
        setting.insert("externalReader");
        //    setting.insert("ioModule");
        //    setting.insert("tbAdaption");
        //    setting.insert("comServer");
        //    setting.insert("tpWrite");
        //    setting.insert("http");
        //    setting.insert("language");
        //    setting.insert("fingerprintUpdate");
        //    setting.insert("setup");
        //    setting.insert("touch");
        //    setting.insert("sensor");
        //    setting.insert("battery");
        //    setting.insert("charge");	
    }

public:
    std::set<std::string> setting;
};

// ########################################################
// #####################   EXPORTE    #####################
// ########################################################

#ifdef _MSC_VER
// ########################################################
/**
* @fn    CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi)
* @brief Ermitteln der DLL - Version. Kompatible zu Windows - Standard.
*
* @param pdvi       { Zeiger auf Struktur zur Informationsuebergabe. }
*/
HRESULT DllGetVersion( DLLVERSIONINFO *pdvi )
{
    TCHAR szModuleName[MAX_PATH];
    DWORD dwHandle;
    DWORD dwLen;
    LPVOID lpBuffer = NULL;
    UINT uiLen;

    unsigned char *pucData = NULL;

    // zulaessige Bereiche pruefen
    if ( pdvi == NULL )
        return E_ABORT;

    if ( pdvi->cbSize != sizeof(DLLVERSIONINFO) )
        return E_ABORT;

    // Modulname der DLL
    if ( !GetModuleFileName( theApp._hModule, szModuleName, MAX_PATH ) )
        return E_ABORT;

    // Laenge der Version - Resource
    dwLen = GetFileVersionInfoSize( szModuleName, &dwHandle );
    if ( !dwLen )
        return E_ABORT;

    pucData = new unsigned char[dwLen + 100]; // unicode
    // DLL - Versioninfo ermitteln
    if ( !GetFileVersionInfo( szModuleName, dwHandle, dwLen, (LPVOID)pucData ) )
    {
        delete pucData;
        return E_ABORT;
    }

    // Pointer auf allgemeine Infostruktur ermitteln
    if ( !VerQueryValue( (LPVOID)pucData, TEXT("\\\0"), &lpBuffer, &uiLen ) )
    {
        delete pucData;
        return E_ABORT;
    }
    // Daten zurueckgeben
    VS_FIXEDFILEINFO *pvs_ffi = (VS_FIXEDFILEINFO*)lpBuffer;
    pdvi->dwMajorVersion = pvs_ffi->dwFileVersionMS >> 16;
    pdvi->dwMinorVersion = pvs_ffi->dwFileVersionMS & 0x0000FFFF;
    pdvi->dwBuildNumber  = pvs_ffi->dwFileVersionLS >> 16;
    pdvi->dwPlatformID   = DLLVER_PLATFORM_WINDOWS;

    delete pucData;

    return NOERROR;
}
#endif

DF_API_EXTERN int DF_API_CALL DFCVersionLibrary( char *value, int valueLength )
{
    int id = LOG_CHANNEL_ID;
    ChannelBridge *pChannel = theApp.GetAEChannel( id );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL ) 
        return 0;

    pChannel->logBegin( "%s", __func__ );

    // zulaessige Bereiche pruefen
    if ( (value == NULL) || (valueLength < 12) )
    {
        pChannel->logEnd( "\t%p\t%i\t(0)", value, valueLength );
        return 0;
    }

    strncpy( value, libraryVersion, valueLength - 1 );

    pChannel->logEnd( "\t%s\t%i\t(1)", value, valueLength );

    return 1;
}

DF_API_EXTERN void DF_API_CALL DFCGetErrorText( int connectionId, int errorNumber, int languageId, char *value, int valueLength )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
        return;

    pChannel->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, errorNumber, languageId );

    // zulaessige Bereiche pruefen
    if ( value == NULL )
    {
        pChannel->logEnd( "\t\t%i", valueLength );
        return;
    }

    switch ( errorNumber )
    {
    case IV_ERROR_CRC_PACKET:
        snprintf( value, valueLength, "CRC Fehler." );
        break;

    case IV_ERROR_CHANNEL_OPEN:
        snprintf( value, valueLength, "Oeffnen der Schnittstelle fehlgeschlagen." );
        break;

    case IV_ERROR_CHANNEL_CLOSED:
        snprintf( value, valueLength, "Schnittstelle ist geschlossen." );
        break;

    case IV_ERROR_CHANNEL_COMMUNICATION:
        snprintf( value, valueLength, "Allgemeiner Fehler waehrend Kommunikation." );
        break;

    case IV_ERROR_RECEIVE:
        snprintf( value, valueLength, "Geraet antwortet nicht." );
        break;

    case IV_ERROR_TRANSMIT:
        snprintf( value, valueLength, "Fehler waehrend Senden eingetreten." );
        break;

        /*
        case ERROR_AT_RECIEVING	:
        snprintf( szText, iLength, "Fehler waehrend Empfang eingetreten." );
        break;
        */

    case IV_ERROR_ADDRESS:
        snprintf( value, valueLength, "Antwort von falschem Geraet oder Antwort nicht sinnvoll." );
        break;

    case IV_ERROR_DEVICE_ID_REMOTE:
        snprintf( value, valueLength, "Keine gueltige Busnummer. (1 - 31, 254)" );
        break;

    case IV_ERROR_CHANNEL:
        snprintf( value, valueLength, "Kanal nicht vorhanden oder nicht erstellbar." );
        break;

    case IV_ERROR_FILE_OPEN:
        snprintf( value, valueLength, "Oeffnen einer Datei gescheitert." );
        break;

    case IV_ERROR_FILE_FORMAT:
        snprintf( value, valueLength, "z.B. falsches Format einer Setupdatei." );
        break;

    case IV_ERROR_LIST_PROCESSING:
        snprintf( value, valueLength, "Fehler bei Listendatenverarbeitung." );
        break;

    case IV_ERROR_VERSION_FIRMWARE:
        snprintf( value, valueLength, "Firmwareversion nicht kompatible zu Routine." );
        break;

    case IV_ERROR_DATA_AVAILABLE:
        snprintf( value, valueLength, "Es sind noch Daten im Geraet vorhanden." );
        break;

    case IV_ERROR_USER_CANCEL:
        snprintf( value, valueLength, "Durchfuehrung wurde von Anwender abgebrochen." );
        break;

    case IV_ERROR_DESCRIPTION_RECORD_UNLOADED:
        snprintf( value, valueLength, "Keine Datensatzbeschreibung geladen." );
        break;

    case IV_ERROR_DESCRIPTION_LIST_UNLOADED:
        snprintf( value, valueLength, "Keine Listenbeschreibung geladen." );
        break;

        /*
        case ERROR_FLASH_PROG:
        snprintf( szText, iLength, "Fehler beim Programmieren des Flash, Firmware." );
        break;
        */

    case IV_ERROR_FUNCTION_PARAMETER:
        snprintf( value, valueLength, "Parameter einer Funktion ausserhalb ihres Gueltigkeitsbereiches." );
        break;

    case IV_ERROR_VARIABLE_UNDEFINED:
        snprintf( value, valueLength, "Globale Variable nicht definiert." );
        break;

        /*
        case ERROR_NO_DATA:
        snprintf( szText, iLength, "Anfrage obwohl keine Daten." );
        break;
        */

        /*
        case ERROR_MULTI:
        snprintf( szText, iLength, "Dublikat von Datensaetzen abgefangen." );
        break;
        */

    case IV_ERROR_BOOTVERSION:
        snprintf( value, valueLength, "Geraetedatei nicht mit Bootloaderversion kompatible." );
        break;

        /*
        case ERROR_BOOTFILEVERSION:
        snprintf( szText, iLength, "Firmwaredatei nicht mit Bootloaderversion kompatible." );
        break;
        */

    case IV_ERROR_VARIABLE_VALUE:
        snprintf( value, valueLength, "Wert einer Variablen konnte nicht gesetzt werden." );
        break;

    case IV_ERROR_FUNCTION_UNAVAILABLE:
        snprintf( value, valueLength, "Funktion steht im vorliegenden Fall nicht zur Verfuegung." );
        break;

    case IV_ERROR_PROCESSING:
        snprintf( value, valueLength, "Der Befehl wurde erfolgreich uebermittelt, konnte aber nicht durchgefuehrt werden." );
        break;

    case IV_ERROR_DEVICE_UNKNOWN:
        snprintf( value, valueLength, "Der Geraetetyp ist unbekannt. Update der Software erforderlich." );
        break;

    case IV_ERROR_DEVICEFILE_WRONG:
        snprintf( value, valueLength, "Die angegebene Geraetedatei stimmt nicht mit der vorliegenden Hardware ueberein." );
        break;

    case IV_ERROR_DEVICE_WRONG:
        snprintf( value, valueLength, "Geraetetyp stimmt nicht. z.B. Versuchtes Einspielen von Setupdatei fuer PZE-MasterIV in BDE-MasterIV." );
        break;

    case IV_ERROR_FILE_EXTENSION:
        snprintf( value, valueLength, "Die vorliegende Dateiendung wird nicht unterstuetzt." );
        break;

    case IV_ERROR_ENTRANCE2_OFF:
        snprintf( value, valueLength, "Die Zutrittskontrolle ist deaktiviert." );
        break;

    case IV_ERROR_NAME_UNKNOWN:
        snprintf(value, valueLength, "Der Tabellenname oder Spaltenname ist undefiniert." );
        break;

    case IV_ERROR_HANDLE:
        snprintf( value, valueLength, "Ein angegebener Handle ist nicht gueltig." );
        break;

    case IV_ERROR_NULL_RESULT:
        snprintf( value, valueLength, "Der anzuwendende Filter wuerde keine Datensaetze liefert." );
        break;

    case IV_ERROR_BOF:
        snprintf( value, valueLength, "Sprung fuehrte zum Anfang der Tabelle. (Base of File)" );
        break;

    case IV_ERROR_EOF:
        snprintf( value, valueLength, "Sprung fuehrte zum Ende der Tabelle. (End of File)" );
        break;

    case IV_ERROR_VALUE:
        snprintf( value, valueLength, "Der angegebene Feldwert ist Fehlerhaft." );
        break;

    case IV_ERROR_OVERFLOW:
        snprintf( value, valueLength, "Bei einer Datenuebertragung ist ein Speicherueberlauf eingetreten." );
        break;

    case IV_ERROR_NO_DATA_AVAILABLE:
        snprintf( value, valueLength, "Es liegen keine Daten vor." );
        break;

    case IV_ERROR_PROCESSING_DLL:
        snprintf( value, valueLength, "Der Befehl konnte zwar uebermittelt werden, bei den Daten liegt jedoch ein Fehler vor." );
        break;

    case IV_ERROR_MODULE_BUSY:
        snprintf( value, valueLength, "Das angesprochene Modul ist zur Zeit durch einen anderen Process blockiert." );
        break;

    case IV_ERROR_FILE_LOAD_XML:
        snprintf( value, valueLength, "Fehler beim Laden der XML-Datei zu modifyStudioFile." );
        break;

    case IV_ERROR_FILE_UNKNOWN_XML:
        snprintf( value, valueLength, "Unbekannte Elemente der XML-Datei zu modifyStudioFile." );
        break;

    case IV_ERROR_FILE_CONTENT_XML:
        snprintf( value, valueLength, "Fehler im Dateninhalt der XML-Datei zu modifyStudioFile." );
        break;

    case IV_ERROR_FILE_MODIFY:
        snprintf( value, valueLength, "Fehler bei Uebernahme (Anwendung) der zu modifizierenden Daten" );
        break;

    case IV_ERROR_ALLOC_MEM:
        snprintf( value, valueLength, "Speicherallokation konnte nicht durchgefuehrt werden." );
        break;

    case IV_ERROR_VERSION_DEVICEFILE:
        snprintf( value, valueLength, "Diese Version der Geraetedatei kann aufgrund ihrer Aktualitaet nicht verarbeitet werden." );
        break;

    case IV_ERROR_FILE_PROCESSING:
        snprintf( value, valueLength, "Bei der Verarbeitung der Dateidaten trat ein Fehler auf." );
        break;

    case IV_ERROR_DEVICEFILE_MATCH:
        snprintf( value, valueLength, "Die vorliegende Geraetedatei passt nicht zur vorliegenden Hardware. Oder aber: Im Archiv wurde keine passende Geraetedatei gefunden." );
        break;

    case IV_ERROR_DEVICEFILE_LOAD:
        snprintf( value, valueLength, "Fehler beim Laden der Geraetedatei(archive) aufgetreten." );
        break;

    case IV_ERROR_AC_STARTED:
        snprintf( value, valueLength, "Die Aktive Verbindung ist gestartet." );
        break;

    case IV_ERROR_AC_STOPPED:
        snprintf( value, valueLength, "Die Aktive Verbindung ist nicht gestartet." );
        break;

    case IV_ERROR_AC_PORTINUSE:
        snprintf( value, valueLength, "Der angegebene Listensocket ist bereits in Verwendung." );
        break;

    case IV_ERROR_AC_NO_NOTIFY_RECORD:
        snprintf( value, valueLength, "Die Aktive Meldung von Datensaetzen ist deaktiviert." );
        break;

    case IV_ERROR_AC_LISTENER:
        snprintf( value, valueLength, "Allgemeiner Fehler beim Starten der Aktiven Verbindung." );
        break;

    case IV_ERROR_NEED_PASSWORD:
        snprintf( value, valueLength, "Das angegebene Passwort stimmt nicht mit dem benoetigten ueberein. Es wird ein Passwort gefordert." );
        break;

    case IV_ERROR_PASSWORD_LOCK:
        snprintf( value, valueLength, "Geraet ist aufgrund von fehlerhaften Passworteingaben fuer 60 Sekunden gesperrt." );
        break;

    case IV_ERROR_FILE_WRITE:
        snprintf( value, valueLength, "Das Schreiben in die Datei ist fehlgeschlagen." );
        break;

    case IV_ERROR_FILE_READ:
        snprintf( value, valueLength, "Das Lesen aus der Datei ist fehlgeschlagen." );
        break;

    case IV_ERROR_FILE_CRC:
        snprintf( value, valueLength, "Fehlerhafter Dateninhalt. Die Pruefung auf die Datenkonsistenz per CRC schlug fehl." );
        break;

    case IV_ERROR_MEMORY_FULL:
        snprintf( value, valueLength, "Es steht kein weiterer Speicherplatz zur Verfuegung.\nZ. B. kann die maximale aufnehmbare Anzahl Fingertemplates erreicht sein." );
        break;

    case IV_ERROR_FILE_DATA_FORMAT:
        snprintf( value, valueLength, "Die in der Datei vorliegenden Daten stimmen nicht mit den geforderten ueberein.\nZ. B. in der Datei liegen Fingertemplates vom Typ Idencom-Compact vor, es werden jedoch DIN Erfordert." );
        break;

    case IV_ERROR_NOT_ENOUGH_FLASH:
        snprintf( value, valueLength, "Es steht nicht genuegend Flash-Speicher zur Verfuegung, um die Daten aufzunehmen." );
        break;

    case IV_ERROR_NO_MODULE:
        snprintf( value, valueLength, "Das von Ihnen angesprochene Modul ist eine Optionale Hardware-Erweiterung und ist im vorliegenden Geraet nicht verfuegbar." );
        break;

    case IV_ERROR_VERSION_SETUPFILE:
        snprintf( value, valueLength, "In der Setupdatei werden Erweiterungen verwendet, die einen aktuelleren Firmwarestand voraussetzen." );
        break;

    case IV_ERROR_WRONG_FIRMWARE:
        snprintf( value, valueLength, "Die abzuarbeitende Aufgabe wird durch die vorliegenden Firmwareversion nicht unterstuetzt." );
        break;

    case IV_ERROR_WRONG_CONFIG:
        snprintf( value, valueLength, "Die abzuarbeitende Aufgabe wird durch die vorliegenden Geraetekonfiguration nicht unterstuetzt." );
        break;

    case IV_ERROR_TIMEBOY_I_II_III:
        snprintf( value, valueLength, "Bei dem angesprochenen Timeboy handelt es sich um kein Model der vierer Reihe." );
        break;

    case IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der DFComDLL voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_LIBRARY_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte Soll-Version der DFComDLL vor." );
        break;

    case IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE1_VERSION:
        snprintf( value, valueLength, "Die Setupdatei wird von der vorliegenden Firmwareversion 04.01.xx nicht unterstuetzt." );
        break;

    case IV_ERROR_SETUPFILE_MIN_FIRMWARE1_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der Firmware 04.01.xx voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_FIRMWARE1_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte, jedoch empfohlene, Firmwareversion 04.01.xx vor." );
        break;

    case IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE2_VERSION:
        snprintf( value, valueLength, "Die Setupdatei wird von der vorliegenden Firmwareversion 04.02.xx nicht unterstuetzt." );
        break;

    case IV_ERROR_SETUPFILE_MIN_FIRMWARE2_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der Firmware 04.02.xx voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_FIRMWARE2_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte, jedoch empfohlene, Firmwareversion 04.02.xx vor." );
        break;

    case IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE3_VERSION:
        snprintf( value, valueLength, "Die Setupdatei wird von der vorliegenden Firmwareversion 04.03.xx nicht unterstuetzt." );
        break;

    case IV_ERROR_SETUPFILE_MIN_FIRMWARE3_VERSION:
        snprintf( value, valueLength, "Die Setupdatei setzt eine nicht erreichte Mindestversion der Firmware 04.03.xx voraus." );
        break;

    case IV_ERROR_SETUPFILE_SHOULD_FIRMWARE3_VERSION:
        snprintf( value, valueLength, "Die Setupdatei gibt eine nicht erreichte, jedoch empfohlene, Firmwareversion 04.03.xx vor." );
        break;

    default:
        snprintf( value, valueLength, "Unbekannter Fehler." );
        break;
    }

    pChannel->logEnd( "\t%s\t%i", value, valueLength );
}

DF_API_EXTERN int DF_API_CALL DFCCheckDevice( int connectionId, int deviceAddress, int *errorNumber, int devicePollRetry )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) || (devicePollRetry < 1))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER, devicePollRetry);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t%i\t(0)", *errorNumber, devicePollRetry);
        return 0;
    }

    int iRes = pChannel->checkDevice(deviceAddress, errorNumber, devicePollRetry);

    pChannel->logEnd("\t%i\t%i\t(%i)", *errorNumber, devicePollRetry, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetDevicePollRetry( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);
    pChannel->logEnd("\t(%i)", pChannel->_devicePollRetry);

    return pChannel->_devicePollRetry;
}

DF_API_EXTERN int DF_API_CALL DFCComSendMessage( int connectionId, int deviceAddress, unsigned char visibleTime, unsigned char doAction, unsigned char audibleSignal, char *text, int textLength )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i\t%s\t%i", __func__,
        connectionId, deviceAddress, visibleTime, doAction, audibleSignal,
        (text == NULL) ? "" : text, textLength);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        !((textLength >= 0) && (textLength <= 242)) ||
        (text == NULL) )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = pChannel->sendMessage(deviceAddress, visibleTime, doAction, audibleSignal,
        text, textLength, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(iError);

    pChannel->logEnd("\t(%i)", iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComSendInfotext( int connectionId, int deviceAddress, char *text, int textLength )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%s\t%i", __func__,
        connectionId, deviceAddress, (text == NULL) ? "" : text, textLength);

    // zulaessige Bereiche pruefen
    if (!((connectionId >= 0) && (connectionId < MAX_CHANNEL)) ||
        !(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        !((textLength >= 0) && (textLength <= 242)) ||
        (text == NULL) )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = pChannel->sendInfotext(deviceAddress, text, textLength, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(iError);

    pChannel->logEnd("\t(%i)", iRes);
    return iRes;
}


DF_API_EXTERN int DF_API_CALL DFCComSetTime( int connectionId, int deviceAddress, unsigned char *dateTimeBuffer )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) )
    {
        pChannel->logEnd("\t\t(0)");
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = pChannel->setDateTime(deviceAddress, dateTimeBuffer, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(iError);

    // Uhrzeit mitloggen
    if (dateTimeBuffer != NULL)
    {
        pChannel->logEnd("\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i)",
            dateTimeBuffer[0], dateTimeBuffer[1], dateTimeBuffer[2], dateTimeBuffer[3],
            dateTimeBuffer[4], dateTimeBuffer[5], dateTimeBuffer[6], iRes);
    }
    else
    {
        pChannel->logEnd("\t\t(%i)", iRes);
    }
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComGetTime( int connectionId, int deviceAddress, unsigned char *dateTimeBuffer )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (dateTimeBuffer == NULL) )
    {
        pChannel->logEnd("\t\t(0)");
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = pChannel->getDateTime(deviceAddress, dateTimeBuffer, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(iError);

    if (iRes)
    {
        pChannel->logEnd("\t%02i%02i-%02i-%02i %02i:%02i:%02i\t(%i)",
            dateTimeBuffer[0], dateTimeBuffer[1], dateTimeBuffer[2], dateTimeBuffer[3],
            dateTimeBuffer[4], dateTimeBuffer[5], dateTimeBuffer[6], iRes);
    }
    else
    {
        pChannel->logEnd("\t\t(%i)", iRes);
    }

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCComVersion( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getProtocolVersion2(deviceAddress, TRUE, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(0x%04X)", *errorNumber, iRes);
    // Kommunikationsversion z.B: 0x00000103
    // Die Versionsnummer ist ein 16Bit - Wert
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetVersionFirmware( int connectionId, int deviceAddress, char *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (value == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getVersionFirmware(deviceAddress, value, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t(%i)", (iRes) ? value : "", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetVersionFirmwareFromFile( int connectionId, char *filePath, char *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((filePath == NULL) || (value == NULL) || (errorNumber == NULL) ||
        (strlen(filePath) < 1))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = pChannel->getVersionFirmwareFromFile(filePath, value, errorNumber);

    pChannel->logEnd("\t%s\t%s\t%i\t(%i)", filePath, (iRes) ? value : "", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetSeriennummer( int connectionId, int deviceAddress, int *errorNumber, int *value )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) || (value == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen()) 
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getSerialNum(deviceAddress, value, errorNumber);

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t%i\t(%i)", *errorNumber, *value, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCStartTransparencyMode( int connectionId, int deviceAddress, int id, int flags, unsigned int sizeBufferRead, unsigned int sizeBufferWrite, int timeout, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, id, flags, sizeBufferRead, sizeBufferWrite );

    // zulaessige Bereiche pruefen
    if ( !(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() ) 
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%i\t(%i)", *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->startTransparencyMode( deviceAddress, id, flags, sizeBufferRead, sizeBufferWrite, timeout, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%i\t(%i)", *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCStopTransparencyMode( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i", __func__, connectionId, deviceAddress );

    // zulaessige Bereiche pruefen
    if ( !(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() ) 
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%i\t(%i)", *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->stopTransparencyMode( deviceAddress, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%i\t(%i)", *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCIsTransparencyMode( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        return 0;
    }

    pChannel->logBegin( "%s\t%i", __func__, connectionId );

    int iRes = pChannel->isTransparencyMode();

    pChannel->logEnd( "\t(%i)", iRes );

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCPing( int connectionId, int deviceAddress, int attempts, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel( connectionId );
    // Wurde Channel-Objekt ermittelt.
    if ( pChannel == NULL )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_CHANNEL;

        return 0;
    }

    pChannel->logBegin( "%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, attempts );

    // zulaessige Bereiche pruefen
    if ( !(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254))
        || attempts < 1 || attempts > 1000 || (errorNumber == NULL) )
    {
        if ( errorNumber )
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;

        pChannel->logEnd( "\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0 );
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if ( !pChannel->isOpen() ) 
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd( "\t%i\t(%i)", *errorNumber, 0 );
        return 0;
    }

    int iRes = pChannel->ping( deviceAddress, attempts, errorNumber );

    // Nach Kommunikationsfehler, Kanal bereinigen
    if ( iRes == 0 )
        pChannel->resetWithError( *errorNumber );

    pChannel->logEnd( "\t%i\t(%i)", *errorNumber, iRes );

    return iRes;
}

DF_API_EXTERN void DF_API_CALL DFCSetCallBack( int connectionId, int (*functionPointer)() )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return;
    }

    pChannel->logBegin("%s\t%i\t", __func__, connectionId);

    pChannel->setCallBackFunction( functionPointer );

    pChannel->logEnd("");
}

DF_API_EXTERN int DF_API_CALL DFCSetGlobVar( int connectionId, int deviceAddress, unsigned char *name, int nameType, unsigned char *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (name == NULL) || (value == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t\t%i\t(%i)", nameType, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich pruefen.
    switch ( nameType )
    {
    case 0: // Ansprechen ueber Nummer
        if (*name < 1 || *name > 8)
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t\t%i\t(%i)", nameType, *errorNumber, 0);
            return 0;
        }
        pChannel->logAppend("\t%i\t%i\t%s", *name, nameType, (char*)value);
        break;

    case 1: // Ansprechen ueber Name
        if ((strlen((const char*)name) < 1) || (strlen((const char*)name) > MAX_NAME))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t\t%i\t(%i)", nameType, *errorNumber, 0);
            return 0;
        }
        pChannel->logAppend("\t%s\t%i\t%s", (char*)name, nameType, (char*)value);
        break;

    case 2: // Ansprechen ueber Systemname
        if (strlen((const char*)name) < 1)
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t\t%i\t(%i)", nameType, *errorNumber, 0);
            return 0;
        }
        // Bereichspruefung der Systemvariablen
        if ((strstr((char*)name, "SERVICE.DBLINPUT") && (atoi((char*)value) < 0 || atoi((char*)value) > 65499)) ||
            (strstr((char*)name, "SERVICE.DEMANDTIMER") && (atoi((char*)value) < 0 || atoi((char*)value) > 255)) )
        {
            *errorNumber = IV_ERROR_VARIABLE_VALUE;
            pChannel->logEnd("\t\t%i\t\t%i\t(%i)", nameType, *errorNumber, 0);
            return 0;
        }
        pChannel->logAppend("\t%s\t%i\t%s", (char*)name, nameType, (char*)value);
        break;

    default:
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t\t%i\t(%i)", nameType, *errorNumber, 0);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->setGlobVar(deviceAddress, name, nameType, (char*)value, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetGlobVar( int connectionId, int deviceAddress, unsigned char *name, int nameType, unsigned char *value, int valueLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (name == NULL) || (value == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", nameType, valueLength, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich pruefen.
    switch ( nameType )
    {
    case 0: // Ansprechen ueber Nummer
        if (*name < 1 || *name > 8)
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", nameType, valueLength, *errorNumber, 0);
            return 0;
        }
        pChannel->logAppend("\t%i\t%i", *name, nameType);
        break;

    case 1: // Ansprechen ueber Name
        if ((strlen((const char*)name) < 1) || (strlen((const char*)name) > MAX_NAME))
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", nameType, valueLength, *errorNumber, 0);
            return 0;
        }
        pChannel->logAppend("\t%s\t%i", (char*)name, nameType);
        break;

    case 2: // Ansprechen ueber Systemname
        if (strlen((const char*)name) < 1)
        {
            *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", nameType, valueLength, *errorNumber, 0);
            return 0;
        }
        pChannel->logAppend("\t%s\t%i", (char*)name, nameType);
        break;

    default:
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t\t%i\t%i\t(%i)", nameType, valueLength, *errorNumber, 0);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t\t%i\t%i\t(%i)", valueLength, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getGlobVar(deviceAddress, name, nameType, (char*)value, valueLength, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (char*)value, valueLength, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCCloseRelay( int connectionId, int deviceAddress, int number, int timeout, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number, timeout);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich der gueltigen Nummer pruefen.
    if ((number < 0) || (number > 8))
    {
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->closeRelay(deviceAddress, number, timeout, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetRelayState( int connectionId, int deviceAddress, int number, int *state, int *timeout, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (state == NULL) || (timeout == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich der gueltigen Nummer pruefen.
    if ((number < 0) || (number > 8))
    {
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t\t\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getRelayState(deviceAddress, number, state, timeout, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t%i\t%i\t(%i)", *state, *timeout, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCOpenRelay( int connectionId, int deviceAddress, int number, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, number);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich der gueltigen Nummer pruefen.
    if ((number < 0) || (number > 8))
    {
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->openRelay(deviceAddress, number, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetInfo( int connectionId, int deviceAddress, char *name, char *params, char *text, int *textLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (name == NULL) || (params == NULL) || (text == NULL) || (textLength == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%s\t%s\t%i\t%i\t(%i)", (name == NULL) ? "" : name, (params == NULL) ? "" : params,
            (text == NULL) ? "" : text, (textLength == NULL) ? 0 : *textLength, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%s\t%s\t%i\t%i\t(%i)", name, params, text, *textLength, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getInfo(deviceAddress, name, params, text, (int&)*textLength);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 6:
                *errorNumber = IV_ERROR_ENTRANCE2_OFF;
                break;

            case 7:
                *errorNumber = IV_ERROR_MODULE_BUSY;
                break;

            case 28:
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29:
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%s\t%s\t%i\t%i\t(%i)", name, params, text, *textLength, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCOpenComServerMode( int connectionId, int deviceAddress, char *comm, int baudRate, int byteSize, int parity, int stopBits, int flags, int timeoutToClose, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen // HK erweitert um 57600 und 115200
    if (/*!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254))*/ (deviceAddress != 254) ||
        (comm == NULL) || ((baudRate != 9600) && (baudRate != 19200) && (baudRate != 38400) && (baudRate != 57600) && (baudRate != 115200)) || (flags & 0xFFFF0000) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t0x%04X\t%i\t%i\t(%i)", (comm == NULL) ? "" : comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int iRes = pChannel->openComServerMode(deviceAddress, comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1: // Es wurden zu wenige Paramteter angegeben.
            case 2: // Es wurden zu viele Paramteter angegeben.
            case 3: // Die Angabe des comm Parameters wird nicht unterstuetzt.
            case 4: // Die Angabe des baudRate Parameters wird nicht unterstuetzt.
            case 5: // Die Angabe des byteSize Parameters wird nicht unterstuetzt.
            case 6: // Die Angabe des parity Paramters wird nicht unterstuetzt.
            case 7: // Die Angabe des stopBits Paramters wird nicht unterstuetzt.
            case 8: // Die Angabe des flags Parameters wird nicht unterstuetzt.
                *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
                break;

            case 28: // Passwortangabe fehlerhaft
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29: // 3 malige flascheingabe, Geraet geblockt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t%i\t%i\t%i\t0x%04X\t%i\t%i\t(%i)", comm, baudRate, byteSize, parity, stopBits, flags, timeoutToClose, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCCloseComServerMode( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (/*!(((deviceID >= 1) && (deviceID <= 31)) || (deviceID == 254))*/ (deviceAddress != 254) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Eine Sekunde Pause, fuer Pufferleerung.
    Sleep(1000);

    int iRes = pChannel->closeComServerMode(deviceAddress);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten.
            switch (iRes & 0xFF)
            {
            case 1:
                *errorNumber = IV_ERROR_OVERFLOW;
                break;

            case 28: // Passwort fehlerhaft
                *errorNumber = IV_ERROR_NEED_PASSWORD;
                break;

            case 29: // 3 malige flascheingabe, Geraet geblockt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
                break;

            default:
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

/*
mask:
0x00000001 External Reader options 0=GIS, 1=PHG
0x00000002 IOModul
0x00000004 TBAdaption
0x00000008 ComServer
0x00000010 TransponderWrite
0x00000020 UpdateBiokey
0x00000040 Http-Protokoll
0x00000080 Signalverarbeitung

options: zustand 0 ist off, und 1 on
*/
DF_API_EXTERN int DF_API_CALL DFCSetOptionFirmware( int connectionId, int deviceAddress, int mask, int options, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, mask, options);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        !(((unsigned)mask > 0) && ((unsigned)mask <= 0x000000FF)) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->setOptionFirmware(deviceAddress, mask, options);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ((iRes & 0xFF) == 28)
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ((iRes & 0xFF) == 29)
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetOptionFirmware( int connectionId, int deviceAddress, int *mask, int *options, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (mask == NULL) || (options == NULL) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t%i\t%i\t(%i)", (mask == NULL) ? 0 : mask, (options == NULL) ? 0 : options, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t%i\t%i\t(%i)", mask, options, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getOptionFirmware(deviceAddress, *mask, *options);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }
        else
        {
            // Funktionsfehler auswerten
            if ((iRes & 0xFF) == 28)
            {
                // Passwort wird benoetigt?
                *errorNumber = IV_ERROR_NEED_PASSWORD;
            }
            else if ((iRes & 0xFF) == 29)
            {
                // Nach 3 maliger Passwortfalscheingabe ist Geraet fuer 60 Sekunden gesperrt.
                *errorNumber = IV_ERROR_PASSWORD_LOCK;
            }
            else
            {
                // Funktionsfehler : Fehler in Verarbeitung.
                *errorNumber = IV_ERROR_PROCESSING;
            }
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t%i\t%i\t(%i)", mask, options, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetFontType( int connectionId, int deviceAddress, int type, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, type);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Bereich der gueltigen Nummer pruefen.
    if ((type < 0) || (type > 6))
    {
        *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->setTypeFont(deviceAddress, type, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCPressVirtualKey( int connectionId, int deviceAddress, int keyCode, int flags, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, keyCode, flags);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (keyCode < 0x70) || (keyCode > 0x7E) || (flags > 1) || (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->pressVirtualKey(deviceAddress, keyCode, flags, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetPasswordKey( int connectionId, int deviceAddress, char *value, int *valueLength, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (value == NULL) || (valueLength == NULL) || (errorNumber == NULL) || (*valueLength < 17))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t%i\t(%i)", (value == NULL) ? "" : value, (valueLength == NULL) ? 0 : *valueLength, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t%i\t(%i)", value, *valueLength, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getPasswordKey(deviceAddress, value);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        *valueLength = (int)strlen(value);
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        } 
        else
        {
            // Funktionsfehler auswerten.
            *errorNumber = IV_ERROR_PROCESSING;
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%s\t%i\t%i\t(%i)", value, *valueLength, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetPassword( int connectionId, char *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((value == NULL) || strlen(value) > 16 || (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t(%i)", (value == NULL) ? "" : "****", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    pChannel->setPassword(value);

    pChannel->logEnd("\t****\t%i\t(%i)", *errorNumber, 1);

    return 1;
}

DF_API_EXTERN int DF_API_CALL DFCReset( int connectionId, int deviceAddress, int mode, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, deviceAddress, mode);

    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) || ((mode != 1) && (mode != 2) && (mode != 3)))
    {
        if (errorNumber != NULL) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
        return 0;
    }

    *errorNumber = IV_ERROR_SUCCESS;
    int iRes;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    iRes = pChannel->reset(deviceAddress, mode, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCGetFlashStatus( int connectionId, int deviceAddress, int *value, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) 
        || (value == NULL) || (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t\t%i\t(0)", IV_ERROR_FUNCTION_PARAMETER);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t%i\t(0)", *value, *errorNumber);
        return 0;
    }

    int iRes = pChannel->getFlashStatus(deviceAddress, value, errorNumber);

    pChannel->logEnd("\t%i\t%i\t(%i)", *value, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCSetCommunicationPassword( int connectionId, unsigned char *value, int valueLength, int sessionTimeout, int *errorNumber )
{
    // Auf Logchannel ausweichen, wenn die Passwortangabe fuer alle gueltig sein soll.
    int id = connectionId;
    if (connectionId == -1)
        id = LOG_CHANNEL_ID;

    ChannelBridge *pChannel = theApp.GetAEChannel(id);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    // zulaessige Bereiche pruefen
    if ((value == NULL) || (valueLength < 0) || (valueLength > 16) || 
        (sessionTimeout < 0) || (sessionTimeout > 0 && sessionTimeout < 60) || (sessionTimeout > 100000) || 
        (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", (value == NULL) ? "" : "****", valueLength, sessionTimeout, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    int res = theApp.SetCommunicationPassword(connectionId, value, valueLength, sessionTimeout);
    if (res == 0) {
        *errorNumber = IV_ERROR_CHANNEL;
    }

    pChannel->logEnd("\t****\t%i\t%i\t%i\t(%i)", valueLength, sessionTimeout, *errorNumber, res);

    return res;
}

DF_API_EXTERN int DF_API_CALL DFCReadHardwareInfo( int connectionId, int deviceAddress, char *value, int *valueLength, int flags, int *errorNumber ) 
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("\t%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) 
        || (value == NULL) || (valueLength == NULL) || (flags != 0) || (errorNumber == NULL)) {

            if (errorNumber != NULL) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
            pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", (value == NULL) ? "" : value, (valueLength == NULL) ? 0 : *valueLength, flags, IV_ERROR_FUNCTION_PARAMETER, 0);
            return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen()) {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", value, *valueLength, flags, *errorNumber, 0);
        return 0;
    }

    int res = 0;
    unsigned char ucVal;
    unsigned int uiVal;
    unsigned short usVal;
    enum { ucType, uiType, usType } putType;

    // Klasse fuer die Verwaltung der Daten der Hardwareinfo
    ini_struct is;
    // Leeres PropertyTree Objekt
    using boost::property_tree::ptree;
    ptree pt;
    int readAll = 0;

    // Laden der vorliegenden Ini-Datei.
    // Ausnahme wird ausgeloest, wenn ein Fehler vorliegt.
    try {
        //read_ini(fileName, pt);
        // Aktuell werden noch alle gelesen
        readAll = 1;
    } catch (boost::property_tree::ini_parser::ini_parser_error &e) {

        // Oeffnen der Datei fehlgeschlagen?
        if (e.message() != "cannot open file") {
            //TRACE("Fehlernachricht: %s\n", e.message().c_str());
        }
        // Es werden alle moeglichen Informationen ermittelt.
        readAll = 1;
    }

    std::set<std::string>::iterator i;
    i = is.setting.begin();
    for (; i != is.setting.end(); i++) {
        if (readAll == 0 && pt.get_child_optional(*i) == boost::none) 
            // Dieser Key soll nicht gelesen werden.
            continue;

        putType = ucType;

        if (*i == "serial") {
            if ((res = pChannel->_channel4.getSerial(deviceAddress, uiVal)) < IV_ERROR_SUCCESS) 
                break;

            putType = uiType;
        } else if (*i == "version") {
            // Version der Hardwareinfo ermitteln
            if ((res = pChannel->_channel4.getHWConfig(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "type") {
            // Geraetetyp ermitteln
            if ((res = pChannel->_channel4.getTypeDevice(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "board") {
            // Boardtyp ermitteln
            if ((res = pChannel->_channel4.getTypeBoard(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "rs485") {
            // RS485 ermitteln
            if ((res = pChannel->_channel4.getComponentRS485_0(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "tcpip") {
            // TCPIP ermitteln
            if ((res = pChannel->_channel4.getTypeTCPIP(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "modemMC") {
            // ModemMC35i ermitteln
            if ((res = pChannel->_channel4.getTypeModemMC(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "rs485ZK") {
            // RS485 (intern) ermitteln
            if ((res = pChannel->_channel4.getComponentRS485_1(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "transponder") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeTransponder(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "display") {
            // Typ ermitteln (erst ab 04.01.01.p / Bootloader 4.0)
            if ((res = pChannel->_channel4.getTypeDisplay(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "keypad") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeKeypad(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "mmc") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeMMC(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "gps") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeGPS(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if( *i == "usb" ) {
            // Komponende ermitteln
            if ((res = pChannel->_channel4.getComponentUSB(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "radio") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeRadio(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "dcf77") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeDCF77(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "inAnalog") {
            // Komponende ermitteln
            if ((res = pChannel->_channel4.getComponentInAnalog(deviceAddress, uiVal)) < IV_ERROR_SUCCESS)
                break;

            putType = uiType;
        } else if (*i == "inDigital") {
            // Komponende ermitteln
            if ((res = pChannel->_channel4.getComponentInDigital(deviceAddress, uiVal)) < IV_ERROR_SUCCESS)
                break;

            putType = uiType;
        } else if (*i == "outDigital") {
            // Komponende ermitteln
            if ((res = pChannel->_channel4.getComponentOutDigital(deviceAddress, uiVal)) < IV_ERROR_SUCCESS)
                break;

            putType = uiType;
        } else if (*i == "fingerprint") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeFingerprint(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "externalReader") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeExternalReader(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "ioModule") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionIOModule(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "tbAdaption") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionTBAdaption(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "comServer") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionComServer(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "tpWrite") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionTPWrite(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "http") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionHTTP(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "language") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionLanguage(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "fingerprintUpdate") {
            // Option ermitteln
            if ((res = pChannel->_channel4.getOptionBiokeyUpdate(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "setup") {
            unsigned short mask = 0;
            // Option ermitteln
            if ((res = pChannel->_channel4.getSetupOptions(deviceAddress, mask, usVal)) < IV_ERROR_SUCCESS)
                break;

            putType = usType;
        } else if (*i == "touch") {
            // Typ ermitteln
            if ((res = pChannel->_channel4.getTypeTouch(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if (*i == "sensor") {
            // Typ ermitteln
            if( (res = pChannel->_channel4.getTypeSensor(deviceAddress, ucVal)) < IV_ERROR_SUCCESS )
                break;

        } else if (*i == "battery") {
            // Komponente ermitteln
            if ((res = pChannel->_channel4.getComponentBattery(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;

        } else if( *i == "charge" ) {
            // Komponente ermitteln
            if ((res = pChannel->_channel4.getComponentCharge(deviceAddress, ucVal)) < IV_ERROR_SUCCESS)
                break;
        } else {
            // Kein bekannter Eintrag.
            continue;
        }

        if (res == IV_ERROR_SUCCESS) 
        {
            switch (putType) {
            case uiType:
                pt.put(*i, uiVal);
                break;

            case usType:
                pt.put(*i, usVal);
                break;

            default:
                pt.put(*i, ucVal);
            }
            //TRACE("Key: %s\n", (*i).c_str());
        }
    }

    try {
        // Write property tree to INI file
        std::ostringstream os;
        write_ini(os, pt);
        os << std::ends;
        strncpy(value, os.str().data(), *valueLength - 1);
        *valueLength = strlen(value);
        res = 1;
    } catch( std::exception &e ) {
        (void)e;
        *errorNumber = IV_ERROR_FILE_WRITE;
        res = 0;
    }

    pChannel->logEnd("\t%s\t%i\t%i\t%i\t(%i)", value, *valueLength, flags, *errorNumber, res);
    return res;
}

DF_API_EXTERN int DF_API_CALL DFCRecordVolume( int connectionId, int deviceAddress, int *recordCount, int *memoryUsage, int *memorySize, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (recordCount == NULL) || (memoryUsage == NULL) || (memorySize == NULL) || (errorNumber == NULL))
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t%i\t%i\t%i\t(%i)", (recordCount == NULL) ? 0 : *recordCount, (memoryUsage == NULL) ? 0 : *memoryUsage, (memorySize == NULL) ? 0 : *memorySize, IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;


    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t%i\t%i\t%i\t(%i)", *recordCount, *memoryUsage, *memorySize, *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->recordVolume(deviceAddress, *recordCount, *memoryUsage, *memorySize);
    if (iRes == IV_ERROR_SUCCESS)
    {
        // Kein Fehler
        *errorNumber = IV_ERROR_SUCCESS;
        iRes = 1;
    }
    else if (iRes > IV_ERROR_SUCCESS)
    {
        // Fehler bei Durchfuehrung des Befehls
        // Modulfehler auswerten.
        if ((iRes >> 8) & 0xFF)
        {
            if (((iRes >> 8) & 0xFF) == 2)
            {
                *errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                *errorNumber = IV_ERROR_PROCESSING;
            }
        } 
        else
        {
            // Funktionsfehler auswerten.
            *errorNumber = IV_ERROR_PROCESSING;
        }

        iRes = 0;
    }
    else
    {
        // Fehler waehrend Kommunikation oder Verarbeitung in DLL
        *errorNumber = -iRes;
        iRes = 0;
    }

    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t%i\t%i\t%i\t(%i)", *recordCount, *memoryUsage, *memorySize, *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCCheckAE( int connectionId, int deviceAddress )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes = pChannel->checkDevice(deviceAddress, &iError, 1);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN void* DF_API_CALL DFCGetClass( int connectionId, int classId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return NULL;
    }

    switch ( classId )
    {
    case 0:
        return pChannel;

    case 1:
    case 2:
        return &pChannel->_channel1_2;

    case 4:
        return &pChannel->_channel4;

    default:
        return NULL;
    }

    return NULL;
}

DF_API_EXTERN int DF_API_CALL DFCCommandInspectionStation( int connectionId, int deviceAddress, int requestAddress, char *requestParams, int requestParamsSize, int *answerAddress, char *answerParams, int *answerParamsSize, int secondsToWaitForAnswer )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return -1;
    }
    // Parameter pruefen
    if (secondsToWaitForAnswer < 1)
    {
        return -IV_ERROR_FUNCTION_PARAMETER;
    }

    return pChannel->_channel4.commandInspectionStation(deviceAddress, requestAddress, requestParams, requestParamsSize, (unsigned char&)*answerAddress, answerParams, (int&)*answerParamsSize, secondsToWaitForAnswer);
}

#ifdef _DEBUG

DF_API_EXTERN int DF_API_CALL DFCWatchdogReset( int connectionId, int deviceAddress )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        //if (piError) *piError = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iError = IV_ERROR_SUCCESS;
    int iRes;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    iRes = pChannel->restart(deviceAddress, &iError);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(iError);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

#endif //_DEBUG

DF_API_EXTERN int DF_API_CALL DFCCustomCommand( int connectionId, int deviceAddress, int identification, int customCommand, char *buffer, int *bufferSize, int *errorNumber )
{
	ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
	// Wurde Channel-Objekt ermittelt.
	if (pChannel == NULL)
	{
		if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
		return 0;
	}

	pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t%i", __func__, connectionId, deviceAddress, customCommand );

	// zulaessige Bereiche pruefen
	if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254))
		|| ( customCommand < 0 )
		|| ((buffer == NULL || bufferSize == NULL))
		|| (errorNumber == NULL) )
	{
		if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
		pChannel->logEnd("\t%s\t%i\t%i\t(%i)",/* (params == NULL) ? "" : params, */(buffer == NULL) ? "" : reinterpret_cast<char*>( buffer ), (bufferSize == NULL) ? 0 : *bufferSize, IV_ERROR_FUNCTION_PARAMETER, 0);
		return 0;
	}
	*errorNumber = IV_ERROR_SUCCESS;


	// Schnittstelle geoeffnet?
	if (!pChannel->isOpen())
	{
		*errorNumber = IV_ERROR_CHANNEL_CLOSED;
		pChannel->logEnd("\t%s\t%i\t%i\t(%i)",/* (params == NULL) ? "" : params,*/ (buffer == NULL) ? "" : reinterpret_cast<char*>( buffer ), (bufferSize == NULL) ? 0 : *bufferSize, *errorNumber, 0);
		return 0;
	}

	int iRes = pChannel->customCommand( deviceAddress, identification, customCommand, reinterpret_cast< char *>( buffer ), bufferSize );
	if (iRes == IV_ERROR_SUCCESS)
	{
		// Kein Fehler
		*errorNumber = IV_ERROR_SUCCESS;
		iRes = 1;
	}
	else if (iRes > IV_ERROR_SUCCESS)
	{
		// Fehler bei Durchfuehrung des Befehls
		// Modulfehler auswerten.
		if ((iRes >> 8) & 0xFF)
		{
			if (((iRes >> 8) & 0xFF) == 2)
			{
				*errorNumber = IV_ERROR_FUNCTION_UNAVAILABLE;
			}
			else
			{
				*errorNumber = IV_ERROR_PROCESSING;
			}
		}
		else
		{
			// Funktionsfehler auswerten.
			switch (iRes & 0xFF)
			{
			case 28:
				*errorNumber = IV_ERROR_NEED_PASSWORD;
				break;

			case 29:
				*errorNumber = IV_ERROR_PASSWORD_LOCK;
				break;

			default:
				*errorNumber = IV_ERROR_PROCESSING;
			}
		}

		iRes = 0;
	}
	else
	{
		// Fehler waehrend Kommunikation oder Verarbeitung in DLL
		*errorNumber = -iRes;
		iRes = 0;
	}

	// Nach Kommunikationsfehler, Kanal bereinigen
	if (iRes == 0) pChannel->resetWithError(*errorNumber);

	pChannel->logEnd("\t%s\t%i\t%i\t(%i)",/* (params == NULL) ? "" : params,*/ (buffer == NULL) ? "" : reinterpret_cast<char*>( buffer ), (bufferSize == NULL) ? 0 : *bufferSize, *errorNumber, iRes);

	return iRes;
}
