#include "../df_setup_lists.h"
#include "../../DatafoxLibraryIV.h"

DF_API_EXTERN int DF_API_CALL DFCLoadListenbeschreibung( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes = pChannel->getTableDescription(deviceAddress, TRUE, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);

    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCListBCnt( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    int iRes = pChannel->GetTableCount(TRUE);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCListBDatensatz( int connectionId, int number, unsigned char *name, int *fieldCount, int *unused_setTo0 )
{
    (void)unused_setTo0;

    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, number);

    // zulaessige Bereiche pruefen
    if (!( ((number >= 0) && (number < MAX_DATENSATZ_BESCHREIBUNG)) ||
        ((number >= 100) && (number < 104)) ||
        ((number >= 200) && (number < 208)) ) ||
        (name == NULL) || (fieldCount == NULL) /*|| (piVar == NULL)*/ )
    {
        pChannel->logEnd("\t\t\t\t(0)");
        return 0;
    }

    unsigned char szTableName[MAX_NAME];
    unsigned char ucFieldCount, ucVariable;

    int iRes = pChannel->GetTableRecord(number, szTableName, &ucFieldCount, &ucVariable, TRUE);

    if (iRes)
    {
        strcpy((char*)name, (const char*)szTableName);
        *fieldCount = ucFieldCount;
        // Variable nicht verwendet, kann NULL uebergeben
    }

    pChannel->logEnd("\t%s\t%i\t0\t(%i)", name, *fieldCount, iRes);

    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCListBFeld( int connectionId, int listBNumber, int number, unsigned char *name, int *type, int *size )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i", __func__, connectionId, listBNumber, number);

    // zulaessige Bereiche pruefen
    if (!( ((listBNumber >= 0) && (listBNumber < MAX_DATENSATZ_BESCHREIBUNG)) ||
        ((listBNumber >= 100) && (listBNumber < 104)) ||
        ((listBNumber >= 200) && (listBNumber < 208)) ) ||
        !((number >= 0) && (number < MAX_FELD)) ||
        (name == NULL) || (type == NULL) || (size == NULL) )
    {
        pChannel->logEnd("\t\t\t\t(0)");
        return 0;
    }

    unsigned char szFieldName[MAX_NAME];
    unsigned char ucType, ucLength;

    int iRes = pChannel->GetTableField(listBNumber, number, szFieldName,
        &ucType, &ucLength, TRUE);

    if (iRes)
    {
        strcpy((char*)name, (const char*)szFieldName);
        *type   = ucType;
        *size = ucLength;
    }

    pChannel->logEnd("\t%s\t%i\t%i\t(%i)", name, *type, *size, iRes);
    return iRes;
}

DF_API_EXTERN int DF_API_CALL DFCMakeListe( int connectionId, int number, int recordCount, int size,
                                           unsigned char *buffer, int unused_setTo0 )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i\t%i\t%i\t\t%i", __func__, connectionId,
        number, recordCount, size, unused_setTo0);

    // zulaessige Bereiche pruefen
    if (false == (number >= 0 && number < MAX_DATENSATZ_BESCHREIBUNG)
        || (buffer == nullptr && (recordCount != 0 || size != 0)) )
    {
        pChannel->logEnd("\t(0)");
        return 0;
    }

    int iRes = pChannel->MakeListe(number, recordCount, size, buffer);

    pChannel->logEnd("\t(%i)", iRes);

    return iRes;
}

DF_API_EXTERN void DF_API_CALL DFCClrListenBuffer( int connectionId )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        return;
    }

    pChannel->logBegin("%s\t%i", __func__, connectionId);

    pChannel->ClearListBuffer();

    pChannel->logEnd("");
}

DF_API_EXTERN int DF_API_CALL DFCLoadListen( int connectionId, int deviceAddress, int *errorNumber )
{
    ChannelBridge *pChannel = theApp.GetAEChannel(connectionId);
    // Wurde Channel-Objekt ermittelt.
    if (pChannel == NULL)
    {
        if (errorNumber) *errorNumber = IV_ERROR_CHANNEL;
        return 0;
    }

    pChannel->logBegin("%s\t%i\t%i", __func__, connectionId, deviceAddress);

    // zulaessige Bereiche pruefen
    if (!(((deviceAddress >= 1) && (deviceAddress <= 250)) || (deviceAddress == 254)) ||
        (errorNumber == NULL) )
    {
        if (errorNumber) *errorNumber = IV_ERROR_FUNCTION_PARAMETER;
        pChannel->logEnd("\t%i\t(%i)", IV_ERROR_FUNCTION_PARAMETER, 0);
        return 0;
    }
    *errorNumber = IV_ERROR_SUCCESS;

    // Schnittstelle geoeffnet?
    if (!pChannel->isOpen())
    {
        *errorNumber = IV_ERROR_CHANNEL_CLOSED;
        pChannel->logEnd("\t%i\t(%i)", *errorNumber, 0);
        return 0;
    }

    int iRes;
    // Listendaten wegschreiben.
    if (pChannel->logIsWriteList())
    {
        char fileNameBin[_MAX_PATH];
        char basePath[_MAX_PATH];
        int listNum;
        time_t t;
        tm *ltm;

        t = time(NULL);
        ltm = localtime(&t);

        for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
        {
            // Dateiname der Logdatei
            sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, pChannel->logGetBasePath(basePath),
                ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, 'L', listNum);

            // Schreiben der Listendaten in die Datei.
            pChannel->WriteListToBinFile( listNum, fileNameBin );
        }
    }

    // Listendaten wegschreiben.
    if (pChannel->logIsWriteList() > 1)
    {
        char fileNameTxt[_MAX_PATH];
        char basePath[_MAX_PATH];
        unsigned char listName[32], fieldCount, varList;
        int listNum;
        time_t t;
        tm *ltm;

        // Laden der Listenbeschreibungen
        iRes = pChannel->getTableDescription(deviceAddress, TRUE, errorNumber);
        // Nach Kommunikationsfehler, Kanal bereinigen
        if (iRes == 0)
        {
            pChannel->resetWithError(*errorNumber);
            pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);
            return iRes;
        }

        t = time(NULL);
        ltm = localtime(&t);

        for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
        {
            if (!pChannel->GetTableRecord(listNum, listName, &fieldCount, &varList, TRUE))
                break;

            // Dateiname der Logdatei
            sprintf(fileNameTxt, LOG_LIST_FILENAME_TXT, pChannel->logGetBasePath(basePath),
                ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, 'L', listName);

            // Schreiben der Daten in die Datei.
            pChannel->WriteListToTextFile(listNum, fileNameTxt);
        }
    }

    iRes = pChannel->writeListData(deviceAddress, errorNumber);
    // Nach Kommunikationsfehler, Kanal bereinigen
    if (iRes == 0) pChannel->resetWithError(*errorNumber);
#ifdef _DEBUG
    // Listendaten nach der Sortierung wegschreiben.
    if (pChannel->logIsWriteList())
    {
        char fileNameBin[_MAX_PATH];
        char basePath[_MAX_PATH];
        int listNum;
        time_t t;
        tm *ltm;

        t = time(NULL);
        ltm = localtime(&t);
        // Änderung des Dateinamens sicherstellen.
        ltm->tm_sec++;

        for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
        {
            // Dateiname der Logdatei
            sprintf(fileNameBin, LOG_LIST_FILENAME_BIN, pChannel->logGetBasePath(basePath),
                ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                ltm->tm_hour, ltm->tm_min, ltm->tm_sec, connectionId, 'L', listNum);

            pChannel->WriteListToBinFile(listNum, fileNameBin);
        }
    }

    // Listendaten wegschreiben.
    if (pChannel->logIsWriteList() > 1)
    {
        char fileNameTxt[_MAX_PATH];
        char basePath[_MAX_PATH];
        unsigned char listName[32], fieldCount, varList;
        int listNum;
        time_t t;
        tm *ltm;

        // Datensatzbeschreibung ist noch von vorher geladen.

        t = time(NULL);
        ltm = localtime(&t);
        // Änderung des Dateinamens sicherstellen.
        ltm->tm_sec++;

        for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
        {
            if (!pChannel->GetTableRecord(listNum, listName, &fieldCount, &varList, TRUE))
                break;

            // Dateiname der Logdatei
            sprintf(fileNameTxt, LOG_LIST_FILENAME_TXT, pChannel->logGetBasePath(basePath),
                ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday,
                ltm->tm_hour, ltm->tm_min, ltm->tm_sec + 1, connectionId, 'L', listName);


            // Daten in die Datei schreiben.
            pChannel->WriteListToTextFile(listNum, fileNameTxt);
        }
    }
#endif
    pChannel->logEnd("\t%i\t(%i)", *errorNumber, iRes);

    return iRes;
}
