#ifndef _CHANNEL4_H_
#define _CHANNEL4_H_

// Basisklasse einbinden
#include "Channel4ModuleTable.h"

// Klasse für den Datenspeicher einbinden.
#include "Channel4Buffer.h"

// defines fuer Setup
#include "../Common/aes.h"

// Geraetedateiarchiv einbinden
#include "../File/filedevicezip.h"

// Studiodatei einbinden
#include "../File/filestudio.h"

#include "../File/filehardwareinfo.h"

#include "../Common/deviceinfo.h"
#include "../Common/tcpipinfo.h"

#include <string>

// Defines setzen, wenn nicht bereits geschehen
#ifndef _MSC_VER

#ifndef WINAPI
#define WINAPI __stdcall
#endif

typedef void *LPVOID;
typedef unsigned int DWORD;

#endif

#ifdef _MSC_VER
#ifndef S_ISREG
#define S_ISREG(m) ((m & _S_IFMT) == _S_IFREG)
#endif

#ifndef S_ISDIR
#define S_ISDIR(m) ((m & _S_IFMT) == _S_IFDIR)
#endif
#endif

#ifndef STRUCT_MEM_FILE_HEADER
#define STRUCT_MEM_FILE_HEADER
struct StructMemFileHeader {
    int structSize;          // Strukturgroesse
    int type;                // Speichertyp
    int baseAddress;         // Basisadresse
    int memAddress;          // Anfangsadresse
    int memSize;
    char tmp[76];
    //...
    int crcHead;    // 32bit CRC fuer Head (Immer letzter Wert in der Struktur; 
    // Berechnungsmenge: (sizeof(SDFFileDeviceHeader) - sizeof(unsigned short)), also ohne sich selbst!)
};
#endif

// Struktur für die Datei der Fingertemplates
//############################################
// Innerer Aufbau der Datei ist wie folgt:
// StructFTFileHeader, (count * [(type == 0) ? structFingertemplateDIN : structFingertemplateIdencom, CRC16 ])
// - Zuerst liegt die Headerstruktur StructFTFileHeader in der Datei.
// - Der erste 16bit-Wert der Datei muss 0xDFDF sein.
// - Der crc der Headerstruktur berechnet sich über die Struktur ohne die zwei Byte des CRC selbst.
// - Die Daten beginnen ab der Position Seek(StructFTFileHeader.offs, Filebegin).
// - Der Datenteil besteht aus StructFTFileHeader.count Segmenten, die folgenden Aufbau haben.
//   + Zuerst kommt die Sturktur des Fingertemplates je nach StructFTFileHeader.type
//   + Darauf folgt ein 16bit CRC der sich über diese Struktur berechnet.
struct StructFTFileHeader {
  unsigned short Id;    // Kennung der Datei: muss immer 0xDFDF sein.
  unsigned short offs;  // Offset auf den Datenbeginn, normalerweise Strukturgröße, falls Erweiterung nötig.
  unsigned short count; // Anzahl enthaltener Fingertemplates (zur Sicherheit mit aufgenommen)
  unsigned char type;   // Templatetyp. 0=>DIN V66400 (161Byte), 1=>Idencom-Compact (216Byte)
  unsigned short crc;   // 16bit CRC über die Headerstruktur (ohne diesen letzten 16bit Wert)
};

class Channel4 : public Channel4ModuleTable, public Channel4Buffer
{
public:
    Channel4( short channelID = IV_DEFAULT_CHANNELID, short deviceID = IV_DEFAULT_DEVICEID );

    virtual ~Channel4();

    friend class ChannelBridge;

    // Ist das Kanalobjekt im Wartungsmodus.
    inline void setServiceState(int state) 
    { 
        _serviceState = state; 
    }
    inline int getServiceState() 
    { 
        return _serviceState; 
    }

    /// Schliessen der Schnittstelle.
    virtual int close(void)
    {
        // Ist zusaetzlich in Verwendung als Aktiver Kanal?
        if (getActiveState() > is_free)
        {
            directClose();
        }

        return Channel4ProtocolCrypt::close();
    }

    // Verbindungsannahme von uebergebenem Listen-Socket
    int connect(SOCKET *listenSocket, short channelID, short deviceID = IV_DEFAULT_DEVICEID, int timeout = 3000, int aliveTO = 10, int infoFlags = 0);
    // Endesignal im Workerthread setzen und auf dessen Beendigung warten
    void directClose();

    // Aktuelle Info über den Verbundenen Teilnehmen abrufen.
    bool getInfoActiveChannel( char *infoString, int *infoStringLength );

// Main - Funktion fuer Befehlsausfuehrung in Workerthread
public:
    tcpipInfo m_tcpipInfo;
    int m_notify;
protected:
#ifdef _MSC_VER
    static unsigned long WINAPI communicationThread(LPVOID lpParam);
#else
    static void *communicationThread(void *lpParam);
#endif
    volatile int _endSignal;
    volatile int _initSignal;
#ifdef _MSC_VER
    HANDLE _communicationThread;
    DWORD _threadID;
#else
    pthread_t _communicationThread;
#endif
    // Socket fuer connect
    SOCKET *_listenSocket;
    int _aliveTO;
    int _infoFlags;
    volatile enum threadCommand {
        commandIdle,
        commandSuspending,
        commandSuspend,
        commandResuming,
        commandResume
    } _threadCommand;
    volatile int _suspendCounter;
    unsigned char _serviceState;

public:
    // Ausfuehren eines Protokollbefehls
    virtual int call(short context, short module, short func, unsigned char *buf, int &count, int contextValue = -1);

    // Kommunikations-Worker anhalten.
    virtual int suspend();
    // Kommunikations-Worker weiterlaufen lassen.
    virtual void resume();

public:
    // Beginn und Ende der Kommunikation ueber einen TimeboySlot
    int setTimeboySlotState(short deviceID, int state, int commandResult = IV_ERROR_SUCCESS)
    {
        static clock_t stamp = 0;
        int res;
        if ((res = Channel4Protocol::setTimeboySlotState(deviceID, state, commandResult)) == IV_ERROR_SUCCESS)
        {
            // TimeboyIV zur Kommunikationsinitialisiserung einen Ping senden
            if ((state && ((clock() - stamp) / CLOCKS_PER_SEC > 3)) || getPacketNumber() == 0)
            {
                if ((res = ping(deviceID, NULL, 0)) == IV_ERROR_SUCCESS)
                {
                    stamp = clock();
                }
            }
        }

        // Wenn es kein TimeboySlot ist, dann trotzdem OK melden.
        if (res == -IV_ERROR_SLOT_NONE)
        {
            res = IV_ERROR_SUCCESS;
        }

        return res;
    }

public:
  int fingerprintList(short deviceID, int flags, int &recordCount);
  int fingerprintBackup(short deviceID, char *backupFile, int flags);
  int fingerprintRestore(short deviceID, char *backupFile, int flags);

#define OFF_EXTERNAL_READER 0x00000001 // External Reader options 0=GIS, 1=PHG
#define OFF_IOMODULE        0x00000002 // IOModul
#define OFF_TBADAPTION      0x00000004 // TBAdaption
#define OFF_COMSERVER       0x00000008 // ComServer
#define OFF_TPWRITE         0x00000010 // TransponderWrite
#define OFF_HTTP            0x00000020 // Http-Protokoll
#define OFF_BIOKEY_UPDATE   0x00000040 // UpdateBiokey
#define OFF_SO_IOCONTROL    0x00000080 // SetupOption: Signalverarbeitung
  int setOptionFirmware(short deviceIDRemote, int mask, int options);
  int getOptionFirmware(short deviceIDRemote, int &mask, int &options);

  int _loadAsDefault; // Nachfolgende Uebertragung der Textdaten als Default.lng
  int setTextDataWithTimeboySlot(short deviceIDRemote, char *fileName);
  int setTextData(short deviceIDRemote, char *fileName);
  int setTouchKeyDataWithTimeboySlot(short deviceIDRemote, char *fileName);
  int setTouchKeyData(short deviceIDRemote, char *fileName);
  int setDisplayColorsWithTimeboySlot(short deviceIDRemote, char *fileName);
  int setDisplayColors(short deviceIDRemote, char *fileName);

  int setWlanConfigDataWithTimeboySlot(short deviceIDRemote, char *fileName);
  int setWlanConfigData(short deviceIDRemote, char *fileName);
  int getWlanConfigDataWithTimeboySlot(short deviceIDRemote, char *fileName);
  int getWlanConfigData(short deviceIDRemote, char *fileName);

  int setHardwareConfigDataWithTimeboySlot(short deviceIDRemote, char *fileName);
  int setHardwareConfigData(short deviceIDRemote, char *fileName);
  int getHardwareConfigDataWithTimeboySlot(short deviceIDRemote, char *fileName);
  int getHardwareConfigData(short deviceIDRemote, char *fileName);

  int writeFileDataWithTimeboySlot(short deviceIDRemote, char *filePath);
  int writeFileData(short deviceIDRemote, char *filePath);
  int readFileDataWithTimeboySlot(short deviceIDRemote, char *filePath);
  int readFileData(short deviceIDRemote, char *filePath);

private:
    // Dateiarchiv fuer Geraetedaten (Firmware) verwalten
    FileDeviceZip _fileDevice;

public:
    int getTablesWithTimeboySlot( short deviceIdRemote, char *typeName );
    int getTables( short deviceIdRemote, char *typeName );

    int writeSetupWithTimeboySlot( short deviceIdRemote, char *szFileName );
    int writeSetupData( short deviceIdRemote, char *szFileName );
    int writeSetupData4( short deviceIdRemote );
    int writeSetupData4_2( short deviceIdRemote );
    
    int readSetupWithTimeboySlot( short deviceIdRemote, char *szFileName );
    int readSetupData( short deviceIdRemote, char *szFileName );
    int readSetupData4( short deviceIdRemote );
    int readSetupData4_2( short deviceIdRemote );
    
    int writeListWithTimeboySlot( short deviceIdRemote );
    int writeListData( short deviceIdRemote );
    int writeListData4( short deviceIdRemote );

    int writeEntrance2ListWithTimeboySlot( short deviceIdRemote, unsigned char listNum );
    int writeEntrance2ListData4( short deviceIdRemote, unsigned char listNum );
    int writeEntrance2List4( short deviceIdRemote, unsigned char listNum );
    
    int writeTimeboyListWithTimeboySlot( short deviceIdRemote, unsigned char id, unsigned char num, int reserve );
    int writeTimeboyList4( short deviceIdRemote, unsigned char id, unsigned char num, int reserve );
    int writeTimeboyListData4( short deviceIdRemote, unsigned char id, unsigned char num, int reserve );

    string _info; // Puffer fuer die Hardwareinfo in Textform.
    int writeDeviceWithTimeboySlot( short deviceIdRemote, char *filePath );
    int writeDevice( short deviceIdRemote, char *filePath );
    int writeDevice4( short deviceIdRemote );

#pragma pack(1)
    struct SUploadModuleSignatur 
    {
        unsigned char version;
        unsigned char type;
        unsigned short flags;
        unsigned int crc32;
    } *_uploadModuleSignatur;
#pragma pack()
    int writeModuleWithTimeboySlot(short deviceIdRemote, int type, char *szFileName);
    int writeModule(short deviceIdRemote, int type, char *szFileName);

    int getVersionFirmwareFromFile( char *fileName, char *versionFirmware );

    int writeMemToFile( short deviceIdRemote, int type, int address, int size, char *fileName );
    int writeFileToMem( short deviceIdRemote, char *fileName );

    int checkData( short deviceIdRemote );

    // Überlagerungen aus Channel4ModuleApplication
    int getRecord( short deviceIdRemote, unsigned char *buf, int &count, unsigned char &status );
    int setRecord( short deviceIdRemote, unsigned char &status );
    int clearRecords( short deviceIdRemote );
    int restoreRecords( short deviceIdRemote );
    int showInfo( short deviceIdRemote, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text );
    int showMessage( short deviceIdRemote, unsigned char time, unsigned char action, unsigned char audio, char *text );
    int setVariable( short deviceIdRemote, char *name, char *val );
    int getVariable( short deviceIdRemote, char *name, char *val );
    int SetTypeFont( short deviceIdRemote, unsigned char type );

    // Überlagerung aus Channel4ModuleSystem
    int systemReadRecord( short deviceIdRemote, unsigned char *buf, int &count, unsigned char &status );
    int systemQuitRecord( short deviceIdRemote, unsigned char &status );
    int systemRestoreRecords( short deviceIdRemote );
    int getRTC( short deviceIdRemote, char *time );
    int setRTC( short deviceIdRemote, char *time );
    int getVersion( short deviceIdRemote, char *version );
    int systemReset( short deviceIdRemote, unsigned short modus );
    int getSerial( short deviceIdRemote, unsigned int &serial );
    int setRelay( short deviceIdRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time );
    int getRelay( short deviceIdRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time );
    int setSystemVariable( short deviceIdRemote, char *name, char *val );
    int getSystemVariable( short deviceId, const char *name, char *val );
    int pressVirtualKey( short deviceIdRemote, unsigned char vk, unsigned char flags );

    // Informationen über das Gerät sammeln.
    int getDeviceInfo( short deviceId, DeviceInfo &deviceInfo );
    // Informationen über die verbaute Hardware im Gerät sammeln.
    int getHardwareInfo( short deviceId, HardwareInfo &hardwareInfo );
    // Informationen über die Enthaltene Firmware.
    int getSoftwareInfo( short deviceId, SoftwareInfo &softwareInfo );
    // Informationen über die Zusatzoptionen der Firmware.
    int getFirmwareOptions( short deviceId, FirmwareOptions &firmwareOptions );
    // Setzen der Zusatzoptionen der Firmware.
    int setFirmwareOptions( short deviceId, const FirmwareOptions &firmwareOptions );

    // Daten getunnelt über Docking schicken.
    int startTransparencyMode( unsigned char deviceAddress, unsigned short id, unsigned short flags, unsigned int sizeReadBuffer, unsigned int sizeWriteBuffer, int timeout );
    int stopTransparencyMode( unsigned char deviceAddress );
    bool isTransparencyMode();

    // Gelbe LED bei Dockingstation-V2 setzen/lesen
    int setLEDState( short deviceId, unsigned slot, unsigned char id, unsigned char state);
    int getLEDState( short deviceId, unsigned slot, unsigned char id, unsigned char &state);
};

#endif
