/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file is part of the DFComDLL module of the Datafox SDK.
**
****************************************************************************/

#ifndef CHANNEL4_BASE_H
#define CHANNEL4_BASE_H

#ifdef _MSC_VER
typedef int socklen_t;
#else
#include </usr/include/pthread.h>
#endif

#ifdef _MSC_VER

#define __func__ __FUNCTION__
#pragma warning(disable : 4996)

#include <winsock.h>
#include <cstdio>
#include <string>
#include <vector>

#ifndef STRING2
#define STRING2(x) #x
#define STRING(x) STRING2(x)
#define TODO(x) __pragma(message(__FILE__ "(" STRING(__LINE__) ") : todo : " #x))
#define FIXME(x) __pragma(message(__FILE__ "(" STRING(__LINE__) ") : fixme : " #x))
#endif

#else

#include <string>
#include <stdio.h>
#include <stdarg.h>
#include <unistd.h>
typedef int HANDLE;
#define SOCKET int
#define MAX_PATH 260
#define _MAX_PATH MAX_PATH
#define _MAX_EXT 256
#define TRUE 1
#define FALSE 0
#include <termio.h>
#include <sys/file.h>

#define INVALID_HANDLE_VALUE -1

#include <errno.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define INVALID_SOCKET       -1
#define SOCKADDR_IN struct sockaddr_in
#define SOCKADDR struct sockaddr
#define HOSTENT struct hostent
#define WSAGetLastError() h_errno
#define SOCKET_ERROR -1

#define Sleep(x) usleep( 1000 * x )
#define FAR

#ifndef STRING2
#define STRING2(x) #x
#define STRING(x) STRING2(x)
#define TODO(x)
#define FIXME(x)
#endif

#endif

/////////////////////////////
// Error's
#include "../Common/AEErrorDef.h"

// Standardwerte als Vorgabe
#define IV_DEFAULT_CHANNELID        0

// Wiederholungsanzahlen
#define IV_READ_TRIALS   3 // Maximale Anzahl Leseversuche.
#define IV_WRITE_TRIALS  3 // Maximale Anzahl Schreibversuche.

// Basisklasse fuer pure Kommunikationsaufgaben.
class Channel4Base  
{
public:
    Channel4Base( int id = IV_DEFAULT_CHANNELID );
    virtual ~Channel4Base();

public:
    // Ermitteln der zugewiesenen Kanalnummer.
    inline int getId() 
    { 
        return _id; 
    }
    // Zuweisen der Kanalnummer.
    inline int setId( int id ) 
    { 
        int old = _id;
        _id = id; 
        return old; 
    }

    // Typ des Kommunikationskanals
    enum enumType 
    { 
        Undefined, 
        Serial, 
        Socket,
        SocketServer
    };
    // Typ des Kanals ermitteln
    inline enumType getType() 
    { 
        return _type; 
    }

    // Oeffnen der Schnittstelle
    virtual int open( enumType type, const std::string &string, int val, int timeout );
    // Schliessen der Schnittstelle.
    virtual int close();

    // Zuweisen von HANDLE oder Socket ohne Schnittstelle selbst zu oeffnen.
    int attachChannel( enumType type, HANDLE val, int timeout );
    // Freigeben der anderweitig zugewiesenen Resourcen
    int detachChannel();

    // Ermitteln ob Schnittstelle geoeffnet.
    int isOpen();

    // Loeschen anstehender Daten.
    void flush();
    int flushSocket();

    // Setzen des Logdateinamen.
    inline void setTraceFilePath( const std::string &filePath )
    { 
        _traceFileName = filePath;
    }
    
    // Trace an-/ausschalten.
    inline void setTrace( int state = TRUE )
    { 
        _trace = state; 
    }

    // Tracezustand pruefen.
    inline int isTrace()
    { 
        return _trace; 
    }

    // Ermitteln der uebergebenen Parameter an Open.
    inline void getOpenParams( enumType &type, std::string &string, int &val, int &timeout ) 
    {
        type = _type;
        string = _commString;
        val = _value;
        timeout = _timeout;
    }

    // Gibt die Grundklasse fuer pure Kommunikationsaufgaben zurueck
    Channel4Base& getChannelBase() 
    { 
        return (Channel4Base&)*this;
    }

    HANDLE _comm;   // Handle serielle   Schnittstelle
    SOCKET _socket; // Handle TCP_Socket Server/Client
    std::string _commString; // Haelt letzten Kommunikationsstring
protected:
    enumType _type; // Typ der Schnittstelle
    int _value;     // Comm-val halten
    int _timeout;   // Timeout fuer Kommunikationsfunktionen halten

public:
    int changeStateSerial( HANDLE handle, int baudRate, int timeout );

protected:
    // Zugriff auf private member: _timeout (Schnittstellentimeout)
    inline void setTimeout( int timeout ) 
    { 
        _timeout = timeout; 
    }
public:
    inline int getTimeout() 
    { 
        return _timeout; 
    }
    inline int getBaudRate()
    {
        return _value;
    }

    // Oeffnen der Schnittstelle fuer RS232 / RS485 Verbindung.
    int openSerial( const std::string &comm, int baudRate, int timeout );
    int closeSerial();

    // Oeffnen der Schnittstelle fuer TCP/IP.
    int openSocket( const std::string &host, int port, int timeout );

    // Oeffnen der Schnittstelle fuer TCP/IP als Server.
    int openSocketServer( const std::string &host, int port, int timeout );

    // Beendet eine Socketverbindung, schliesst _socket.
    int closeSocket();

    // Main - Funktion fuer Befehlsausfuehrung in Workerthread
protected:
#ifdef _MSC_VER
    static unsigned long WINAPI communicationServerThread( LPVOID lpParam );
#else
    static void *communicationServerThread( void *lpParam );
#endif
    volatile int _endSignal;
    volatile int _initSignal;
    enum t_state 
    {
        Down, 
        Reset, 
        Listening, 
        Accepting,
        Connected
    };
    t_state _communicationServerThreadState;
#ifdef _MSC_VER
    HANDLE _communicationServerThread;
#else
    pthread_t _communicationServerThread;
#endif
    // Socket fuer Listensocket.
    SOCKET _listenSocket;
    struct serverThreadData
    {
        SOCKADDR_IN sin;
        SOCKADDR_IN sa;
        int timeout;
    } _serverThreadData;

    // Endesignal im Workerthread setzen und auf dessen Beendigung warten
    void closeSocketServer();

protected:
    virtual int callServer() = 0;

protected:
    // Datenempfang
    int readSerial( void *buffer, int readCount, int &readInCount, int readTrials = IV_READ_TRIALS );
    int readSocket( void *buffer, int readCount, int &readInCount, int readTrials = IV_READ_TRIALS );

public:
    virtual int readData( void *buffer, int readCount, int &readInCount, int readTrials = IV_READ_TRIALS );

protected:
    // Datenversand
    int writeSerial( const void *buffer, int writeCount, int &writeOutCount, int writeTrials = IV_WRITE_TRIALS );
    int writeSocket( const void *buffer, int writeCount, int &writeOutCount, int writeTrials = IV_WRITE_TRIALS );

public:
    virtual int writeData( const void *buffer, int writeCount, int &writeOutCount, int writeTrials = IV_WRITE_TRIALS );

protected: // Statistik
    int _writtenSize;     // Anzahl der versendeten Bytes
    int _amountSize;        // Anzahl der gelesenen Bytes

protected: // Logging
    int _trace;
    std::string _traceFileName;

public:
    virtual void trace( const char *format, ... );

protected:
    // zugewiesene Kanalnummer
    int _id;
#ifndef _MSC_VER
    struct termios _termio;
    struct flock  _flock;
#endif
};

#endif
