#ifndef _DFSOFTWAREVERSION_
#define _DFSOFTWAREVERSION_

#ifdef _MSC_VER
#pragma warning(disable : 4996)
#endif

#ifndef _MSC_VER
#define _T(x) x
#endif

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <memory.h>

// Maximale Anzahl
#define VF_MAX_DEVICES                22
#define VF_MAX_DEVICE_NAME            19

#define VF_DEVICE_NA                   0xFF
#define VF_DEVICE_PZE                  0
#define VF_DEVICE_ZK                   1
#define VF_DEVICE_BDE                  2
#define VF_DEVICE_MOBIL                3
#define VF_DEVICE_MDE                  4
#define VF_DEVICE_AE                   5
#define VF_DEVICE_TIMEBOY              6
#define VF_DEVICE_EXKLUSIV             7
#define VF_DEVICE_FLEX                 8
#define VF_DEVICE_TIMEBOY_MOBIL_PZE    9
#define VF_DEVICE_EVO_2_8             10
#define VF_DEVICE_EVO_4_3             11
#define VF_DEVICE_IPC_EXTENSION_A     12
#define VF_DEVICE_IPC_EXTENSION_B     13
#define VF_DEVICE_IPC_EXTENSION_C     14
#define VF_DEVICE_IO_BOX              15
#define VF_DEVICE_ZK_BOX              16
#define VF_DEVICE_MOBIL_BOX           17
#define VF_DEVICE_TIMEBOY_TBDOCK      18
#define VF_DEVICE_MOBIL_TBDOCK        19
#define VF_DEVICE_ZK_KNOTEN           20
#define VF_DEVICE_ZK_TERMINAL         21
#define VF_DECIVE_SAMS_OCL			  22

#define IV_DEVICE_NAME_NA                "NA"
#define VF_DEVICE_NAME_PZE               "PZE"
#define VF_DEVICE_NAME_ZK                "ZK"
#define VF_DEVICE_NAME_BDE               "BDE"
#define VF_DEVICE_NAME_MOBIL             "MOBIL"
#define VF_DEVICE_NAME_MDE               "MDE"
#define VF_DEVICE_NAME_AE                "AE"
#define VF_DEVICE_NAME_TIMEBOY           "TIMEBOY"
#define VF_DEVICE_NAME_EXKLUSIV          "EXKLUSIV"
#define VF_DEVICE_NAME_FLEX              "FLEX"
#define VF_DEVICE_NAME_TIMEBOY_MOBIL_PZE "TB-MOBIL-PZE"
#define VF_DEVICE_NAME_EVO_2_8           "Evo28"
#define VF_DEVICE_NAME_EVO_4_3           "Evo43"
#define VF_DEVICE_NAME_IPC_EXTENSION_A   "VARIO"
#define VF_DEVICE_NAME_IPC_EXTENSION_B   "IPC"
#define VF_DEVICE_NAME_IPC_EXTENSION_C   "Q7"
#define VF_DEVICE_NAME_IO_BOX            "IOBOX"
#define VF_DEVICE_NAME_ZK_BOX            "ZKBOX"
#define VF_DEVICE_NAME_MOBIL_BOX         "MOBILBOX"
#define VF_DEVICE_NAME_TIMEBOY_TBDOCK    "DockingV2"
#define VF_DEVICE_NAME_MOBIL_TBDOCK      "LoggerV2"
#define VF_DEVICE_NAME_ZK_KNOTEN         "ZKKNOTEN"
#define VF_DEVICE_NAME_ZK_TERMINAL       "ZKTERMINAL"
#define VF_DEVICE_NAME_SAMS_OCL          "SAMS_OCL"

class CDFSoftwareVersion
{
public:
    CDFSoftwareVersion();
    CDFSoftwareVersion(const char *version);
    CDFSoftwareVersion(unsigned int version);
    CDFSoftwareVersion(unsigned char master, char major, unsigned char minor, unsigned char build, unsigned char typeDevice);
    virtual ~CDFSoftwareVersion();
  friend class CDFSoftwarebuildVersion;

private:
    static const char m_typeDeviceText[VF_MAX_DEVICES][VF_MAX_DEVICE_NAME];
    char m_versionFirmware[64];
    unsigned char m_master;
    unsigned char m_major;
    unsigned char m_minor;
    unsigned char m_build;
    unsigned char m_typeDevice;
    
public:
    void Reset()
    {
        m_master = m_major = m_minor = m_build = 0;
        m_typeDevice = 0;
        memset(m_versionFirmware, 0, sizeof(m_versionFirmware));
        sprintf(m_versionFirmware, "%02i.%02i.%02i.%02i", m_master, m_major, m_minor, m_build);
    }
    int Parse(const char *version);
    int Parse(unsigned int version);

  inline unsigned char GetMaster() {
    return m_master;
  }
  inline unsigned char GetMajor() {
    return m_major;
  }
  inline unsigned char GetMinor() {
    return m_minor;
  }
  inline unsigned char GetBuild() {
    return m_build;
  }
    inline unsigned char GetTypeDevice() {
        return m_typeDevice;
    }
public: // Zuweisungsoperator
    CDFSoftwareVersion& operator= (const char *version)
    {
        if (!this->Parse(version))
        {
            Reset();
        }
        return *this;
    }
    CDFSoftwareVersion& operator= (unsigned int version)
    {
        if (!this->Parse(version))
        {
            Reset();
        }
        return *this;
    }


public: // Vergleichsoperatoren
    // Gleichheit
    bool operator== (CDFSoftwareVersion &version)
    {
        if (this->m_master == version.m_master &&
            this->m_major == version.m_major &&
            this->m_minor == version.m_minor)
        {
            return true;
        }

        return false;
    }
    bool operator== (const char *version)
    {
        CDFSoftwareVersion lVersion(version);
        return (*this == lVersion);
    }
    // Ungleichheit
    bool operator!= (CDFSoftwareVersion &version)
    {
        if (this->m_master != version.m_master ||
            this->m_major != version.m_major ||
            this->m_minor != version.m_minor)
        {
            return true;
        }

        return false;
    }
    bool operator!= (const char *version)
    {
        CDFSoftwareVersion lVersion(version);
        return (*this != lVersion);
    }
    // Groesser
    bool operator> (CDFSoftwareVersion &version)
    {
        if (this->m_master > version.m_master ||
            (this->m_master == version.m_master && this->m_major > version.m_major) ||
            (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor))
        {
            return true;
        }

        return false;
    }
    bool operator> (const char *version)
    {
        CDFSoftwareVersion lVersion(version);
        return (*this > lVersion);
    }
    // Groesser gleich
    bool operator>= (CDFSoftwareVersion &version)
    {
        if (*this > version || *this == version)
        {
            return true;
        }

        return false;
    }
    bool operator>= (const char *version)
    {
        CDFSoftwareVersion lVersion(version);
        return (*this >= lVersion);
    }
    // Kleiner
    bool operator< (CDFSoftwareVersion &version)
    {
        if (this->m_master < version.m_master ||
            (this->m_master == version.m_master && this->m_major < version.m_major) ||
            (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor))
        {
            return true;
        }

        return false;
    }
    bool operator< (const char *version)
    {
        CDFSoftwareVersion lVersion(version);
        return (*this < lVersion);
    }
    // Kleiner gleich
    bool operator<= (CDFSoftwareVersion &version)
    {
        if (*this < version || *this == version)
        {
            return true;
        }

        return false;
    }
    bool operator<= (const char *version)
    {
        CDFSoftwareVersion lVersion(version);
        return (*this <= lVersion);
    }

public: // Cast operatoren
    operator const char* ()
    {
        return m_versionFirmware;
    }
    operator unsigned int ()
    {
        return static_cast<unsigned int>( (m_master << 24) | (m_major << 16) | (m_minor << 8) | m_build );
    }
};

class CDFSoftwarebuildVersion : public CDFSoftwareVersion
{
public:
  CDFSoftwarebuildVersion() : CDFSoftwareVersion() {

  }
  CDFSoftwarebuildVersion(const char *version) : CDFSoftwareVersion(version) {

  }
  CDFSoftwarebuildVersion(unsigned int version) : CDFSoftwareVersion(version) {

  }
  CDFSoftwarebuildVersion(unsigned char master, char major, unsigned char minor, unsigned char build, unsigned char typeDevice) : CDFSoftwareVersion(master, major, minor, build, typeDevice) {
  }

public: // Vergleichsoperatoren
  // Gleichheit
  bool operator== (CDFSoftwarebuildVersion &version) {
    if (this->m_master == version.m_master &&
      this->m_major == version.m_major &&
      this->m_minor == version.m_minor &&
      this->m_build == version.m_build) {
      return true;
    }

    return false;
  }
  bool operator== (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this == lVersion);
  }
  // Ungleichheit
  bool operator!= (CDFSoftwarebuildVersion &version) {
    if (this->m_master != version.m_master ||
      this->m_major != version.m_major ||
      this->m_minor != version.m_minor ||
      this->m_build != version.m_build) {
      return true;
    }

    return false;
  }
  bool operator!= (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this != lVersion);
  }
  // Groesser
  bool operator> (CDFSoftwarebuildVersion &version) {
    if (this->m_master > version.m_master ||
      (this->m_master == version.m_master && this->m_major > version.m_major) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor > version.m_minor) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build > version.m_build)) {
      return true;
    }

    return false;
  }
  bool operator> (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this > lVersion);
  }
  // Groesser gleich
  bool operator>= (CDFSoftwarebuildVersion &version)
  {
    if (*this > version || *this == version)
    {
      return true;
    }

    return false;
  }
  bool operator>= (const char *version)
  {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this >= lVersion);
  }
  // Kleiner
  bool operator< (CDFSoftwarebuildVersion &version) {
    if (this->m_master < version.m_master ||
      (this->m_master == version.m_master && this->m_major < version.m_major) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor < version.m_minor) ||
      (this->m_master == version.m_master && this->m_major == version.m_major && this->m_minor == version.m_minor && this->m_build < version.m_build)) {
      return true;
    }

    return false;
  }
  bool operator< (const char *version) {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this < lVersion);
  }
  // Kleiner gleich
  bool operator<= (CDFSoftwarebuildVersion &version)
  {
    if (*this < version || *this == version)
    {
      return true;
    }

    return false;
  }
  bool operator<= (const char *version)
  {
    CDFSoftwarebuildVersion lVersion(version);
    return (*this <= lVersion);
  }

};
#endif
