#include "Channel4.h"
#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <array>
#include <bitset>
#include <boost/assign/list_of.hpp>
#include <cstring>

#ifdef _MSC_VER
#define stat _stat
#endif

// Versionspruefung
#include "../Common/DFComUtils.h"
#include "../../DatafoxLibraryIV/libraryversion.h"

#include "../File/filedevicetext.h"
#include "../File/filetouchkeys.h"
#include "../File/filedisplaycolors.h"
#include "../File/filewlanconfig.h"

#ifdef _MSC_VER
#define snprintf	    _snprintf
#endif

// ##################################################################
/**
* @brief Initialisiert die Membervariablen und 
*        benoetigte Umgebung.
*
* @param id Kanalnummer die fuer Log usw. verwendet wird. Sie identifiziert 
*           die Klasse.
*/
Channel4::Channel4(short channelID, short deviceID) : Channel4ModuleTable(channelID, deviceID) 
{
    trace("%s(%i):INFO(3): %s [ %i, %i ]\n", __FILE__, __LINE__, __func__, channelID, deviceID);

    /*	//#####################################################
    // SOCKETS
    // Benutzung der Socket - Funktionalitaet initialisieren
    WSADATA WindowsData;
    WORD    Version;

    Version = MAKEWORD(2, 2);
    WSAStartup(Version, &WindowsData);
    //#####################################################
    */
    // Status fuer ServiceStatus
    _serviceState = 0;

    // Workerthread Ende - Signal
    _endSignal = FALSE;

    // Sockets initialisieren
    _listenSocket  = NULL;

    // Verbindungsthread
    _communicationThread = 0;

    // Callback fuer Nachricht auf Datensaetze
    m_CBNotifyRecord = NULL;

    // Anzahl der Suspendanforderungen halten
    _suspendCounter = 0;

    // Melden von Datensaetzen aktiviert?
    m_notify = 0;

    // Hardwareinfo aufsetzen
    memset(&_hwInfo, 0, sizeof(StructHardwareInfo));
    _hwInfo.typeDevice           = IV_VALUE_TYPE_DEVICE_PZE;
    _hwInfo.typeBoard            = IV_VALUE_TYPE_BOARD_2_1;
    _hwInfo.componentRS485_0     = 0;
    _hwInfo.typeTCPIP            = IV_VALUE_TYPE_TCPIP_XPORT;
    _hwInfo.typeModemMC          = IV_VALUE_TYPE_MODEM_MC_NONE;
    _hwInfo.componentRS485_1     = 0;
    _hwInfo.typeTransponder      = IV_VALUE_TYPE_TRANSPONDER_TSR32;
    _hwInfo.typeDisplay          = IV_VALUE_TYPE_DISPLAY_SMALL;
    _hwInfo.typeKeypad           = IV_VALUE_TYPE_KEYPAD_9;
    _hwInfo.typeMMC              = IV_VALUE_TYPE_MMC_NONE;
    _hwInfo.typeGPS              = IV_VALUE_TYPE_GPS_NONE;
    _hwInfo.componentUSB         = 0;
    _hwInfo.typeRadio            = IV_VALUE_TYPE_RADIO_NONE;
    _hwInfo.typeDCF77            = IV_VALUE_TYPE_DCF77_NONE;
    _hwInfo.componentInAnalog    = IV_VALUE_COMPONENT_IN_ANALOG_NONE;
    _hwInfo.componentInDigital   = IV_VALUE_COMPONENT_IN_DIGITAL_1;
    _hwInfo.componentOutDigital  = IV_VALUE_COMPONENT_OUT_DIGITAL_1;
    _hwInfo.typeFingerprint      = IV_VALUE_TYPE_FINGERPRINT_NONE;
    _hwInfo.typeExternalReader   = IV_VALUE_TYPE_EXTERNAL_READER_UNDEFINED;
    _hwInfo.optionIOModule       = 0;
    _hwInfo.optionTBAdaption     = 0;

    // Textdaten als Default uebertragen
    _loadAsDefault = 0;
}

// ##################################################################
/**
* @brief Destruktor bereinigt die verwendete Umgebung.
*/
Channel4::~Channel4()
{
    //#####################################################
    // SOCKETS

    // Socket - Funktionalitaet exit
    //	WSACleanup();
    //#####################################################
}

// ##################################################################
/**
* @brief Entgegennehmen der Verbindung und starten eines Verarbeitungsthreads.
*/
int Channel4::connect(SOCKET *listenSocket, short channelID, short deviceID, int timeout, int aliveTO, int infoFlags) 
{
    trace("%s(%i):INFO(3): %s [ 0x08X, %i, %i, %i, %i ]\n", __FILE__, __LINE__, __func__, listenSocket, channelID, deviceID, timeout, aliveTO);

    // Hauptthread starten
    if (_communicationThread == 0)
    {	
        // Kanalobjekt nimmt Verbindung entgegen
        setActiveState(is_accepting);
        trace("%s(%i):INFO(1): %s [ SetActiveState(%i) is_accepting ]\n", __FILE__, __LINE__, __func__, channelID);

        // Kanalnummer und DeviceID uebernehmen.
        setId(channelID);
        setDeviceId(deviceID);

        // Listensocket von dem die Verbindungsanforderung kommt wird 
        // ueber lokale Membervariable uebergeben.
        _listenSocket = listenSocket;
        if (timeout < 3000)
        {
            trace("%s(%i):WARNING(1): %s(%i) [ Timeout von %i auf mindestens 3000ms gesetzt. ]\n", __FILE__, __LINE__, __func__, channelID, timeout);
            timeout = 3000;
        }
        setTimeout(timeout);
        _aliveTO = aliveTO;
        _infoFlags = infoFlags;
        _initSignal = -1; // Undefinierter Fehler
#ifdef _MSC_VER
        _communicationThread = ::CreateThread(NULL, 0, communicationThread, this, 0, &_threadID);
#else
        pthread_create(&_communicationThread, NULL, communicationThread, this);
#endif

        int i;
        // Hochlauf des Workers pruefen
        for (i=0; (_communicationThread != 0) && (_initSignal == -1) && (i < ((timeout*6) / 100)); i++)
        {
            Sleep(100);
        }
        // Wurde ein Fehler ausgeloest.
        if (_initSignal != IV_ERROR_SUCCESS)
        {
            // Grundlegender Initialiserungsfehler?
            if (_initSignal == -1)
            {

            } 
            else 
            {

                // Fehler von Workerthread 1:1 weitergeben.
            }

            // Verbindung schliessen
            directClose();
            return 0;
        }
    }

    return (_communicationThread != 0);
}

// ##################################################################
/**
* @brief Beendigungssignal im Workerthread setzen und auf dessen beendigung warten.
*/
void Channel4::directClose()
{
    trace("%s(%i):INFO(3): %s", __FILE__, __LINE__, __func__);

    // Socket-Thread deaktivieren, wenn aktiv
    if (_communicationThread != 0)
    {
        // WorkerThread fuer Socket beenden.
        _endSignal = TRUE;

        // Thread umschalten
        Sleep(10);

        // Thread Beendigung abwarten (communicationThread stellt _endSignal auf FALSE)
        int i;
        for (i=0; i < (getTimeout()*6 / 100) && _endSignal; i++)
        {				
            Sleep(100);
        }

        // Hartes Schliessen, wenn noch nicht selbst beendet.
        if (_communicationThread)
        {
#ifdef _MSC_VER
            ::CloseHandle(_communicationThread);
#else
            pthread_exit(&_communicationThread);
#endif
            _communicationThread = 0;
        }
    }
}

// ##################################################################
/**
* @brief Gewünschte Info über die Active Verbindung erstellen.
*/
bool Channel4::getInfoActiveChannel( char *infoString, int *infoStringLength )
{
    trace("%s(%i):INFO(3): %s [ 0x%p, 0x%p ]\n", __FILE__, __LINE__, __func__, infoString, infoStringLength );

    if ( getActiveState() == is_active )
    {
        char info[255];
        int infoLength = 0;
        int infoLengthMax = 0;
        memset(info, 0, sizeof(info));
        std::bitset<32> flags( _infoFlags );

        for ( int paramNumber = 0; infoLength != -1 && infoLength < *infoStringLength; ++paramNumber )
        {
            infoLengthMax = infoLength;

            if ( paramNumber == 0 ) // Gerätetyp
            {
                infoLength = snprintf( info, sizeof(info), "%u\r", _hwInfo.typeDevice );
            }
            else if ( paramNumber == 1 ) // Seriennummer
            {
                infoLength += snprintf( &info[infoLength], sizeof(info) - infoLength, "%u\r", (unsigned int)_serial );
            }
            else if ( paramNumber == 2 ) // Firmwareversion optional
            {
                // Wenn keine Flags gesetzt wurden, dann Ende.
                if ( !flags.count() )
                    break;

                if ( flags.test( 0 ) )
                    infoLength += snprintf( &info[infoLength], sizeof(info) - infoLength, "%s\r", _versionFirmware );
                else
                    infoLength += snprintf( &info[infoLength], sizeof(info) - infoLength, "\r" );
            }
            else if ( paramNumber == 3 ) // TCPIP Adresse und Port optional
            {
                // Wenn keine weiteren Flags gesetzt wurden, dann Ende.
                if ( !flags.test( 1 ) && !flags.test( 2 ) )
                    break;

                if ( flags.test( 1 ) )
                    infoLength += snprintf( &info[infoLength], sizeof(info) - infoLength, "%s:%i\r", m_tcpipInfo.ip, m_tcpipInfo.port );
                else
                    infoLength += snprintf( &info[infoLength], sizeof(info) - infoLength, "\r" );
            }
            else if ( paramNumber == 4 ) // Mac optional
            {
                if ( flags.test( 2 ) )
                    infoLength += snprintf( &info[infoLength], sizeof(info) - infoLength, "%s\r", m_tcpipInfo.mac );
            }
            else
                break;
        }
        // Pruefen ob die Werte kopiert werden koennen; die Puffergroesse muss passen!
        if ( infoLengthMax < *infoStringLength )
        {
            snprintf( infoString, infoLengthMax, "%s", info );
            infoString[infoLengthMax] = 0;
            *infoStringLength = infoLengthMax;
            return true;
        }
        else
        {
            *infoStringLength = 0;
        }
    }

    return false;
}

/*
int Channel4::CallFunctionNotify(short module, short func, unsigned char *data, int &count)
{
Trace("%s(%i):INFO(3): %s [ %i, %i, 0x08X, %i ]\n", __FILE__, __LINE__, __func__, module, func, data, count);

switch(module)
{
case IV_MODULE_APPLICATION:
switch (func)
{
case IV_FUNCTION_NOTIFY:
{
switch(data[0])
{
case IV_VALUE_NOTIFY_RECORD:
{
CNotifyRecord nr;

if (m_CBNotifyRecord)
{				
int pos = 0;

// Meldungstypen uebernehmen
nr.type = data[pos];
pos += sizeof(unsigned char);

// Meldungsprioritaet uebernehmen
nr.priority = *(short*)&data[pos];
pos += sizeof(unsigned short);

// Zeitstempel der Meldungserzeugung uebernehmen
pos += sprintf(nr.timestamp, "%s", (char*)&data[pos]);

// Aktuellen Zeitstempel fuer Nachrichtenannahme halten
nr.dt = time(NULL);

// Meldung in Warteschlange stellen.
m_CBNotifyRecord(nr);

Trace("%s(%i):INFO(1): %s [ %i, %i, %s ]\n", __FILE__, __LINE__, __func__, GetID(), nr.type, nr.timestamp);
}
break;
}

default:
Trace("%s(%i):INFO(1): %s [ Unkown type %i ]\n", __FILE__, __LINE__, __func__, data[0]);
break;
}


break;
}
}
break;
}

return IV_ERROR_SUCCESS;
}
*/
// ##################################################################
/**
* @brief Workerthread fuer die Verarbeitung der Kommunikationsaufgaben.
*/
#ifdef _MSC_VER
unsigned long WINAPI Channel4::communicationThread(LPVOID lpParam)
#else
void *Channel4::communicationThread(void *lpParam)
#endif
{
    if (lpParam == NULL)
    {	
        // Fehlerhafter Parameter
        return 0;
    }

    // Es wird davon ausgegangen, dass der Parameter ein Channel4 - Objekt ist.
    Channel4 &own = *(Channel4*)lpParam;

    own.trace("%s(%i):INFO(3): %s [ %p ]\n", __FILE__, __LINE__, __func__, lpParam);

#ifdef _MSC_VER
#ifndef MYCONST
#define MYCONST const
#endif
#else
#define MYCONST
#endif

    // Hauptschleife ausfuehren?
    int run = TRUE;
    own._threadCommand = commandIdle;

    int res, failCount = 0, resumeCount = 0;
    SOCKADDR_IN sa;
    struct timeval seltime;
    fd_set readfds;
    int count;
    unsigned char data[IV_PACKET_LENGTH_DATA]; // 21 Byte Paketframe

    int len;

    // Timeout fuer Abfrage auf Daten
    seltime.tv_sec  = 0;
    seltime.tv_usec = 10000;

    // Verbindungsgesuch annehmen.

    len = sizeof(sa);
    own._socket = accept(*own._listenSocket, (SOCKADDR *)&sa, (socklen_t*)&len);

    if(own._socket == INVALID_SOCKET)
    {
        own.trace("%s(%i):ERROR(1): %s [ accept fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
        // Initialisierungsfehler setzen.
        own._initSignal = IV_ERROR_FAIL;
        // Fehlerfall abfangen
        goto fail;
    }

    // Verbindungsmeldung ausgeben
    own.trace("%s(%i):INFO(1): %s [ (%i) Connected to %s, port %d ]\n", __FILE__, __LINE__, __func__, own.getId(), inet_ntoa(sa.sin_addr), htons(sa.sin_port));

    // Mindesttimeout sicherstellen
    if (own.getTimeout() < 3000)
    {
        own.setTimeout(3000);
    }

    // Socket mit Basisklasse verbinden
    //	own.AttachChannel(Channel4Base::Socket, (void*)own.m_communicationSocket, own.GetTimeout());
    own._type = Channel4Base::Socket;
    //m_socket = (SOCKET)val;
    //own.m_to     = own.GetTimeout();

    // Grunddaten zur Verbindung abfragen
    for (failCount=0; failCount < 4; failCount++)
    {
        // Paketabarbeitung einleiten
        if ((res = own.ping(254, 0, 0)) != IV_ERROR_SUCCESS)
        {
            if ((res == -IV_ERROR_CHANNEL_OPEN_REMOTE) || (failCount > 2))
            {				
                // Wenn Daten gemeldet werden,
                // die Anzahl aber 0 ist, so wurde die Verbindung beendet.
#ifdef _MSC_VER
                // Verbindung schliessen
                if (closesocket(own._socket) == SOCKET_ERROR)
                {					
                    own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                }
#else
                if (shutdown(own._socket, 2) == SOCKET_ERROR)
                {
                    own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                }
                if (::close(own._socket) == SOCKET_ERROR)
                {
                    own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                }
#endif

                own._type = Channel4Base::Undefined;
                own._socket = INVALID_SOCKET;
                own.trace("%s(%i):INFO(1): %s [ Disconnect (%i) ]\n", __FILE__, __LINE__, __func__, own.getId());

                // Hauptschleife verlassen
                run = FALSE;
                // Initialisierungsfehler setzen.
                own._initSignal = IV_ERROR_FAIL;

                // Abbruchmarke anspringen
                goto fail;
            }
        }
        else
        {
            // Verbindung OK...
            break;
        }
    }

    // Nur wenn Geraet antwortet, weitere Auswertungen.
    if (run && !own._endSignal)
    {
        // com.                  Praefix fuer Kommunikationsvariablen
        // active                Aktivieren der Aktiven Verbindung
        // notify                notify => Aktivieren der Datensatzmeldung
        // prio		             Prioritaet des Geraetes
        // host                  Hostname des Remoterechners
        // port                  Portnummer des Remoterechners
        // retry                 Anzahl Versuche fuer Verbindungsaufbau
        // timeout               Auszeit zwischen den Versuchsbloecken
        // repeat                Auszeit zur wiederholten Datensatzmeldung, wenn diese nicht abgerufen wurden.
        // alive                 Pinganfrage wenn fuer diese Zeit keine Kommunikation zustande kam.
        // Alle Zeitangaben in Sekunden und 32bit

        char val[32], notify[32], status[IV_PACKET_LENGTH_DATA];
        memset(val, 0, sizeof(val));
        memset(status, 0, sizeof(status));
        sprintf(val, "%i", 3 + (int)ceil((double)own._aliveTO * 1.05)); // 5% fuer Synkronisation dazugeben
        // Aliveparameter berechnen und setzen.
        // Geraetetyp und Seriennummer als GeraeteID abfragen
        if ((res = own.setSystemVariable(254, (char*)"COM.ALIVE", val)) != IV_ERROR_SUCCESS ||
            (res = own.getTypeDevice(254, own._hwInfo.typeDevice)) != IV_ERROR_SUCCESS || 
            (res = own.getSerial(254, own._serial)) != IV_ERROR_SUCCESS ||
            (res = own.getSystemVariable(254, (char*)"COM.NOTIFY", notify)) != IV_ERROR_SUCCESS ||
            (res = own.getVersion(254, own._versionFirmware)) != IV_ERROR_SUCCESS ||
            (own.isTrace() && (res = own.getSystemVariable(254, (char*)"SYSTEM.STATUS", status)) != IV_ERROR_SUCCESS) )
        {
#ifdef _MSC_VER
            // Verbindung schliessen
            if (closesocket(own._socket) == SOCKET_ERROR)
            {		
                own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
            }
#else
            if (shutdown(own._socket, 2) == SOCKET_ERROR)
            {
                own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
            }
            if (::close(own._socket) == SOCKET_ERROR)
            {
                own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
            }
#endif
            own._type = Channel4Base::Undefined;
            own._socket = INVALID_SOCKET;
            own.trace("%s(%i):INFO(1): %s [ Disconnect (%i) ]\n", __FILE__, __LINE__, __func__, own.getId());

            // Hauptschleife verlassen
            run = FALSE;
            // Initialisierungsfehler setzen.
            own._initSignal = IV_ERROR_FAIL;

            // Abbruchmarke anspringen
            goto fail;
        }
        else 
        {
            // Melden von Datensaetzen
            own.m_notify = atoi(notify);

            // Bereits ermittelte Info ausgeben
            own.trace("%s(%i):INFO(1): %s [ getTypeDevice( %i ), getSerial( %u ) ]\n", __FILE__, __LINE__, __func__, own._hwInfo.typeDevice, own._serial);
            // Systemstatus auswerten
            if (own.isTrace() && strlen(status))
            {
                char *token = strtok(status, "\r");
                while (token != NULL)
                {
                    own.trace("%s(%i):INFO(1): %s [ SystemStatusLine( %s ) ]\n", __FILE__, __LINE__, __func__, token);
                    token = strtok(NULL, "\r");
                }
            }

            // Daten fuer das TCPIP-Modul ermitteln
            own.m_tcpipInfo.Reset();

            // Informationen zum TCP/IP abrufen
            // Erst nachtraeglich hinzugefuegt, deswegen separat ausfuehren.
            if ((res = own.getInfo(254, "COM.TCPIP", "", own.m_tcpipInfo.data, own.m_tcpipInfo.length)) != IV_ERROR_SUCCESS)
            {				
                // Nur der Fehler "Majorname unbekannt" ist hier gueltig. Dann wird der Befehl noch nicht unterstuetzt.
                if ((res & 0xFF) != 1)
                {					
                    // Wenn Daten gemeldet werden,
                    // die Anzahl aber 0 ist, so wurde die Verbindung beendet.

#ifdef _MSC_VER
                    // Verbindung schliessen
                    if (closesocket(own._socket) == SOCKET_ERROR)
                    {						
                        own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
#else
                    if (shutdown(own._socket, 2) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
                    if (::close(own._socket) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
#endif

                    own._type = Channel4Base::Undefined;
                    own._socket = INVALID_SOCKET;
                    own.trace("%s(%i):INFO(1): %s [ Disconnect (%i) ]\n", __FILE__, __LINE__, __func__, own.getId());

                    // Hauptschleife verlassen
                    run = FALSE;
                    // Initialisierungsfehler setzen.
                    own._initSignal = IV_ERROR_FAIL;

                    // Abbruchmarke anspringen
                    goto fail;
                }
            }
            else
            {
                // Werte aufschluesseln
                own.m_tcpipInfo.Parse();

                // Bereits ermittelte Info ausgeben
                own.trace("%s(%i):INFO(1): %s [ GetInfoTCPIP( %i ), ModulVersion( %s ), IP( %s ), Port( %i ), MAC( %s ) ]\n", __FILE__, __LINE__, __func__, own.getId(), own.m_tcpipInfo.version, own.m_tcpipInfo.ip, own.m_tcpipInfo.port, own.m_tcpipInfo.mac);
            }

            // Gewuenschte Zusatzinfos abrufen
            if (own._infoFlags & 0x01)
            {

            }
        }

        // Ermittelbare Daten zur Aktiven Verbindung erfolgreich ermittelt.
        own.setActiveState(is_usable);
        own.trace("%s(%i):INFO(1): %s [ SetActiveState(%i) is_usable ]\n", __FILE__, __LINE__, __func__, own.getId());
        // Initialisierung erfolgreich.
        own._initSignal = IV_ERROR_SUCCESS;
    }

    // Fehlerzaehler neu initialisieren
    failCount = 0;

    // Hauptschleife des Worker
    while (run)
    {
        // Beendigungssignal abfragen
        if (own._endSignal)
        {
            // Verbindung herunterfahren
            if (own._socket != INVALID_SOCKET)
            {
                // Senden am Socket verhindern
                if (shutdown(own._socket, 0x01/*SD_SEND*/) == SOCKET_ERROR)
                {
                    own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                }
                else 
                {
                    // Alle anstehenden Daten lesen
                    do {

                        FD_ZERO((fd_set FAR *) &readfds);
                        FD_SET(own._socket, (fd_set FAR *) &readfds);

                        if ((res = select(FD_SETSIZE,
                            (fd_set FAR *)&readfds, (fd_set FAR *) 0, (fd_set FAR *) 0,
                            (MYCONST struct timeval FAR *)&seltime)) == SOCKET_ERROR)
                        {
                            own.trace("%s(%i):ERROR(1): %s [ select fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                        }
                        else if (res)
                        {
                            // Anstehende Daten lesen
                            if ((res = recv(own._socket, (char*)own._packetRead, sizeof(own._packetRead), 0 )) == SOCKET_ERROR)
                            {
                                own.trace("%s(%i):ERROR(1): %s [ recv fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());							
                            }
                            else if (res == 0)
                            {
                                // Disconnect von Gegenstelle
                            }
                        }
                    } while((res > 0) && (res != SOCKET_ERROR));

#ifdef _MSC_VER
                    // Verbindung schliessen
                    if (closesocket(own._socket) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
#else
                    if (shutdown(own._socket, 2) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
                    if (::close(own._socket) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
#endif

                }

                own._type = Channel4Base::Undefined;
                own._socket = INVALID_SOCKET;
            }

            // Hauptschleife verlassen
            run = FALSE;
            continue;
        }


        switch (own._threadCommand)
        {
        case commandIdle:
            {	
                // Socketdaten initialisieren
                FD_ZERO((fd_set FAR *) &readfds);
                FD_SET(own._socket, (fd_set FAR *) &readfds);

                // Auf eintreffende Daten warten
                if ((res = select(FD_SETSIZE,
                    (fd_set FAR *)&readfds, (fd_set FAR *) 0, (fd_set FAR *) 0,
                    (MYCONST struct timeval FAR *)&seltime)) != SOCKET_ERROR)
                {
                    // Stehen Daten an? (ansonsten Timeout)
                    if (res)
                    {					
                        // Paketabarbeitung einleiten
                        if ((res = own.call((short)IV_CONTEXT_SLAVE, (short)IV_MODULE_BASE, (short)IV_FUNCTION_BASE, data, count, 1)) != IV_ERROR_SUCCESS)
                        {						
                            if ((res == -IV_ERROR_CHANNEL_OPEN_REMOTE) || (failCount > 2))
                            {							
                                // Wenn Daten gemeldet werden,
                                // die Anzahl aber 0 ist, so wurde die Verbindung beendet.
#ifdef _MSC_VER
                                // Verbindung schliessen
                                if (closesocket(own._socket) == SOCKET_ERROR)
                                {								
                                    own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                                }
#else
                                if (shutdown(own._socket, 2) == SOCKET_ERROR)
                                {
                                    own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                                }
                                if (::close(own._socket) == SOCKET_ERROR)
                                {
                                    own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                                }
#endif

                                own._type = Channel4Base::Undefined;
                                own._socket = INVALID_SOCKET;
                                own.trace("%s(%i):INFO(1): %s [ Disconnect ]\n", __FILE__, __LINE__, __func__, own.getId(), inet_ntoa(sa.sin_addr), htons(sa.sin_port));

                                // Hauptschleife verlassen
                                run = FALSE;
                                continue;
                            }

                            // Fehlerzaehler erhoehen
                            failCount++;
                        }
                        else
                        {						
                            // Fehlerzaehler zuruecksetzen
                            failCount = 0;
                        }
                    }
                }
                else
                {				
                    own.trace("%s(%i):ERROR(1): %s [ select fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());

#ifdef _MSC_VER
                    // Verbindung schliessen
                    if (closesocket(own._socket) == SOCKET_ERROR)
                    {					
                        own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
#else
                    if (shutdown(own._socket, 2) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
                    if (::close(own._socket) == SOCKET_ERROR)
                    {
                        own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                    }
#endif

                    own._type = Channel4Base::Undefined;
                    own._socket = INVALID_SOCKET;
                    own.trace("%s(%i):INFO(1): %s [ Disconnect ]\n", __FILE__, __LINE__, __func__, own.getId(), inet_ntoa(sa.sin_addr), htons(sa.sin_port));

                    // Hauptschleife verlassen
                    run = FALSE;
                    continue;
                }

                // Aliveping senden?
                //double tmRP = (double)own.m_clockReceivePacket / (double)CLOCKS_PER_SEC,
                //	now = (double)clock() / (double)CLOCKS_PER_SEC;
                //if ((tmRP + own._aliveTO) < now) {
                if ((own._timeReceivePacket + own._aliveTO) < time(NULL))
                {
                    // Paketabarbeitung einleiten
                    if ((res = own.ping(254, 0, 0)) != IV_ERROR_SUCCESS)
                    {					
                        if ((res == -IV_ERROR_CHANNEL_OPEN_REMOTE) || (failCount > 2))
                        {						
                            // Wenn Daten gemeldet werden,
                            // die Anzahl aber 0 ist, so wurde die Verbindung beendet.
#ifdef _MSC_VER
                            // Verbindung schliessen
                            if (closesocket(own._socket) == SOCKET_ERROR)
                            {							
                                own.trace("%s(%i):ERROR(1): %s [ closesocket fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                            }
#else
                            if (shutdown(own._socket, 2) == SOCKET_ERROR)
                            {
                                own.trace("%s(%i):ERROR(1): %s [ shutdown fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                            }
                            if (::close(own._socket) == SOCKET_ERROR)
                            {
                                own.trace("%s(%i):ERROR(1): %s [ close fail (%i) ]\n", __FILE__, __LINE__, __func__, WSAGetLastError());
                            }
#endif
                            own._type = Channel4Base::Undefined;
                            own._socket = INVALID_SOCKET;
                            own.trace("%s(%i):INFO(1): %s [ Disconnect (%i) ]\n", __FILE__, __LINE__, __func__, own.getId());

                            // Hauptschleife verlassen
                            run = FALSE;
                            continue;
                        }

                        // Fehlerzaehler erhoehen
                        failCount++;
                    }
                    else
                    {					
                        // Fehlerzaehler zuruecksetzen
                        failCount = 0;
                    }

                    //own.Trace("%s(%i):INFO(1): %s [ Alive - Ping (%i) %is ]\n", __FILE__, __LINE__, __func__, own.GetID(), int(now - tmRP));
                    own.trace("%s(%i):INFO(1): %s [ Alive - Ping (%i) %is ]\n", __FILE__, __LINE__, __func__, own.getId(), int(time(NULL) - own._timeReceivePacket));
                }
                break;}

        case commandSuspending:
            own._threadCommand = commandSuspend;
            break;

        case commandSuspend: 
            // Kurze Auszeit.
            Sleep(10);
            break;

        case commandResuming:
            resumeCount = 0;
            own._threadCommand = commandResume;
            break;

        case commandResume:

            resumeCount++;

            if (resumeCount >= 10)
            {
                // Sicherstellen, dass keine Anforderung auf Suspending besteht.
                if (own._suspendCounter > 0)
                {
                    own._threadCommand = commandSuspend;
                }
                else
                {				
                    own._threadCommand = commandIdle;
                }
            }
            Sleep(30);
            break;
        }

        // Kurze Ausszeit um die Systemlast zu reduzieren.
        Sleep(10);
    }

fail:
    // Eventuellen Eintrag aus Benachrichtungsschlange entfernen.
    if (own.m_CBNotifyRecord)
    {
        CNotifyRecord nr;

        // Meldungen dieses Kanals loeschen
        nr.erase = 1;

        // Aktuelle Kanalnummer eintragen
        nr.channelID = own.getId();

        // Meldung aus Warteschlange entfernen.
        own.m_CBNotifyRecord(nr);
    }

    // Ggf. Bereinigen
    if ( own.getType() != Undefined )
    {
        // connect-Socket entfernen
        own.detachChannel();
    }

    // Thread wird beendet
    own._communicationThread = 0;
    // Thread Beendigung ruecksetzen
    own._endSignal = FALSE;

    // Kanalobjekt wieder frei
    own.setActiveState(is_free);
    own.trace("%s(%i):INFO(1): %s [ SetActiveState(%i) is_free ]\n", __FILE__, __LINE__, __func__, own.getId());

#ifndef _MSC_VER
    pthread_exit(NULL);
#endif
    return 0;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
* @param deviceIDRemote Anzusprechende Geraetenummer.
*/
int Channel4::call(short context, short module, short func, unsigned char *buf, int &count, int contextValue)
// Modulbefehl senden und Antwort zurueckgeben
//  deviceIDRemote -> Adresse (Geraetenummer) des Empfaengers
//  module   -> Nummer des anzusprechenden Modules
//  func -> Nummer der Funktion im Module
//	buf	     -> Buffer fuer Funktionsparameter.
//	count    -> Anzahl der Funktionsparameter in Byte. Rueckgabewert bei Parameterempfang.
//
//  return -> <0 -> Kommunikationsfehler
//             0 -> senden und empfangen und ausfuehren erfolgreich
//			  >0 -> module-/Funktionsfehler
{
    trace("%s(%i):INFO(3): %s [ %i, %i, %i, %p, %i, %i ]\n", __FILE__, __LINE__, __func__, context, module, func, buf, count, contextValue);

    int i;

    // Laeuft mit Kommunikationsworkerthread?
#ifdef _MSC_VER
    if (_communicationThread && _threadID != GetCurrentThreadId())
    {
#else
    if (_communicationThread && _communicationThread != pthread_self())
    {
#endif
        // Verschachtelung zulassen.
        ////////////////////////////
        // Achtung: Diese Routine kann durch Aufruf unter bestimmten Umstaenden nochmals aufgerufen werden.
        // Z. B. Aktive Antwort oder ein Befehl des Geraetes kommt dazwischen.
        // Es muss mit einem Levelcounter das fruehzeitiges Freischalten des Communikationsworkers verhindert werden.
        if (_suspendCounter == 0)
        {		
            _threadCommand = commandSuspending;
            trace("%s(%i):INFO(1): %s [ suspending ]\n", __FILE__, __LINE__, __func__);

            for (i=0; i < ((getTimeout() + 1000)/10) && _threadCommand != commandSuspend; i++)
            {
                Sleep(10);
            }

            if (i == ((getTimeout() + 1000)/10))
            {
                trace("%s(%i):ERROR(1): %s [ Befehl nicht ausgefuehrt ]\n", __FILE__, __LINE__, __func__);
                _threadCommand = commandIdle;
                return -IV_ERROR_FAIL;
            }
        }
        _suspendCounter++;

        i = Channel4ModuleSystem::call(context, module, func, buf, count, contextValue);
        // Verschachtelung zuruecknehmen
        _suspendCounter--;
        if (_suspendCounter == 0)
        {		
            _threadCommand = commandResuming;
            trace("%s(%i):INFO(1): %s [ resuming ]\n", __FILE__, __LINE__, __func__);
        }

        return i;
    }

    return Channel4ModuleSystem::call(context, module, func, buf, count, contextValue);
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
*/
int Channel4::suspend()
{
    trace("%s(%i):INFO(3): %s", __FILE__, __LINE__, __func__);

    int i;

    // Laeuft mit Kommunikationsworkerthread?
#ifdef _MSC_VER
    if (_communicationThread && _threadID != GetCurrentThreadId())
    {
#else
    if (_communicationThread && _communicationThread != pthread_self())
    {
#endif
        // Verschachtelung zulassen.
        ////////////////////////////
        // Achtung: Diese Routine kann durch Aufruf unter bestimmten Umstaenden nochmals aufgerufen werden.
        // Z. B. Aktive Antwort oder ein Befehl des Geraetes kommt dazwischen.
        // Es muss mit einem Levelcounter das fruehzeitiges Freischalten des Communikationsworkers verhindert werden.
        if (_suspendCounter == 0)
        {		
            _threadCommand = commandSuspending;
            trace("%s(%i):INFO(1): %s [ suspending ]\n", __FILE__, __LINE__, __func__);

            for (i=0; i < ((getTimeout() + 1000)/10) && _threadCommand != commandSuspend; i++)
            {
                Sleep(10);
            }

            if (i == ((getTimeout() + 1000)/10))
            {
                trace("%s(%i):ERROR(1): %s [ Befehl nicht ausgefuehrt ]\n", __FILE__, __LINE__, __func__);
                _threadCommand = commandIdle;
                return -IV_ERROR_FAIL;
            }
        }
        _suspendCounter++;
    }

    return IV_ERROR_SUCCESS;
}

// ##################################################################
/**
* @brief Beschreibung fehlt noch.
*
*/
void Channel4::resume()
{
    trace("%s(%i):INFO(3): %s", __FILE__, __LINE__, __func__);

    // Laeuft mit Kommunikationsworkerthread?
#ifdef _MSC_VER
    if (_communicationThread && _threadID != GetCurrentThreadId())
    {
#else
    if (_communicationThread && _communicationThread != pthread_self())
    {
#endif
        // Verschachtelung zulassen.
        ////////////////////////////
        // Achtung: Diese Routine kann durch Aufruf unter bestimmten Umstaenden nochmals aufgerufen werden.
        // Z. B. Aktive Antwort oder ein Befehl des Geraetes kommt dazwischen.
        // Es muss mit einem Levelcounter das fruehzeitiges Freischalten des Communikationsworkers verhindert werden.

        if (_suspendCounter > 0)
        {		
            // Verschachtelung zuruecknehmen
            _suspendCounter--;
            if (_suspendCounter == 0)
            {		
                _threadCommand = commandResuming;
                trace("%s(%i):INFO(1): %s [ resuming ]\n", __FILE__, __LINE__, __func__);
            }
        }
    }
}

int Channel4::fingerprintList(short deviceIDRemote, int flags, int &recordCount)
{
    int res;
    unsigned char val = 0;
    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res = getTypeDevice(deviceIDRemote, typeDevice)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Boardversion lesen.
    if ((res = getTypeBoard(deviceIDRemote, typeBoard)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    bool evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Pruefen ob das Modul verfuegbar ist.
        if ((res = getTypeFingerprint(deviceIDRemote, val)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        if (!val)
        {
            return -IV_ERROR_NO_MODULE;
        }
    }

    if ((res = tableGenerate(deviceIDRemote, (char*)"FINGERPRINT.IDLIST", (char*)"")) != IV_ERROR_SUCCESS)
    {
        // Liegen im Modul keine Daten vor?
        if (res == 133)
        {
            return -IV_ERROR_NO_DATA_AVAILABLE;
        }
    }
    // Flag fuer anschliessende Ermittlung der Datensatzanzahl gesetzt?
    if (flags & 0x01)
    {
        int handle;
        recordCount = -1;
        // Erstellte Tabelle oeffnen.
        if ((res = tableOpen(deviceIDRemote, (char*)"FINGERPRINT.IDLIST", 0, handle, (char*)"")) != IV_ERROR_SUCCESS)
        {
            return res;
        }

        // Anzahl enthaltener Datensaetze abrufen.
        if ((res = tableGetRecordCount(deviceIDRemote, handle, recordCount)) != IV_ERROR_SUCCESS)
        {
            return res;
        }

        // Tabelle schliessen.
        if ((res = tableClose(deviceIDRemote, handle)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
    }

    return res;
}

// Backup der Fingerprints erstellen
int Channel4::fingerprintBackup(short deviceID, char *backupFile, int flags)
{
    (void)flags;
    int res = IV_ERROR_SUCCESS;
    unsigned char val = 0;
    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res = getTypeDevice(deviceID, typeDevice)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Boardversion lesen.
    if ((res = getTypeBoard(deviceID, typeBoard)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    bool evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Pruefen ob das Modul verfuegbar ist.
        if ((res = getTypeFingerprint(deviceID, val)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        if (!val)
        {
            return -IV_ERROR_NO_MODULE;
        }
    }

    // Tabelle mit Fingerprintdaten erzeugen
    if ((res = tableGenerate(deviceID, (char*)"FINGERPRINT.IDLIST", (char*)"")) != IV_ERROR_SUCCESS)
    {
        // Liegen im Modul keine Daten vor?
        if (res == 133)
        {
            res = -IV_ERROR_NO_DATA_AVAILABLE;
        }

        return res;
    }
    int hTable;
    // Tabelle mit Fingerprintdaten oeffnen
    if ((res = tableOpen(deviceID, (char*)"FINGERPRINT.IDLIST", 0, hTable, (char*)"")) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    int recordCount;
    // Pruefen ob Datensaetze vorliegen
    if ((res = tableGetRecordCount(deviceID, hTable, recordCount)) != IV_ERROR_SUCCESS)
    {
        // Modul oder Funktionsfehler?
        if (res > 0) tableClose(deviceID, hTable);
        return res;
    }
    if (recordCount == 0)
    {
        // Es liegen keine Daten vor
        return -IV_ERROR_NO_DATA_AVAILABLE;
    }
    FILE *fp;
    StructFingertemplateId Id;
    unsigned char type = 2; // Idencom-Standard zuerst verwenden.
    unsigned char data[IV_PACKET_LENGTH_DATA];
    int size;
    unsigned short crc;

    // Datei oeffnen
    if ((fp = fopen(backupFile, "wb")) == NULL)
    {
        tableClose(deviceID, hTable);
        return -IV_ERROR_FILE_OPEN;
    }

    // Kopfstruktur vorbereiten
    StructFTFileHeader structHeader;
    memset(&structHeader, 0, sizeof(StructFTFileHeader));
    // Kopfdaten reservieren
    if (fwrite(&structHeader, 1, sizeof(StructFTFileHeader), fp) < sizeof(StructFTFileHeader))
    {
        // Daten wurden nicht geschrieben
        tableClose(deviceID, hTable);
        fclose(fp);
        return -IV_ERROR_FILE_WRITE;
    }
    // Kopfdaten aufsetzen. Die Datei ist erst gueltig, wenn die Kopfdaten geschrieben wurden.
    structHeader.Id = 0xDFDF;
    structHeader.offs = sizeof(StructFTFileHeader);
    structHeader.count = recordCount;

    // Fingertemplates abrufen.
    do
    {
        // Datenspeicher loeschen
        size = sizeof(data);
        memset(data, 0, sizeof(data));

        // Datensatz aus Liste abrufen
        if ((res = tableGetRecord(deviceID, hTable, data, (short&)size)) != IV_ERROR_SUCCESS)
        {
            // Modul oder Funktionsfehler? Dann ist die Kommunikation noch moeglich.
            if (res > 0) tableClose(deviceID, hTable);
            fclose(fp);
            return res;
        }
        // bekannte Daten in struktur uebernehmen
        memcpy((unsigned char*)&Id, data, sizeof(StructFingertemplateId));

        // Datenspeicher loeschen
        size = sizeof(data);
        memset(data, 0, sizeof(data));

        // Das dazugehoerige Fingertemplate abfragen
        while ((res = fingerprintTableGetRecord(deviceID, type, (unsigned int)atof(Id.pid), atoi(Id.fid), data, size)) != IV_ERROR_SUCCESS)
        {
            // Wenn das Templateformat unbekannt oder nicht dem verwendeten entspricht, dann ggf. noch anderes versuchen.
            // Funktionsfehler 1 und 3
            if ((res == 1 || res == 3) && type != 0)
            {
                type--; // Nächstes Format prüfen.
                continue;
            }

            // Modul oder Funktionsfehler? Dann ist die Kommunikation noch moeglich.
            if (res > 0) tableClose(deviceID, hTable);
            fclose(fp);
            return res;
        }

        // Groesse ist abhaengig von verwendetem Templatetyp.
        switch ( type )
        {
        case 0:
            size = sizeof(StructFingertemplateDIN);
            break;

        case 1:
            size = sizeof(StructFingertemplateIdencom);
            break;

        case 2:
            size = sizeof(StructFingertemplateIdencomStandard);
            break;

        default:
            size = 0;
            break;
        }
        // CRC16 ueber das Fingertemplate erstellen
        crc = GetCRC16(data, size);
        // Datensatz in die Datei wegschreiben
        if (fwrite(data, 1, (size_t)size, fp) < (size_t)size)
        {
            // Daten wurden nicht geschrieben
            tableClose(deviceID, hTable);
            fclose(fp);
            return -IV_ERROR_FILE_WRITE;
        }
        // CRC schreiben
        if (fwrite(&crc, 1, sizeof(unsigned short), fp) < sizeof(unsigned short))
        {
            // Daten wurden nicht geschrieben
            tableClose(deviceID, hTable);
            fclose(fp);
            return -IV_ERROR_FILE_WRITE;
        }

        // Naechsten Datensatz abrufen
    } while ((res = tableSkip(deviceID, hTable, 1, 1)) == IV_ERROR_SUCCESS);

    // Nicht: Sprung fuehrte zu EOF?
    // Funktionsfehler: 4
    if (res != 4)
    {
        // Modul oder Funktionsfehler?
        if (res > 0) tableClose(deviceID, hTable);
        fclose(fp);
        return res;
    }

    // Headerstruktur abschliessen und schreiben
    structHeader.type = type;
    structHeader.crc = GetCRC16((unsigned char*)&structHeader, sizeof(structHeader) - 2);

    // Auf Dateianfang und Headerstruktur schreiben
    if (fseek(fp, 0, 0) != 0 || fwrite(&structHeader, 1, sizeof(structHeader), fp) < sizeof(structHeader))
    {
        // Daten wurden nicht geschrieben
        tableClose(deviceID, hTable);
        fclose(fp);
        return -IV_ERROR_FILE_WRITE;
    }

    // Tabelle schliessen
    tableClose(deviceID, hTable);

    // Datei schliessen
    fclose(fp);

    return IV_ERROR_SUCCESS;
}

// Flags:
// Bit 1: gesetzt bedeutet: Enthaltene Fingertemplates beibehalten.
// Wiederherstellen der Fingerprints
int Channel4::fingerprintRestore(short deviceID, char *backupFile, int flags)
{
    int res = IV_ERROR_SUCCESS;
    FILE *fp;
    StructFingertemplateId Id;
    unsigned char data[IV_PACKET_LENGTH_DATA];
    size_t size, sizeCheck;
    unsigned short crc, crcSegment;
    int recordSkiped = 0, recordCountAfter;
    unsigned char val = 0;
    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res = getTypeDevice(deviceID, typeDevice)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Boardversion lesen.
    if ((res = getTypeBoard(deviceID, typeBoard)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    bool evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Pruefen ob das Modul verfuegbar ist.
        if ((res = getTypeFingerprint(deviceID, val)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        if (!val)
        {
            return -IV_ERROR_NO_MODULE;
        }
    }

    // Datei oeffnen
    if ((fp = fopen(backupFile, "rb")) == NULL)
    {
        return -IV_ERROR_FILE_OPEN;
    }

    // Kopfstruktur vorbereiten
    StructFTFileHeader structHeader;
    memset(&structHeader, 0, sizeof(StructFTFileHeader));
    // Kopfdaten lesen
    if (fread(&structHeader, 1, sizeof(StructFTFileHeader), fp) < sizeof(StructFTFileHeader))
    {
        // Daten wurden nicht gelesen
        fclose(fp);
        return -IV_ERROR_FILE_READ;
    }
    // Kopfdaten pruefen.
    crc = GetCRC16((unsigned char*)&structHeader, sizeof(StructFTFileHeader));
    if (structHeader.Id != 0xDFDF || 
        crc != 0 ||
        structHeader.type > 2)
    {
        fclose(fp);
        return -IV_ERROR_FILE_FORMAT;
    }
    // Auf Datenanfang wechseln
    fseek(fp, structHeader.offs, 0);

    // Groesse ist abhaengig von verwendetem Templatetyp.
    switch ( structHeader.type )
    {
    case 0:
        size = sizeof(StructFingertemplateDIN);
        break;

    case 1:
        size = sizeof(StructFingertemplateIdencom);
        break;

    case 2:
        size = sizeof(StructFingertemplateIdencomStandard);
        break;

    default:
        size = 0;
    }

    // Datensaetze loeschen?
    if (!(flags & 1))
    {
        // Alle Datensaetze vom Modul loeschen
        if ((res = fingerprintTableDeleteRecord(deviceID, 0, 0)) != IV_ERROR_SUCCESS)
        {
            fclose(fp);
            return res;
        }
    }

    // Fingertemplates einspielen.
    while (!feof(fp) && !ferror(fp))
    {
        // Datenspeicher loeschen
        memset(data, 0, sizeof(data));
        memset(&Id, 0, sizeof(StructFingertemplateId));

        // Template aus Datei laden
        if ((sizeCheck = fread(data, 1, size, fp)) < size)
        {
            if (sizeCheck == 0 && feof(fp))
            {
                continue;
            }
            else if (sizeCheck && !ferror(fp))
            {
                fclose(fp);
                return -IV_ERROR_FILE_CRC; // Daten sind inkonsistent
            }
            // Daten wurden nicht gelesen
            fclose(fp);
            return -IV_ERROR_FILE_READ;
        }
        // CRC16 des Fingertemplates erstellen
        crc = GetCRC16(data, (unsigned int)size);
        // CRC lesen
        if ((sizeCheck = fread(&crcSegment, 1, sizeof(unsigned short), fp)) < sizeof(unsigned short)) {
            if ((sizeCheck == 0 && feof(fp)) || (sizeCheck && !ferror(fp)))
            {
                fclose(fp);
                return -IV_ERROR_FILE_CRC; // Daten sind inkonsistent
            }
            // Daten wurden nicht gelesen
            fclose(fp);
            return -IV_ERROR_FILE_READ;
        }
        // CRC pruefen
        if (crc != crcSegment)
        {
            fclose(fp);
            return -IV_ERROR_FILE_CRC;
        }

        // Das Fingertemplate anfuegen
        if ((res = fingerprintTableAppendRecord(deviceID, structHeader.type, data, (unsigned int)size)) != IV_ERROR_SUCCESS)
        {
            // Wenn das Template bereits vorhanden war, dann ggf. uebergehen.
            // Funktionsfehler 134
            if (res == 134)
            {
                recordSkiped++;
                continue;
            }
            else if (res == 132)
            {
                // Speicher des Moduls ist voll
                res = -IV_ERROR_MEMORY_FULL;
            }
            else if (res == 1 || res == 3)
            {
                // Falsches Templateformat
                res = -IV_ERROR_FILE_DATA_FORMAT;

            }
            else if (res == 2)
            {
                int failLoop = 0;
                // Bei aelterer Hardware des Biokey3000 kann eine Verzoegerung nach dem loeschen der Datensaetze
                // vorkommen. Hier wird versucht das Template nochmals zu uebertragen.
                for (; failLoop < 2; failLoop++)
                {
                    if ((res = fingerprintTableAppendRecord(deviceID, structHeader.type, data, (int)size)) == 2)
                    {
                        // Kleine Verzoegerung
                        //m_channel4.Trace("ERROR: GetInfoVerzoegerung: %i\n", failLoop);
                        Sleep(2000);
                        continue;
                    }
                    break;
                }

                // Konnte der Befehl ausgefuehrt werden?
                // Wenn das Template bereits vorhanden war, dann ggf. uebergehen.
                // Funktionsfehler 134
                if (res == 134)
                {
                    recordSkiped++;
                    continue;
                }
                else if (res == 132)
                {
                    // Speicher des Moduls ist voll
                    res = -IV_ERROR_MEMORY_FULL;
                }
                else if (res == 1 || res == 3)
                {
                    // Falsches Templateformat
                    res = -IV_ERROR_FILE_DATA_FORMAT;
                }
                else if (res == IV_ERROR_SUCCESS)
                {
                    //m_channel4.Trace("OK: Verzoegerung: %i\n", failLoop);
                    continue;
                }
            }

            fclose(fp);
            return res;
        }
    }

    // Nicht: Sprung fuehrte zu EOF?
    if (ferror(fp))
    {
        fclose(fp);
        return -IV_ERROR_FILE_READ;
    }

    // Datei schliessen
    fclose(fp);

    // Tabelle mit Fingerprintdaten erzeugen
    if ((res = tableGenerate(deviceID, (char*)"FINGERPRINT.IDLIST", (char*)"")) != IV_ERROR_SUCCESS)
    {
        // Liegen im Modul keine Daten vor?
        if (res == 133)
        {
            res = -IV_ERROR_NO_DATA_AVAILABLE;
        }
        return res;
    }
    int hTable;
    // Tabelle mit Fingerprintdaten oeffnen
    if ((res = tableOpen(deviceID, (char*)"FINGERPRINT.IDLIST", 0, hTable, (char*)"")) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Pruefen ob Datensaetze vorliegen
    if ((res = tableGetRecordCount(deviceID, hTable, recordCountAfter)) != IV_ERROR_SUCCESS)
    {
        // Modul oder Funktionsfehler?
        if (res > 0) tableClose(deviceID, hTable);
        return res;
    }
    // Tabelle schliessen
    if ((res = tableClose(deviceID, hTable)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Pruefen ob die Datensatzanzahl passt.
    if (structHeader.count > recordCountAfter)
    {
        // Es muessen mindestens die Anzahl geforderter Datensaetze vorhanden sein.		
    }

    return IV_ERROR_SUCCESS;
}

int Channel4::setOptionFirmware(short deviceID, int mask, int options)
{
    int res = IV_ERROR_SUCCESS;

    // Externe Leserserie
    if (mask & OFF_EXTERNAL_READER)
    {
        if ((res = setTypeExternalReader(deviceID, (options & OFF_EXTERNAL_READER) ? 2 : 1)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
    }

    // IOModule
    if (mask & OFF_IOMODULE)
    {
        if ((res = setOptionIOModule(deviceID, (options & OFF_IOMODULE) ? 1 : 0)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 2 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    // Timeboy anbindung
    if (mask & OFF_TBADAPTION)
    {
        if ((res = setOptionTBAdaption(deviceID, (options & OFF_TBADAPTION) ? 1 : 0)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 2 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    // ComServer
    if (mask & OFF_COMSERVER)
    {
        if ((res = setOptionComServer(deviceID, (options & OFF_COMSERVER) ? 1 : 0)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 2 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    // Transponder schreiben
    if (mask & OFF_TPWRITE)
    {
        if ((res = setOptionTPWrite(deviceID, (options & OFF_TPWRITE) ? 1 : 0)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 2 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    // HTTP-Protokoll
    if (mask & OFF_HTTP)
    {
        if ((res = setOptionHTTP(deviceID, (options & OFF_HTTP) ? 1 : 0)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 2 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    // Biokey update
    if (mask & OFF_BIOKEY_UPDATE)
    {
        if ((res = setOptionBiokeyUpdate(deviceID, (options & OFF_BIOKEY_UPDATE) ? 1 : 0)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 2 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    // SetupOption: Signalverarebeitung
    if (mask & OFF_SO_IOCONTROL)
    {
        if ((res = setSetupOptions(deviceID, 0x0001, (options & OFF_SO_IOCONTROL) ? 0x0001 : 0x0000)) != IV_ERROR_SUCCESS)
        {
            // Erst ab Version 3 verfuegbar
            if (((res >> 8) & 0xFF) == 2)
            {
                // Die weitere Verarbeitung abbrechen.
                return IV_ERROR_SUCCESS;
            }

            return res;
        }
    }

    return res;
}

int Channel4::getOptionFirmware(short deviceID, int &mask, int &options)
{
    int res = IV_ERROR_SUCCESS;
    unsigned char val;

    // Zuruecksetzen der Werte
    mask = options = 0;

    // Externe Leserserie
    if ((res = getTypeExternalReader(deviceID, val)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    mask |= OFF_EXTERNAL_READER;
    options |= (val == 2) ? OFF_EXTERNAL_READER : 0;

    // IOModule
    if ((res = getOptionIOModule(deviceID, val)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    mask |= OFF_IOMODULE;
    options |= (val == 1) ? OFF_IOMODULE : 0;

    // Timeboy anbindung
    if ((res = getOptionTBAdaption(deviceID, val)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    mask |= OFF_TBADAPTION;
    options |= (val == 1) ? OFF_TBADAPTION : 0;

    // ComServer
    if ((res = getOptionComServer(deviceID, val)) != IV_ERROR_SUCCESS)
    {	
        // Erst ab Version 2 verfuegbar
        if (((res >> 8) & 0xFF) == 2)
        {
            // Die weitere Verarbeitung abbrechen.
            return IV_ERROR_SUCCESS;
        }

        return res;
    }
    mask |= OFF_COMSERVER;
    options |= (val == 1) ? OFF_COMSERVER : 0;

    // Transponder schreiben
    if ((res = getOptionTPWrite(deviceID, val)) != IV_ERROR_SUCCESS)
    {
        // Erst ab Version 2 verfuegbar
        if (((res >> 8) & 0xFF) == 2)
        {
            // Die weitere Verarbeitung abbrechen.
            return IV_ERROR_SUCCESS;
        }

        return res;
    }
    mask |= OFF_TPWRITE;
    options |= (val == 1) ? OFF_TPWRITE: 0;

    // Http-Protokoll
    if ((res = getOptionHTTP(deviceID, val)) != IV_ERROR_SUCCESS)
    {
        // Erst ab Version 2 verfuegbar
        if (((res >> 8) & 0xFF) == 2)
        {
            // Die weitere Verarbeitung abbrechen.
            return IV_ERROR_SUCCESS;
        }

        return res;
    }
    mask |= OFF_HTTP;
    options |= (val == 1) ? OFF_HTTP : 0;

    // Biokey Update
    if ((res = getOptionBiokeyUpdate(deviceID, val)) != IV_ERROR_SUCCESS)
    {
        // Erst ab Version 2 verfuegbar
        if (((res >> 8) & 0xFF) == 2)
        {
            // Die weitere Verarbeitung abbrechen.
            return IV_ERROR_SUCCESS;
        }

        return res;
    }
    mask |= OFF_BIOKEY_UPDATE;
    options |= (val == 1) ? OFF_BIOKEY_UPDATE : 0;

    unsigned short soMask, soValue;
    // Setup Options: Signalverarbeitung
    if ((res = getSetupOptions(deviceID, soMask, soValue)) != IV_ERROR_SUCCESS)
    {
        // Erst ab Version 2 verfuegbar
        if (((res >> 8) & 0xFF) == 2)
        {
            // Die weitere Verarbeitung abbrechen.
            return IV_ERROR_SUCCESS;
        }

        return res;
    }
    mask |= OFF_SO_IOCONTROL;
    options |= (soValue & 0x0001) ? OFF_SO_IOCONTROL : 0;

    return res;
}

int Channel4::setWlanConfigDataWithTimeboySlot(short deviceIdRemote, char *fileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = setWlanConfigData( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setWlanConfigData(short deviceIDRemote, char *fileName)
{
    // WlanConfig Daten
    FileWlanConfig fileWlanConfig;
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
    unsigned char *buf;
    unsigned int size = 0;
    unsigned int crc = 0;
    unsigned char val;
    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res = getTypeDevice(deviceIDRemote, typeDevice)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Boardversion lesen.
    if ((res = getTypeBoard(deviceIDRemote, typeBoard)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    bool evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Pruefen ob ein WLAN in den Hardwaresettings vorliegt
        if ((res = getTypeModemMC(deviceIDRemote, val)) != IV_ERROR_SUCCESS)
        {
            return res;
        }

#ifndef _DEBUG
        // Ist ein WLAN Modul von Redpine verfuegbar.
        if (val != IV_VALUE_TYPE_MODEM_RS9110_N_11_24)
            return -IV_ERROR_NO_MODULE;
#endif
    }

    // WLAN Konfigurationsdatei laden
    if (!fileWlanConfig.Load(fileName))
    {
        switch (fileWlanConfig.errorState())
        {
		case FileWlanConfig::FileOpen:
            return -IV_ERROR_FILE_OPEN;

		case FileWlanConfig::FileFormat:
        default:
            return -IV_ERROR_FILE_FORMAT;
        }
    }

    if (!fileWlanConfig.WlanConfigCheckCRC(-1))
        return -IV_ERROR_FILE_CRC;

    //	memset(version, 0, sizeof(version));
    // Bootversion lesen 
    //	if ((res = m_channel4.GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS)
    //{
    //		return res;
    //	}

    // Ggf. gesetztes Streaming deaktivieren.
    setStreamTransfer(false);

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Sind WLAN Daten vorhanden?
    if (fileWlanConfig.GetSizeData())
    {
        address = 0;
        buf = fileWlanConfig.m_memWlanConfig;
        size = fileWlanConfig.m_sizeData;
        crc = GetCRC32(buf, size);

        // Systemstopp setzen
        res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_WLANCONFIGS, address, size, crc);
        if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
            return res;

        // Funktionsfehler nicht Blocktyp unbekannt?
        if ((res & 0xFF) != 1)
        {
            // Textdaten uebertragen
            if ((res = setFlash(deviceIDRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Systemstopp aufheben
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_WLANCONFIGS, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        else
        {
            // Funktion wird von Firmware nicht unterstuetzt.
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
    }
    else
    {
        // Es liegen keine Daten fuer die Uebertragung vor.
        return -IV_ERROR_NO_DATA_AVAILABLE;

    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE_BG, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // WLAN Daten erfolgreich uebertragen
    return res;
}

int Channel4::getWlanConfigDataWithTimeboySlot(short deviceIdRemote, char *fileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = getWlanConfigData( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getWlanConfigData(short deviceIDRemote, char *fileName)
{
    FileWlanConfig fileWlanConfig;
    int res = IV_ERROR_SUCCESS;
    unsigned char val;
    //
    unsigned int address, size, crc;
    address = size = crc = 0;
    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res = getTypeDevice(deviceIDRemote, typeDevice)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Boardversion lesen.
    if ((res = getTypeBoard(deviceIDRemote, typeBoard)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    bool evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Pruefen ob ein WLAN in den Hardwaresettings vorliegt
        if ((res = getTypeModemMC(deviceIDRemote, val)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
#ifndef _DEBUG
        // Ist ein WLAN Modul von Redpine verfuegbar.
        if (val != IV_VALUE_TYPE_MODEM_RS9110_N_11_24)
            return -IV_ERROR_NO_MODULE;
#endif
    }

    // Segmentbegin: Bedienung stoppen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        if ((res & 0xFF) == 1)
        {
            // Funktionsfehler Typ: IV_VALUE_SEGMENT_GET_UPDATE_BG unbekannt.
            // Wenn dass der Fall ist, wird das Auslesen noch nicht unterstuetzt.
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
        else
        {
            return res;
        }
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc);

    fileWlanConfig.reset();
    //////////////////////////////
    // Begin

    // Segmentbegin: Setup uebertragen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_WLANCONFIGS, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc);


    //////////////////////////////
    // Daten

    // EEPROM uebertragen
    if ((res = getFlash(deviceIDRemote, address, fileWlanConfig.m_memWlanConfig, size)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    //////////////////////////////
    // Ende

    // Segmentend: Setup uebertragen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_WLANCONFIGS)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP);

    // Segmentend: Bedienung stoppen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG);

    // Datenanzahl in Datei uebernehmen.
    fileWlanConfig.m_sizeData = size;
    unsigned int crcTest = GetCRC32(fileWlanConfig.m_memWlanConfig, fileWlanConfig.m_sizeData);
    if (crc != crcTest)
    {
        trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
        return -IV_ERROR_CRC_SEGMENT;
    }

    // Datei speichern
    if (!fileWlanConfig.Save(fileName)) {

        trace("%s(%i):ERROR(1): %s [ Schreiben der Datei gescheitert. ]\n", __FILE__, __LINE__, __func__);
        return -IV_ERROR_FILE_WRITE;
    }

    return res;
}

int Channel4::setHardwareConfigDataWithTimeboySlot(short deviceIdRemote, char *fileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = setHardwareConfigData( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setHardwareConfigData(short deviceIDRemote, char *fileName)
{
    // HardwareConfig Daten
    HardwareInfo hardwareInfo;
    FileHardwareInfo &fileHardwareConfig = hardwareInfo.fileHardwareInfo();
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
    unsigned char *buf;
    unsigned int size = 0;
    unsigned int crc = 0;

    // Hardware Konfigurationsdatei laden
    if ( !fileHardwareConfig.load( fileName ) )
    {
        switch ( fileHardwareConfig.errorState() )
        {
		case FileHardwareInfo::FileOpen:
            return -IV_ERROR_FILE_OPEN;

		case FileHardwareInfo::FileFormat:
        default:
            return -IV_ERROR_FILE_FORMAT;
        }
    }

    // Handelt es sich um eine MasterIV - HWI Datei, dann Abbruch mit Fehler.
    if ( hardwareInfo.isMasterIV() )
    {
        trace("%s(%i):ERROR(1): %s [ Es handelt sich um eine HIP-Datei für MasterIV. ]\n", __FILE__, __LINE__, __func__);
        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    }

    //	memset(version, 0, sizeof(version));
    // Bootversion lesen 
    //	if ((res = m_channel4.GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS)
    //{
    //		return res;
    //	}

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Sind HardwareConfig Daten vorhanden?
    if ( fileHardwareConfig.dataSize() )
    {
        address = 0;
        buf = const_cast<unsigned char*>( fileHardwareConfig.data() );
        size = fileHardwareConfig.dataSize();
        crc = GetCRC32(buf, size);

        // Systemstopp setzen
        res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_HARDWARECONFIGS, address, size, crc);
        if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
            return res;

        // Funktionsfehler nicht Blocktyp unbekannt?
        if ((res & 0xFF) != 1)
        {
            // Textdaten uebertragen
            if ((res = setFlash(deviceIDRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Systemstopp aufheben
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_HARDWARECONFIGS, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        else
        {
            // Funktion wird von Firmware nicht unterstuetzt.
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
    }
    else
    {
        // Es liegen keine Daten fuer die Uebertragung vor.
        return -IV_ERROR_NO_DATA_AVAILABLE;

    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE_BG, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Hardware Daten erfolgreich uebertragen
    return res;
}

int Channel4::getHardwareConfigDataWithTimeboySlot(short deviceIdRemote, char *fileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = getHardwareConfigData( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getHardwareConfigData(short deviceIDRemote, char *fileName)
{
    HardwareInfo hardwareInfo;
    int res = IV_ERROR_SUCCESS;
 
    if ( (res = getHardwareInfo( deviceIDRemote, hardwareInfo )) == IV_ERROR_SUCCESS )
    {
        // Datei speichern
        if ( !hardwareInfo.fileHardwareInfo().save( fileName ) )
        {
            trace("%s(%i):ERROR(1): %s [ Schreiben der Datei gescheitert. ]\n", __FILE__, __LINE__, __func__);
            return -IV_ERROR_FILE_WRITE;
        }
    }

    return res;
}

int Channel4::writeFileDataWithTimeboySlot(short deviceIdRemote, char *filePath)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = writeFileData( deviceIdRemote, filePath );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::writeFileData(short deviceIDRemote, char *filePath)
{
    // HardwareConfig Daten
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
    unsigned char *buf = 0;
    unsigned int size = 0;
    unsigned int crc = 0;
    char *fileName = filePath + strlen( filePath ) - 1;
    struct stat fileStat;

    for ( ;fileName != filePath; fileName-- )
    {
        if (*fileName == '\\' || *fileName == '/')
        {
            fileName++;
            break;
        }
    }

    if ( stat( filePath, &fileStat ) != 0 )
    {
        return -IV_ERROR_FILE_OPEN;
    }

    FILE *fp = fopen( filePath, "rb" );
    // Datei oeffnen
    if ( fp == NULL )
    {
        return -IV_ERROR_FILE_OPEN;
    }

    buf = new unsigned char[ fileStat.st_size ];
    if ( fread( buf, sizeof(unsigned char), fileStat.st_size, fp ) != static_cast<size_t>( fileStat.st_size ) )
    {
        fclose( fp );
        trace("%s(%i):ERROR(1): %s [ Lesen der Datei gescheitert. ]\n", __FILE__, __LINE__, __func__);
        res = -IV_ERROR_FILE_READ;
        goto label_end;
    }

    fclose( fp );

    if ((res = setSystemVariable( deviceIDRemote, (char*)"SYSTEM.FILETRANSFER", fileName ) ) != IV_ERROR_SUCCESS )
    {
        // Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
        // und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
        if ((res & 0xFF) == 3)
        {
            // Fehler liegt bei Wert
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
            goto label_end;
        }
        else if ((res & 0xFF) == 2)
        {
            // Variable nicht bekannt
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
            goto label_end;
        }

        goto label_end;
    }

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        goto label_end;
    }

    address = 0;
    size = fileStat.st_size;
    crc = GetCRC32(buf, size);

    // Systemstopp setzen
    res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_FILETRANSFER, address, size, crc);
    if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
    {
        goto label_end;
    }

    // Funktionsfehler nicht Blocktyp unbekannt?
    if ((res & 0xFF) != 1)
    {
        // Textdaten uebertragen
        if ((res = setFlash(deviceIDRemote, address, buf, size)) != IV_ERROR_SUCCESS)
        {
            goto label_end;
        }

        // Systemstopp aufheben
        if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_FILETRANSFER, crc)) != IV_ERROR_SUCCESS)
        {
            goto label_end;
        }
    }
    else
    {
        // Funktion wird von Firmware nicht unterstuetzt.
        res = -IV_ERROR_FUNCTION_UNAVAILABLE;
        goto label_end;
    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE_BG, crc)) != IV_ERROR_SUCCESS)
    {
        goto label_end;
    }

label_end:
    if ( buf )
        delete[] buf;

    // Daten erfolgreich uebertragen
    return res;
}

int Channel4::readFileDataWithTimeboySlot(short deviceIdRemote, char *filePath)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = readFileData( deviceIdRemote, filePath );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::readFileData(short deviceIDRemote, char *filePath)
{
    int res = IV_ERROR_SUCCESS;
    std::vector<unsigned char> buffer;
    char *fileName = filePath + strlen( filePath ) - 1;
    unsigned int address, size, crc, crcTest;
    FILE *fp = NULL;
    address = size = crc = 0;

    for ( ;fileName != filePath; fileName-- )
    {
        if (*fileName == '\\' || *fileName == '/')
        {
            fileName++;
            break;
        }
    }
    //

    if ((res = setSystemVariable( deviceIDRemote, (char*)"SYSTEM.FILETRANSFER", fileName ) ) != IV_ERROR_SUCCESS )
    {
        // Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
        // und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
        if ((res & 0xFF) == 3)
        {
            // Fehler liegt bei Wert
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
        else if ((res & 0xFF) == 2)
        {
            // Variable nicht bekannt
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
        }

        return res;
    }

    // Segmentbegin: Bedienung stoppen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        if ((res & 0xFF) == 1)
        {
            // Funktionsfehler Typ: IV_VALUE_SEGMENT_GET_UPDATE_BG unbekannt.
            // Wenn dass der Fall ist, wird das Auslesen noch nicht unterstuetzt.
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
        else
        {
            return res;
        }
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc);

    //////////////////////////////
    // Begin

    // Segmentbegin: Setup uebertragen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_FILETRANSFER, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc);

    buffer.resize( size );
    //////////////////////////////
    // Daten

    // EEPROM uebertragen
    if ((res = getFlash(deviceIDRemote, address, buffer.data(), buffer.size())) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    //////////////////////////////
    // Ende

    // Segmentend: Setup uebertragen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_FILETRANSFER)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP);

    // Segmentend: Bedienung stoppen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG);

    crcTest = GetCRC32( buffer.data(), buffer.size() );
    if (crc != crcTest)
    {
        trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
        return -IV_ERROR_CRC_SEGMENT;
    }

    // Datenanzahl in Datei uebernehmen.
    fp = fopen( filePath, "wb" );
    // Datei oeffnen
    if ( fp == NULL )
    {
        return -IV_ERROR_FILE_OPEN;
    }

    if ( fwrite( buffer.data(), sizeof(unsigned char), buffer.size(), fp ) != size )
    {
        fclose( fp );
        trace("%s(%i):ERROR(1): %s [ Schreiben der Datei gescheitert. ]\n", __FILE__, __LINE__, __func__);
        return -IV_ERROR_FILE_WRITE;
    }

    fclose( fp );

    return res;
}

int Channel4::setTextDataWithTimeboySlot( short deviceIdRemote, char *fileName )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = setTextData( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setTextData(short deviceIDRemote, char *fileName)
{
    // Zeichensatz- und Grafikdaten
    FileDeviceText *pFileDeviceText = _fileDevice.GetFileDeviceText();
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
    unsigned char *buf;
    unsigned int size = 0;
    unsigned int crc = 0;

    if (!pFileDeviceText->Import(fileName))
    {
        switch (pFileDeviceText->errorState())
        {
		case FileIntelHex::FileOpen:
            return -IV_ERROR_FILE_OPEN;

		case FileIntelHex::FileFormat:
        default:
            return -IV_ERROR_FILE_FORMAT;
        }
    }

    //	memset(version, 0, sizeof(version));
    // Bootversion lesen 
    //	if ((res = GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS)
    //	{

    //		return res;
    //	}

    // Ggf. gesetztes Streaming deaktivieren.
    setStreamTransfer(false);

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Sind Textdaten vorhanden?
    if (pFileDeviceText->GetSizeData())
    {
        // Texte als ersten Block uebertragen
        // TEXTE

        // Type User setzen.
        TEXTDATA *td = (TEXTDATA*)pFileDeviceText->m_memData;
        td->type = (_loadAsDefault) ? 2 : 3; // Textdata Default/User
        _loadAsDefault = 0; // ggf. zuruecksetzen

        address = 0;
        buf = pFileDeviceText->m_memData;
        size = (unsigned int)pFileDeviceText->m_sizeData;
        crc = GetCRC32(buf, size);

        // Systemstopp setzen
        res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_TEXT, address, size, crc);
        if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
        {
            return res;
        }

        // Funktionsfehler nicht Blocktyp unbekannt?
        if ((res & 0xFF) != 1)
        {
            // Textdaten uebertragen
            if ((res = setFlash(deviceIDRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Systemstopp aufheben
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_TEXT, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

        }
        else
        {
            // Funktion wird von Firmware nicht unterstuetzt.
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
    }
    else
    {
        // Es liegen keine Daten fuer die Uebertragung vor.
        return -IV_ERROR_NO_DATA_AVAILABLE;

    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Textdaten erfolgreich uebertragen
    return res;
}

int Channel4::setTouchKeyDataWithTimeboySlot(short deviceIdRemote, char *fileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = setTouchKeyData( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setTouchKeyData(short deviceIDRemote, char *fileName)
{
    // Touchkeydaten
    FileTouchKeys fileTouchKeys;;
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
    unsigned char *buf;
    unsigned int size = 0;
    unsigned int crc = 0;
    unsigned char val;
    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res = getTypeDevice(deviceIDRemote, typeDevice)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Boardversion lesen.
    if ((res = getTypeBoard(deviceIDRemote, typeBoard)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    bool evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Pruefen ob ein Touch in den Hardwaresettings vorliegt
        if ((res = getTypeTouch(deviceIDRemote, val)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        if (val == IV_VALUE_TYPE_TOUCH_NONE)
        {
            return -IV_ERROR_NO_MODULE;
        }
    }

    // Touchdatendatei laden
    if (!fileTouchKeys.Load(fileName))
    {
        switch (fileTouchKeys.errorState())
        {
		case FileTouchKeys::FileOpen:
            return -IV_ERROR_FILE_OPEN;

		case FileTouchKeys::FileFormat:
        default:
            return -IV_ERROR_FILE_FORMAT;
        }
    }

    //	memset(version, 0, sizeof(version));
    // Bootversion lesen 
    //	if ((res = m_channel4.GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS)
    //{
    //		return res;
    //	}

    // Ggf. gesetztes Streaming deaktivieren.
    setStreamTransfer(false);

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Sind TouchKeyDaten vorhanden?
    if (fileTouchKeys.GetSizeData())
    {
        address = 0;
        buf = fileTouchKeys.m_memTouchKeys;
        size = fileTouchKeys.m_sizeData;
        crc = GetCRC32(buf, size);

        // Systemstopp setzen
        res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_TOUCHKEYS, address, size, crc);
        if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
        {
            return res;
        }

        // Funktionsfehler nicht Blocktyp unbekannt?
        if ((res & 0xFF) != 1)
        {
            // Textdaten uebertragen
            if ((res = setFlash(deviceIDRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Systemstopp aufheben
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_TOUCHKEYS, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        else
        {
            // Funktion wird von Firmware nicht unterstuetzt.
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
    }
    else
    {
        // Es liegen keine Daten fuer die Uebertragung vor.
        return -IV_ERROR_NO_DATA_AVAILABLE;

    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // TouchKeyDaten erfolgreich uebertragen
    return res;
}

int Channel4::setDisplayColorsWithTimeboySlot(short deviceIdRemote, char *fileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = setDisplayColors( deviceIdRemote, fileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setDisplayColors(short deviceIDRemote, char *fileName)
{
    // Farbdaten
    FileDisplayColors fileDisplayColors;
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
    unsigned char *buf;
    unsigned int size = 0;
    unsigned int crc = 0;
    unsigned char val;

    // Gerätetyp auslesen.
    if ((res = getTypeDevice(deviceIDRemote, val)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    // Übertragung nur möglich bei:
    // - TimeboyIV
    if (val != IV_VALUE_TYPE_DEVICE_TIMEBOY)
    {
        return -IV_ERROR_DEVICE_WRONG;
    }

    // Pruefen ob ein Farbdisplay in den Hardwaresettings vorliegt
    if ((res = getTypeDisplay(deviceIDRemote, val)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    if (val != IV_VALUE_TYPE_DISPLAY_SMALLER && val != IV_VALUE_TYPE_DISPLAY_TIMEBOY_2ZOLL)
    {
        return -IV_ERROR_NO_MODULE;
    }

    // Touchdatendatei laden
    if (!fileDisplayColors.Load(fileName))
    {
        switch (fileDisplayColors.errorState())
        {
		case FileDisplayColors::FileOpen:
            return -IV_ERROR_FILE_OPEN;

		case FileDisplayColors::FileFormat:
        default:
            return -IV_ERROR_FILE_FORMAT;
        }
    }

    //	memset(version, 0, sizeof(version));
    // Bootversion lesen 
    //	if ((res = m_channel4.GetVersionBootloader(deviceIDRemote, version)) != IV_ERROR_SUCCESS)
    //{
    //		return res;
    //	}
    
    // Ggf. gesetztes Streaming deaktivieren.
    setStreamTransfer(false);

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Sind TouchKeyDaten vorhanden?
    if (fileDisplayColors.GetSizeData())
    {
        address = 0;
        buf = fileDisplayColors.m_memDisplayColors;
        size = fileDisplayColors.m_sizeData;
        crc = GetCRC32(buf, size);

        // Systemstopp setzen
        res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_DISPLAYCOLORS, address, size, crc);
        if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
        {
            return res;
        }

        // Funktionsfehler nicht Blocktyp unbekannt?
        if ((res & 0xFF) != 1)
        {
            // Textdaten uebertragen
            if ((res = setFlash(deviceIDRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Systemstopp aufheben
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_DISPLAYCOLORS, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        else
        {
            // Funktion wird von Firmware nicht unterstuetzt.
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
    }
    else
    {
        // Es liegen keine Daten fuer die Uebertragung vor.
        return -IV_ERROR_NO_DATA_AVAILABLE;
    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // DisplayColorDaten erfolgreich uebertragen
    return res;
}

int Channel4::getTablesWithTimeboySlot( short deviceIdRemote, char *typeName )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = getTables( deviceIdRemote, typeName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

// Datensatzbeschreibungen aus Geraet lesen
int Channel4::getTables( short deviceIdRemote, char *typeName )
{
    char name[64];
    unsigned char tableCount, fieldCount, i;
    int err = IV_ERROR_SUCCESS;

    StructTable2 table;
    StructTable2Field field;
    StructDatensatzBeschreibung *pTable = NULL;
    StructDatensatzFeld *pField = NULL;

    // Struktur uebernehmen
    if (strcmp(typeName, "LIST") == 0)
    {
        MaxListB = 0;
    }
    else if (strcmp(typeName, "RECORD") == 0)
    {
        MaxDatB = 0;
    }
    else
    {
        return -IV_ERROR_LIST_PROCESSING;
    }

    // Alle Tabellenbeschreibungen lesen
    for (tableCount=0; tableCount < MAX_DATENSATZ_BESCHREIBUNG; tableCount++)
    {
        sprintf(name, "%s.%i", typeName, tableCount);

        if ((err = getTable(deviceIdRemote, name, table)) != IV_ERROR_SUCCESS)
        {
            // Nur wenn der Minorname (Index) nicht bekannt ist, 
            // liegt wirklich ein Fehler vor.
            if (err < 0)
            {
                // Kommunikationsfehler
                return err;
            }
            else if (err == 2)
            { 
                // Tabellenname unbekannt
                // Fehlercode zuruecksetzen
                err = IV_ERROR_SUCCESS;
            }
            else
            {
                // Funktionsfehler
                return err;
            }

            break;
        }

        // Struktur uebernehmen
        if (strcmp(typeName, "LIST") == 0)
        {
            pTable = &ListenBeschreibung[tableCount];
        }
        else if (strcmp(typeName, "RECORD") == 0)
        {
            pTable = &DatensatzBeschreibung[tableCount];
        }

        strcpy((char*)pTable->Name, table.name);
        pTable->Felder = table.count;
        pTable->P = 0;
    }

    // Feldbeschreibung lesen
    for (i=0; i < tableCount; i++)
    {
        // Struktur uebernehmen
        if (strcmp(typeName, "LIST") == 0)
        {
            pTable = &ListenBeschreibung[i];
        }
        else if (strcmp(typeName, "RECORD") == 0)
        {
            pTable = &DatensatzBeschreibung[i];
        }

        // Alle Felder lesen
        for (fieldCount=0; fieldCount < pTable->Felder; fieldCount++)
        {
            sprintf(name, "%s.%i.%i", typeName, i, fieldCount);

            if ((err = getField(deviceIdRemote, name, field)) != IV_ERROR_SUCCESS)
            {
                if (err < 0)
                {
                    // Kommunikationsfehler
                    return err;
                }
                else
                {
                    // Funktionsfehler
                    return err;
                }
            }

            // Struktur uebernehmen
            if (strcmp(typeName, "LIST") == 0)
            {
                pField = &ListenFeld[i][fieldCount];
            }
            else if (strcmp(typeName, "RECORD") == 0)
            {
                pField = &DatensatzFeld[i][fieldCount];
            }

            pField->Typ = field.type;
            pField->Cnt = field.size;
            strcpy((char*)pField->Name, field.name);
        }
    }

    // Information uebernehmen
    if (strcmp(typeName, "LIST") == 0)
    {
        MaxListB = tableCount;
    }
    else if (strcmp(typeName, "RECORD") == 0)
    {
        MaxDatB = tableCount;
    }

    return err;
}

int Channel4::writeSetupWithTimeboySlot( short deviceIdRemote, char *szFileName )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        //char val[80];
        //// Systemvariablen fuer MD5-Pruefwerte lesen.
        //// Wenn die Systemvariable definiert ist, dann wird eine andere Art der Setupuebertragung verwendent.
        //if ((res = getSystemVariable(deviceIDRemote, "SYSTEM.TMP1", val)) != IV_ERROR_SUCCESS) {

        //    // Bei Kommunikationsfehler, abbruch.
        //    if (res < IV_ERROR_SUCCESS)
        //        return res;

        //    // Befehlsausfuehrung starten.
        res = writeSetupData( deviceIdRemote, szFileName );
        //} else {

        //    // Befehlsausfuehrung starten.
        //    res = writeSetupData4_2( deviceIdRemote );
        //}

        if (res == IV_ERROR_SUCCESS && !isPasswordProtection() )
            res = clearRecords( deviceIdRemote );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::writeSetupData( short deviceIdRemote, char *szFileName )
{
    int i, fileError;
    char ext[5];
    memset(ext, 0, sizeof(ext));
    strcpy(ext, &szFileName[strlen(szFileName) - 4]);
    for (i=0; ext[i] && i < 4; i++) {
        ext[i] = tolower(ext[i]);
    }
    if (strcmp(ext, ".aes") != 0 && strcmp(ext, ".ael") != 0) {
        // Die Anwendung kann ueber den gewaehlten Weg, nur Dateien vom Typ (*.aes; *.ael) oeffnen.
        return -IV_ERROR_FILE_EXTENSION;
    }

    // Reset der StudioFile-Klasse
    m_file.SetDefault();

    // Laden der Dateidaten. Keine Beachtung der Studioversion.
    if (!m_file.Load(szFileName, &fileError, FILESTUDIOHEADER_IGNORE_MIN_STUDIO_VERSION | FILESTUDIOHEADER_IGNORE_SHOULD_STUDIO_VERSION)) {
        switch (fileError) {
        case -3:
            return -IV_ERROR_FILE_READ;

        case -7: // Mindestversion DLL wird nicht eingehalten.
            return -IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION;

        case -8: // Sollteversion DLL
            return -IV_ERROR_SETUPFILE_SHOULD_LIBRARY_VERSION;

        case -9: // Format der Setupdatei
            return -IV_ERROR_FILE_FORMAT;

        case -4: // CRC in Dateidaten
        case -10: case -11: case -12: case -13: case -14: case -15: case -16: case -17: case -18:
            return -IV_ERROR_FILE_CRC;
        }

        return -IV_ERROR_FILE_OPEN;
    }

    int res;
    unsigned char type = 0;
    if ( (res = getTypeDevice(deviceIdRemote, type)) != IV_ERROR_SUCCESS )
    {
        if ( res == -IV_ERROR_FUNCTION_UNAVAILABLE )
        {
            // Geraet als PZE-MasterIV annehmen
            type = 0;
        }
        else
        {
            return res;
        }
    }

    char version[32];
    memset(version, 0, sizeof(version));
    if ( (res = getVersion(deviceIdRemote, version)) != IV_ERROR_SUCCESS )
        return res;

    // Pruefen ob die Firmwareversion unterstuetzt wird.
    if ( !m_file.IsSupportedFirmwareVersion( version ) )
        return -IV_ERROR_VERSION_FIRMWARE;

    if ( (m_file.m_eep[P_DEVICE_TYPE] - 1) != type )
    {
        // Sonderfall IPC-Extension
        switch ( type )
        {
        case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7:
        case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10:
        case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7:
            {
                // Alle IPC-Extensions werden als die SetupId IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 geführt.
                if ( (m_file.m_eep[P_DEVICE_TYPE] - 1) != IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 )
                    return -IV_ERROR_DEVICE_WRONG;
   
                break;}
        default:
            return -IV_ERROR_DEVICE_WRONG;
        }
    }

    // Ist ein Dateiheader verfuegbar, ist die Firmwareversion zu pruefen.
    if ( m_file.IsHeader() )
    {
        CDFSoftwarebuildVersion fwVersion(version);
        // Achtung:
        // Bei der Auswertung ist die Variante "SHOULD" durch die Library nicht zu berücksichtigen, da sonst ein Einspielen verhindert würde.
        switch (m_file.ValidFirmwareVersion(fwVersion)) {
        case 0: // Kein Dateiheader, hier unmoeglich.
            return -IV_ERROR_FILE_FORMAT;

        case -1: // Unsupported 04.01.xx
            return -IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE1_VERSION;

        case -2: // Mindestversion 04.01.xx
            return -IV_ERROR_SETUPFILE_MIN_FIRMWARE1_VERSION;

        //case -3: // Sollteversion 04.01.xx
            //return -IV_ERROR_SETUPFILE_SHOULD_FIRMWARE1_VERSION;

        case -4: // Unsupported 04.02.xx
            return -IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE2_VERSION;

        case -5: // Mindestversion 04.02.xx
            return -IV_ERROR_SETUPFILE_MIN_FIRMWARE2_VERSION;

        //case -6: // Sollteversion 04.02.xx
            //return -IV_ERROR_SETUPFILE_SHOULD_FIRMWARE2_VERSION;

        case -7: // Firmware wird nicht unterstuetz.
            return -IV_ERROR_VERSION_FIRMWARE;

        case -8: // Unsupported 04.03.xx
            return -IV_ERROR_SETUPFILE_UNSUPPORTED_FIRMWARE3_VERSION;

        case -9: // Mindestversion 04.03.xx
            return -IV_ERROR_SETUPFILE_MIN_FIRMWARE3_VERSION;

        //case -10: // Sollteversion 04.03.xx
            //return -IV_ERROR_SETUPFILE_SHOULD_FIRMWARE3_VERSION;
        }
    } else {

        // Wenn kein Dateiheader vorliegt, wird sichergestellt, dass eine Setupdatei mit der hinterlegten
        // DatafoxStudioIV Version 04.02.xx nur auf eine Hardware >= V3 aufgespielt werden kann.

        // Boardtyp ermitteln und Pruefen ob die Setupversion eingespielt werden darf.
        if ( (res = getTypeBoard(deviceIdRemote, type)) != IV_ERROR_SUCCESS)
        {
            if (res  == -IV_ERROR_FUNCTION_UNAVAILABLE)
            {
                // Baugruppe als 1.4 annehmen
                type = 0x14;
            }
            else
            {
                return res;
            }
        }

        if (type < IV_VALUE_TYPE_BOARD_3_0) {
            // Neuere Setupdateien nicht in aeltere Hardwarestaende einspielen.
            //############################
            // 2010-03-04
            if ((m_file.m_eep[P_SETUP_VERSION] > 4) || ((m_file.m_eep[P_SETUP_VERSION] == 4) && (m_file.m_eep[P_SETUP_VERSION + 1] > 1))) {
                return -IV_ERROR_VERSION_SETUPFILE;
                return 0;
            }
        }
    }

    if ( (res = checkData( deviceIdRemote )) == -1 )
        return -IV_ERROR_UNREACHABLE;
    else if (res)
        return -IV_ERROR_DATA_AVAILABLE;

    res = writeSetupData4( deviceIdRemote );

    if (res == IV_ERROR_SUCCESS && !isPasswordProtection()) {
        res = Channel4ModuleTable::clearRecords(deviceIdRemote);
    }

    return res;
}

int Channel4::writeSetupData4(short deviceIDRemote)
{
    int i, res = IV_ERROR_SUCCESS;
    unsigned char dummyConfig[0x1C]; // Groesse darf nicht veraendert werden!!!
    // Berechnung 0x80 - sizeof(StructConfig)

    //@@: Uebergangsweise zur 04.01.01 release
    unsigned char *buf = NULL;
    unsigned char countTable, countField;
    unsigned int sizeAccess;

    //////////////////////////////
    // Vorbereitung (Berechnung von Groessen und CRC)
    unsigned int sizeSetup = GetFlashSetupSize();

    // Vorhandene zur moeglichen Groesse pruefen
    // 	if (sizeSetup > (EEP_SIZE_SU + sizeof(StructConfig) + sizeof(dummyConfig) + FLASH_SIZE_SU)) {
    // 
    // 		return -IV_ERROR_FILE_FORMAT;
    // 	}

    unsigned int sizeEK = GetEntrance2EKFlashSize();
    // Speicher reservieren
    buf = new unsigned char[sizeEK];
    if (buf == NULL) {
        return -IV_ERROR_ALLOC_MEM;
    }
    memset(buf, 0, sizeEK);


    //@@: Uebergangsweise zur 04.01.01 release
    // Zeiger auf Eingabeketten ohne FLASH_KONFIG_START
    // und die Zeiger der Eingabeketten mit einem Offset versehen.
    PrepareEntrance2EK(sizeSetup, buf);

    // Groesse der Flashdaten in EepMem eintragen
    *(unsigned int*)&m_file.m_eep[P_FLASH_SIZE_SU] = sizeSetup;
    ChangeOrderLong(&m_file.m_eep[P_FLASH_SIZE_SU]);

    // Anfangsadresse der Zutrittseingabeketten eintragen
    *(unsigned int*)&m_file.m_eep[P_IC_ACCESS] = sizeSetup;
    ChangeOrderLong(&m_file.m_eep[P_IC_ACCESS]);

    // Anfangsadresse der Zutrittskontrolllisten eintragen
    *(unsigned int*)&m_file.m_eep[P_LIST_ACCESS] = sizeSetup + sizeEK;
    ChangeOrderLong(&m_file.m_eep[P_LIST_ACCESS]);

    // Groesse der MDE - Eingabeketten in EepMem eintragen.
    *(unsigned int*)&m_file.m_eep[P_MDE_EK_SIZE] = GetMDEEKFlashSize();
    //ChangeOrderLong(&m_file.m_eep[P_MDE_EK_SIZE]);

    // Groesse der Zutrittskontroll - Eingabeketten in EepMem eintragen.
    *(unsigned int*)&m_file.m_eep[P_ENTRANCE_EK_SIZE] = sizeEK;
    ChangeOrderLong(&m_file.m_eep[P_ENTRANCE_EK_SIZE]);


    //
    unsigned int address, size, crc;
    address = size = crc = 0;

    // Segmentbegin: Bedienung stoppen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        // Puffer Freigeben
        delete[] buf;
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginSetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_UPDATE, address, size, crc);

    // Pruefen ob Streaming moeglich.
    clearStreamTransfer();
    // Pruefen ob die Uebertragung per Stream moeglich ist.
    if ((res = writeStream(deviceIDRemote, update, NULL, 0)) == IV_ERROR_SUCCESS) 
        setStreamTransfer(true);
    else
        setStreamTransfer(false);

    ///////////////
    // Grunddaten
    memset(dummyConfig, 0, sizeof(dummyConfig));

    // CRC berechnen
    crc = 0xFFFFFFFF;
    crc = crc32(m_file.m_eep, EEP_SIZE_SU, crc, FALSE);
    crc = crc32(m_file.m_configMem, sizeof(StructConfig), crc, FALSE);
    crc = crc32(dummyConfig, sizeof(dummyConfig), crc, FALSE);
    crc = crc32(m_file.m_flash, sizeSetup, crc, FALSE);
    //@@: Uebergangsweise zur 04.01.01 release
    crc = crc32(buf, sizeEK, crc, FALSE);
    //	crc = crc32(buf, sizeEK, crc);


    ///////////////
    // ZK II

    // Anzahl der Zutrittslisten ermitteln
    for (i=0, countTable=0; countTable < (sizeof(m_entrance2Pointer) / sizeof(unsigned int)); countTable++)
    {
        if (m_entrance2Pointer[countTable] == 0)
        {
            break;
        }

        // Gesamte Feldanzahl fuer Groeszenberechnung sammeln.
        i += m_entrance2Table[countTable].count;
    }
    sizeAccess = sizeof(m_entrance2Pointer);
    sizeAccess += countTable * sizeof(StructTable2);
    sizeAccess += i * sizeof(StructTable2Field);

    // CRC der Zutrittslisten ermitteln
    crc = crc32((unsigned char*)m_entrance2Pointer, sizeof(m_entrance2Pointer), crc, FALSE);

    for (i=0; i < countTable; i++)
    {
        crc = crc32((unsigned char*)&m_entrance2Table[i], sizeof(StructTable2), crc, FALSE);

        for (countField=0; countField < m_entrance2Table[i].count; countField++)
        {
            // Beim letzten Feld der letzten Tabelle musz der CRC abgeschlossen werden.
            crc = crc32((unsigned char*)&m_entrance2TableField[i][countField], sizeof(StructTable2Field), crc, ((i == (countTable - 1)) && (countField == (m_entrance2Table[i].count - 1))));
        }
    }

    //////////////////////////////
    // Begin
    //	double now = (double)clock() / (double)CLOCKS_PER_SEC;
    //	int nr = 0;
    //	Trace("Elapse (%i): %f\n", nr++, ((double)clock() / (double)CLOCKS_PER_SEC) - now);

    // Gesamtgroesze berechnen
    size = EEP_SIZE_SU + sizeof(StructConfig) + sizeof(dummyConfig) + sizeSetup + sizeEK + sizeAccess;

    // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
    if (!isStreamTransfer()) {
        // Segmentbeginn: Setup uebertragen
        if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            // Puffer Freigeben
            delete[] buf;
            // Zu grosse Setupdatei, es ist nicht genug speicher verfuegbar.
            if (res == 3) {
                res = -IV_ERROR_NOT_ENOUGH_FLASH;
            }
            return res;
        }
        trace("%s(%i):INFO(1): %s [ beginSetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_SETUP, address, size, crc);
    }

    //////////////////////////////
    // Grunddaten

    // EEPROM uebertragen
    if ((res = setFlash(deviceIDRemote, address, m_file.m_eep, EEP_SIZE_SU)) != IV_ERROR_SUCCESS)
    {
        // Puffer Freigeben
        delete[] buf;
        return res;
    }

    // CONFIG uebertragen (100 Byte)
    address += EEP_SIZE_SU;
    if ((res = setFlash(deviceIDRemote, address, m_file.m_configMem, sizeof(StructConfig))) != IV_ERROR_SUCCESS)
    {
        // Puffer Freigeben
        delete[] buf;
        return res;
    }

    // CONFIG dummyConfig (28 Byte)
    address += sizeof(StructConfig);
    if ((res = setFlash(deviceIDRemote, address, dummyConfig, sizeof(dummyConfig))) != IV_ERROR_SUCCESS)
    {
        // Puffer Freigeben
        delete[] buf;
        return res;
    }

    // FLASH uebertragen
    address += sizeof(dummyConfig);
    if ((res = setFlash(deviceIDRemote, address, m_file.m_flash, sizeSetup)) != IV_ERROR_SUCCESS)
    {
        // Puffer Freigeben
        delete[] buf;
        return res;
    }

    //@@: Uebergangsweise zur 04.01.01 release
    //////////////////////////////
    // Pointerarray + Strukturen der Zutrittslisten uebertragen

    // Eingabekette der Zutrittskontrolle
    address += sizeSetup;
    if ((res = setFlash(deviceIDRemote, address, buf, sizeEK)) != IV_ERROR_SUCCESS)
    {
        // Puffer Freigeben
        delete[] buf;
        return res;
    }

    // Puffer nach Uebertragung freigeben
    delete[] buf;
    buf = NULL;

    address += sizeEK;
    // Pointerarray uebertragen
    if ((res = setFlash(deviceIDRemote, address, (unsigned char*)m_entrance2Pointer, sizeof(m_entrance2Pointer))) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeof(m_entrance2Pointer);

    // Strukturen uebertragen
    for (i=0; i < countTable; i++)
    {
        // Tabellenstruktur uebertragen
        if ((res = setFlash(deviceIDRemote, address, (unsigned char*)&m_entrance2Table[i], sizeof(StructTable2))) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        address += sizeof(StructTable2);

        // Feldstrukturen uebertragen
        if ((res = setFlash(deviceIDRemote, address, (unsigned char*)m_entrance2TableField[i], m_entrance2Table[i].count * sizeof(StructTable2Field))) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        address += m_entrance2Table[i].count * sizeof(StructTable2Field);
    }

    // Im Modus Streaming ?, dann gepufferte Daten senden.
    if (isStreamTransfer() && m_streamBuffer.size()) {
        if ((res = writeStream(deviceIDRemote, setup, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
            // Zu grosse Setupdatei, es ist nicht genug speicher verfuegbar.
            if (res == 3) {
                res = -IV_ERROR_NOT_ENOUGH_FLASH;
            }
            return res;
        }
        // Gepufferte Daten loeschen.
        clearStreamTransfer();
    }

    // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
    if (!isStreamTransfer()) {

        //////////////////////////////
        // Ende
        // Segmentend: Setup uebertragen
        if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP, crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        trace("%s(%i):INFO(1): %s [ endSetSegment(%i): 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_SETUP, crc);
    }

    // Extendedstruktur lesen
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;

    ///////////////
    // Extended - Speicher Vorbereitung

    // Zeigeroffsets anpassen (substrahieren von Offset)
    AdjustExtendedMemPointer();

    // Groesse MDE und dessen Eingabeketten
    size = extBegin.offsetFreeScope;

    // CRC berechnen
    crc = 0xFFFFFFFF;
    // IOModul - Konfiguration
    crc = crc32(m_file.m_flashExtended, size, crc, TRUE);

    // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
    if (!isStreamTransfer()) {
        //////////////////////////////
        // Begin

        // Segmentbegin: Setup uebertragen
        if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP_EXTENDED, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            if ((res & 0xFF) == 1)
            {
                // Funktionsfehler Typ IV_VALUE_SEGMENT_SETUP_EXTENDED unbekannt.
            }
            else
            {		
                return res;
            }
        }
        trace("%s(%i):INFO(1): %s [ (%i)beginSetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, res, IV_VALUE_SEGMENT_SETUP_EXTENDED, address, size, crc);
    }

    if (res == IV_ERROR_SUCCESS)
    {
        //////////////////////////////
        // Extended - Speicher Uebertragen

        if ((res = setFlash(deviceIDRemote, address, m_file.m_flashExtended, size)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        address += size;

        // Im Modus Streaming ?, dann gepufferte Daten senden.
        if (isStreamTransfer() && m_streamBuffer.size()) {
            if ((res = writeStream(deviceIDRemote, setup_extended, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
                // Gepufferte Daten loeschen.
                clearStreamTransfer();
                return res;
            }
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
        }

        // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
        if (!isStreamTransfer()) {
            //////////////////////////////
            // Ende

            // Segmentend: Setup uebertragen
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP_EXTENDED, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
            trace("%s(%i):INFO(1): %s [ endSetSegment(%i): 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_SETUP_EXTENDED, crc);
        }
    }

    // Zeigeroffsets anpassen (addieren von Offset)
    AdjustExtendedMemPointer();

    // Pause wegen Verarbeitungsporblemen bis 4.1.5
    Sleep(1000);

    crc = 0;
    // Segmentend: Bedienung stoppen
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endSetSegment(%i): 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_UPDATE, crc);


    // Pruefen und setzen des Status von Passwortschutz.
    passwordCheck(deviceIDRemote, (char*)"ISPW");
    ////////////////////////////////////////////////////////////////////
    // Der folgende Teil ist nur bei Versionen unter 04.01.07 noetig.
    // Ab dieser Version ist auch der Passwortschutz moeglich.
    // Es ist hier nicht moeglich, ein Setup mit einem gesetzten Passwort zu uebertragen 
    // und darauffolgend gleich weitere Funktionen aufrufen, weil ansonsten der
    // Schluessel fuer die Passwortverschluesselung offen gelegt werden muesste.
    if (!isPasswordProtection())
    {
        // GV setzen?
        if (extBegin.offsetGlobalVariableValues)
        {		
            StructGlobVar *globVar = (StructGlobVar*)m_file.m_flash;
            char name[64];
            memset(name, 0, sizeof(name));

            // Struktur lesen
            StructValueGVx ValueGVx = *(StructValueGVx*)&m_file.m_flashExtended[extBegin.offsetGlobalVariableValues];

            for (i=0; i < 8; i++)
            {
                if (ValueGVx.ucFlags & (1 << i))
                {				
                    sprintf(name, "SETUP.%s", (char*)&globVar->SFeld[i]);
                    if ((res = setVariable(deviceIDRemote, name, (char*)&ValueGVx.szValue[i])) != IV_ERROR_SUCCESS)
                    {					
                        // Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
                        // und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
                        if ((res & 0xFF) == 3)
                        {
                            // Fehler liegt bei Wert
                            //res = -IV_ERROR_VARIABLE_VALUE;
                        }
                        else if ((res & 0xFF) == 2)
                        {
                            // Variable nicht bekannt
                            //res = -IV_ERROR_VARIABLE_UNDEFINED;
                        }

                        return res;
                    }
                }
            }
        }	

        // Standardwerte fuer Systemvariable
        //############################
        // 2005-09-19: Erweiterte Struktur fuer Parameter
        if (((m_file.m_eep[P_SETUP_VERSION    ] == 3) &&
            (m_file.m_eep[P_SETUP_VERSION + 1] == 1) &&
            ((m_file.m_eep[P_SETUP_VERSION + 2] >= 34) &&
            (m_file.m_eep[P_SETUP_VERSION + 3] >  5)) ) ||

            ((m_file.m_eep[P_SETUP_VERSION   ] == 4) &&
            (m_file.m_eep[P_SETUP_VERSION + 1] == 1) &&
            (m_file.m_eep[P_SETUP_VERSION + 3] > 2)) )
        {
            StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;

            // Liegt die Struktur vor?
            if (extBegin.offsetExtendedStruct)
            {
                StructExtended extParam = *(StructExtended*)&m_file.m_flashExtended[extBegin.offsetExtendedStruct];

                char val[32], name[64];
                // Buchungswiederholsperrdauer
                sprintf(name, "_SERVICE.DBLINPUT");
                if (extParam.doubleInputTime > 65499)
                {
                    extParam.doubleInputTime = 65499;
                }
                sprintf(val, "%i", extParam.doubleInputTime);
                // 
                if ((res = setSystemVariable(deviceIDRemote, name, val)) != IV_ERROR_SUCCESS)
                {				
                    // Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
                    // und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
                    if ((res & 0xFF) == 3)
                    {
                        // Fehler liegt bei Wert
                        //res = -IV_ERROR_VARIABLE_VALUE;
                        res = IV_ERROR_SUCCESS;
                    }
                    else if ((res & 0xFF) == 2)
                    {
                        // Variable nicht bekannt
                        //res = -IV_ERROR_VARIABLE_UNDEFINED;
                        res = IV_ERROR_SUCCESS;
                    }
                }
                // Abblendedauer fuer Bestaetigungsfeld
                sprintf(name, "_SERVICE.DEMANDTIMER");
                if (extParam.confirmationTime > 255)
                {
                    extParam.confirmationTime = 255;
                }
                sprintf(val, "%i", extParam.confirmationTime);
                // 
                if ((res = setSystemVariable(deviceIDRemote, name, val)) != IV_ERROR_SUCCESS)
                {				
                    // Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
                    // und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
                    if ((res & 0xFF) == 3)
                    {
                        // Fehler liegt bei Wert
                        //res = -IV_ERROR_VARIABLE_VALUE;
                        res = IV_ERROR_SUCCESS;
                    }
                    else if ((res & 0xFF) == 2)
                    {
                        // Variable nicht bekannt
                        //res = -IV_ERROR_VARIABLE_UNDEFINED;
                        res = IV_ERROR_SUCCESS;
                    }
                }

                // Tiefentladungsschutz
                sprintf(name, "POWER.LOW");
                sprintf(val, "%i", extParam.energieOption.powerLow);
                // 
                if ((res = setSystemVariable(deviceIDRemote, name, val)) != IV_ERROR_SUCCESS)
                {			
                    // Das Befehlsbyte wird bei unbekannter Variabler nicht veraendert
                    // und wenn der Wert nicht gesetzt werden konnte wird das hoechste bit gesetzt.
                    if ((res & 0xFF) == 3)
                    {
                        // Fehler liegt bei Wert
                        //res = -IV_ERROR_VARIABLE_VALUE;
                        res = IV_ERROR_SUCCESS;
                    }
                    else if ((res & 0xFF) == 2)
                    {
                        // Variablenname nicht bekannt
                        //res = -IV_ERROR_VARIABLE_UNDEFINED;
                        res = IV_ERROR_SUCCESS;
                    }
                    else if ((res & 0xFF) == 1)
                    {
                        // Majorbestandteil unbekannt
                        //res = -IV_ERROR_VARIABLE_UNDEFINED;
                        res = IV_ERROR_SUCCESS;
                    }
                }

            }
        }
    }

    return res;
}

int Channel4::writeSetupData4_2(short deviceIDRemote)
{
    int res = IV_ERROR_SUCCESS;
    unsigned char dummyConfig[0x1C]; // Groesse darf nicht veraendert werden!!!
    // Berechnung 0x80 - sizeof(StructConfig)

    //////////////////////////////
    // Vorbereitung (Berechnung von Groessen und CRC)
    unsigned int sizeSetup = GetFlashSetupSize();

    // Extendedstruktur
    StructExtendedFlash extBegin = *(StructExtendedFlash*)m_file.m_flashExtended;

    // Groesse der Flashdaten in EepMem eintragen
    *(unsigned int*)&m_file.m_eep[P_FLASH_SIZE_SU] = sizeSetup;
    ChangeOrderLong(&m_file.m_eep[P_FLASH_SIZE_SU]);

    // Anfangsadresse der Zutrittseingabeketten eintragen
    *(unsigned int*)&m_file.m_eep[P_IC_ACCESS] = 0;
    ChangeOrderLong(&m_file.m_eep[P_IC_ACCESS]);

    // Anfangsadresse der Zutrittskontrolllisten eintragen
    *(unsigned int*)&m_file.m_eep[P_LIST_ACCESS] = 0;
    ChangeOrderLong(&m_file.m_eep[P_LIST_ACCESS]);

    // Groesse der MDE - Eingabeketten in EepMem eintragen.
    *(unsigned int*)&m_file.m_eep[P_MDE_EK_SIZE] = GetMDEEKFlashSize();
    //ChangeOrderLong(&m_file.m_eep[P_MDE_EK_SIZE]);

    // Groesse der Zutrittskontroll - Eingabeketten in EepMem eintragen.
    *(unsigned int*)&m_file.m_eep[P_ENTRANCE_EK_SIZE] = 0;
    ChangeOrderLong(&m_file.m_eep[P_ENTRANCE_EK_SIZE]);

    // Groesse der Headerstruktur in EepMem eintragen.
    *(unsigned int*)&m_file.m_eep[P_HEADER_SIZE] = m_file.m_fileHeader->size;

    // Groesse des Erweiterten Speicherbereichs eintragen.
    *(unsigned int*)&m_file.m_eep[P_EXTENDED_SIZE] = extBegin.offsetFreeScope;

    //
    unsigned int address, size, crc;
    address = size = crc = 0;

    // Segmentbegin: Bedienung stoppen
    if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginSetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_UPDATE, address, size, crc);

    // Es wird ab der Version 04.02.04 ein MD5 ueber den Dateiheader die Listen- und Datensatzbeschreibung als Systemvariable abgelegt.
    // Vor der Uebertragung des Setups werden diese hier geloescht, falls verfuegbar.
    bool md5Sysvars = false;

    // Systemvariablen fuer MD5-Pruefwerte loeschen.
    if ((res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP1", (char*)"")) != IV_ERROR_SUCCESS) {

        // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
        if (res != 1 && res != 2)
            return res;
    } else if ((res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP2", (char*)"")) != IV_ERROR_SUCCESS) {

        // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
        if (res != 1 && res != 2)
            return res;
    } else if ((res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP3", (char*)"")) != IV_ERROR_SUCCESS) {

        // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
        if (res != 1 && res != 2)
            return res;
    } else if ((res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP4", (char*)"")) != IV_ERROR_SUCCESS) {

        // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
        if (res != 1 && res != 2)
            return res;
    } else {

        // Die Systemvariablen wurde beide erfolgreich geleert.
        md5Sysvars = true;
    }

    // Pruefen ob Streaming moeglich.
    clearStreamTransfer();
    // Pruefen ob die Uebertragung per Stream moeglich ist.
    if ((res = writeStream(deviceIDRemote, update, NULL, 0)) == IV_ERROR_SUCCESS) 
        setStreamTransfer(true);
    else
        setStreamTransfer(false);

    ///////////////
    // Grunddaten
    memset(dummyConfig, 0, sizeof(dummyConfig));

    // CRC berechnen
    crc = 0xFFFFFFFF;
    crc = crc32(m_file.m_eep, EEP_SIZE_SU, crc, FALSE);
    crc = crc32(m_file.m_configMem, sizeof(StructConfig), crc, FALSE);
    crc = crc32(dummyConfig, sizeof(dummyConfig), crc, FALSE);
    crc = crc32(m_file.m_flash, sizeSetup, crc, FALSE);
    crc = crc32(m_file.m_header, m_file.m_fileHeader->size, crc, TRUE);

    //////////////////////////////
    // Begin
    //	double now = (double)clock() / (double)CLOCKS_PER_SEC;
    //	int nr = 0;
    //	Trace("Elapse (%i): %f\n", nr++, ((double)clock() / (double)CLOCKS_PER_SEC) - now);

    // Gesamtgroesze berechnen
    // In dem Block des Setups werden verschiedene Einzelbloecke zusammengefasst uebertragen.
    size = EEP_SIZE_SU + sizeof(StructConfig) + sizeof(dummyConfig) + sizeSetup + m_file.m_fileHeader->size;

    // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
    if (!isStreamTransfer()) {
        // Segmentbeginn: Setup uebertragen
        if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            // Zu grosse Setupdatei, es ist nicht genug speicher verfuegbar.
            if (res == 3) {
                res = -IV_ERROR_NOT_ENOUGH_FLASH;
            }
            return res;
        }
        trace("%s(%i):INFO(1): %s [ beginSetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_SETUP, address, size, crc);
    }

    //////////////////////////////
    // Grunddaten

    // EEPROM uebertragen
    if ((res = setFlash(deviceIDRemote, address, m_file.m_eep, EEP_SIZE_SU)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // CONFIG uebertragen (100 Byte)
    address += EEP_SIZE_SU;
    if ((res = setFlash(deviceIDRemote, address, m_file.m_configMem, sizeof(StructConfig))) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // CONFIG dummyConfig (28 Byte)
    address += sizeof(StructConfig);
    if ((res = setFlash(deviceIDRemote, address, dummyConfig, sizeof(dummyConfig))) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // FLASH uebertragen
    address += sizeof(dummyConfig);
    if ((res = setFlash(deviceIDRemote, address, m_file.m_flash, sizeSetup)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Header der Datei uebertragen
    address += sizeSetup;
    if ((res = setFlash(deviceIDRemote, address, m_file.m_header, m_file.m_fileHeader->size)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Im Modus Streaming ?, dann gepufferte Daten senden.
    if (isStreamTransfer() && m_streamBuffer.size()) {
        if ((res = writeStream(deviceIDRemote, setup, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
            return res;
        }
        // Gepufferte Daten loeschen.
        clearStreamTransfer();
    }

    // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
    if (!isStreamTransfer()) {

        //////////////////////////////
        // Ende
        // Segmentend: Setup uebertragen
        if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP, crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        trace("%s(%i):INFO(1): %s [ endSetSegment(%i): 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_SETUP, crc);
    }

    ///////////////
    // Extended - Speicher Vorbereitung

    // Zeigeroffsets anpassen (substrahieren von Offset)
    AdjustExtendedMemPointer();

    // Groesse MDE und dessen Eingabeketten
    size = extBegin.offsetFreeScope;

    // CRC berechnen
    crc = 0xFFFFFFFF;
    // IOModul - Konfiguration
    crc = crc32(m_file.m_flashExtended, size, crc, TRUE);

    // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
    if (!isStreamTransfer()) {
        //////////////////////////////
        // Begin

        // Segmentbegin: Setup uebertragen
        if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP_EXTENDED, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            // Zu grosse Setupdatei, es ist nicht genug speicher verfuegbar.
            if (res == 3) {
                res = -IV_ERROR_NOT_ENOUGH_FLASH;
            }

            return res;
        }
        trace("%s(%i):INFO(1): %s [ (%i)beginSetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, res, IV_VALUE_SEGMENT_SETUP_EXTENDED, address, size, crc);
    }

    if (res == IV_ERROR_SUCCESS)
    {
        //////////////////////////////
        // Extended - Speicher Uebertragen

        if ((res = setFlash(deviceIDRemote, address, m_file.m_flashExtended, size)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        address += size;

        // Im Modus Streaming ?, dann gepufferte Daten senden.
        if (isStreamTransfer() && m_streamBuffer.size()) {
            if ((res = writeStream(deviceIDRemote, setup_extended, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
                // Gepufferte Daten loeschen.
                clearStreamTransfer();
                return res;
            }
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
        }

        // Wenn kein Streaming moeglich, dann mit inneren Bloecken uebertragen.
        if (!isStreamTransfer()) {
            //////////////////////////////
            // Ende

            // Segmentend: Setup uebertragen
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_SETUP_EXTENDED, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
            trace("%s(%i):INFO(1): %s [ endSetSegment(%i): 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_SETUP_EXTENDED, crc);
        }
    }

    // Zeigeroffsets anpassen (addieren von Offset)
    AdjustExtendedMemPointer();

    // Pause wegen Verarbeitungsporblemen bis 4.1.5
    Sleep(1000);

    // Werden die Systemvariablen fuer die MD5 unterstuetzt?
    if (md5Sysvars) {

        char *md5Tables = m_file.GetMD5Header(); // Dateiheader
        // Systemvariablen fuer MD5 loeschen.
        if (md5Tables && (res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP1", md5Tables)) != IV_ERROR_SUCCESS) {

            // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
            if (res != 1 && res != 2)
                return res;
        } 

        md5Tables = m_file.GetMD5Tables(true); // Listenbeschreibungen
        if (md5Tables && (res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP2", md5Tables)) != IV_ERROR_SUCCESS) {

            // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
            if (res != 1 && res != 2)
                return res;
        }

        md5Tables = m_file.GetMD5Tables(false); // Datensatzbeschreibungen
        if (md5Tables && (res = setSystemVariable(deviceIDRemote, (char*)"SYSTEM.TMP3", md5Tables)) != IV_ERROR_SUCCESS) {

            // Wenn der Fehler nicht Major oder Minorbestandteil unbekannt ist, dann hier bereits abbrechen.
            if (res != 1 && res != 2)
                return res;
        }
    }

    crc = 0;
    // Segmentend: Bedienung stoppen
    if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endSetSegment(%i): 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_UPDATE, crc);


    // Pruefen und setzen des Status von Passwortschutz.
    passwordCheck(deviceIDRemote, (char*)"ISPW");

    return res;
}

int Channel4::readSetupWithTimeboySlot( short deviceIdRemote, char *szFileName )
{
    int res;
    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        //char val[80];
        //// Systemvariablen fuer MD5-Pruefwerte ermitteln.
        //// Wenn die Systemvariable definiert ist und einen gueltigen MD5 beinhaltet, 
        //// dann wird eine andere Art der Setupuebertragung verwendent.
        //if ((res = getSystemVariable(deviceIDRemote, "SYSTEM.TMP1", val)) == IV_ERROR_SUCCESS && IsMD5(val)) {

        //    // Befehlsausfuehrung starten.
        //    res = readSetupData4_2(deviceIdRemote);
        //} else {

        //    // Bei einem Kommunikationsfehler, abbrechen.
        //    if (res < IV_ERROR_SUCCESS)
        //        return res;

        //    // Befehlsausfuehrung starten.
        res = readSetupData( deviceIdRemote, szFileName );
        //}


        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::readSetupData( short deviceIdRemote, char *szFileName )
{
    int i;

    char ext[_MAX_EXT];          // 256
#ifdef _MSC_VER
    char drive[_MAX_DRIVE];      // 3
    char dir[_MAX_DIR];          // 256
    char fname[_MAX_FNAME];      // 256

    // Dateiendung pruefen
    _splitpath(szFileName, drive, dir, fname, ext);
#else
    strcpy(ext, &szFileName[strlen(szFileName) - 4]);
#endif
    for (i=0; ext[i] && i < 4; i++)
    {
        ext[i] = tolower(ext[i]);
    }
    if (strcmp(ext, ".aes") != 0 && strcmp(ext, ".ael") != 0)
    {
        // Die Anwendung kann ueber den gewaehlten Weg, nur Dateien vom Typ (*.aes; *.ael) oeffnen.
        return -IV_ERROR_FILE_EXTENSION;
    }

    int res = 0;

    // Speicher initialisieren
    m_file.SetDefault();

    char version[64];
    memset(version, 0, sizeof(version));
    if ( (res = getVersion(deviceIdRemote, version)) != IV_ERROR_SUCCESS)
        return res;

    // Pruefen ob die Firmwareversion unterstuetzt wird.
    if (!m_file.IsSupportedFirmwareVersion(version)) {
        return -IV_ERROR_VERSION_FIRMWARE;
    }

    // Setupdatei nach erfolgreichem Auslesen, abspeichern.
    if ( (res = readSetupData4( deviceIdRemote )) == IV_ERROR_SUCCESS )
    {
        // Diese Version der Firmware der Setupdatei als Mindestversion vorgeben.
        CDFSoftwarebuildVersion fwVersion(version);
        m_file.NeedFirmwareVersion(fwVersion);

        int error;
        if ( !m_file.Save(szFileName, &error) )
            res = -IV_ERROR_FILE_OPEN;
    }

    return res;
}

int Channel4::readSetupData4(short deviceIDRemote)
{
    int i, res = IV_ERROR_SUCCESS;
    unsigned char dummyConfig[0x1C]; // Groesse nicht veraendern !!!
    // Berechnet sich aus 0x80 - sizeof(StructConfig)

    unsigned char *buf = NULL;
    unsigned int sizeSetup = 0;
    unsigned char countTable, countField;
    unsigned int sizeAccess;

    //
    unsigned int address, size, crc, crcTest = 0;
    address = size = crc = 0;

    // Segmentbegin: Bedienung stoppen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        if ((res & 0xFF) == 1)
        {
            // Funktionsfehler Typ: IV_VALUE_SEGMENT_GET_UPDATE_BG unbekannt.
            // Wenn dass der Fall ist, wird das Auslesen noch nicht unterstuetzt.
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
        else
        {
            return res;
        }
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc);

    //////////////////////////////
    // Begin

    // Segmentbegin: Setup uebertragen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc);

    // Vorhandene zur moeglichen Groesse pruefen
    if (size < (EEP_SIZE_SU + (sizeof(StructConfig) + sizeof(dummyConfig))))
    {
        return -IV_ERROR_PROCESSING;
    }

    //////////////////////////////
    // Grunddaten

    // EEPROM uebertragen
    if ((res = getFlash(deviceIDRemote, address, m_file.m_eep, EEP_SIZE_SU)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += EEP_SIZE_SU;
    // CONFIG uebertragen (100 Byte)
    if ((res = getFlash(deviceIDRemote, address, m_file.m_configMem, sizeof(StructConfig))) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeof(StructConfig);
    // CONFIG dummybytes (28 Byte)
    if ((res = getFlash(deviceIDRemote, address, dummyConfig, sizeof(dummyConfig))) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeof(dummyConfig);

    // Groesse der Flashdaten fuer Setup aus EepMem entnehmen
    ChangeOrderLong(&m_file.m_eep[P_FLASH_SIZE_SU]);
    sizeSetup = *(unsigned int*)&m_file.m_eep[P_FLASH_SIZE_SU];
    ChangeOrderLong(&m_file.m_eep[P_FLASH_SIZE_SU]);

    // FLASH uebertragen
    if ((res = getFlash(deviceIDRemote, address, m_file.m_flash, sizeSetup)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeSetup;


    unsigned int sizeEK;

    // Groesse der Zutrittskontroll - Eingabeketten aus EepMem entnehmen.
    ChangeOrderLong(&m_file.m_eep[P_ENTRANCE_EK_SIZE]);
    sizeEK = *(unsigned int*)&m_file.m_eep[P_ENTRANCE_EK_SIZE];
    ChangeOrderLong(&m_file.m_eep[P_ENTRANCE_EK_SIZE]);

    // Speicher reservieren
    buf = new unsigned char[sizeEK + 1];
    // Zeigeroffsets fuer Firmware anpassen
    if (buf == NULL)
    {
        // Speicherallokation nicht moeglich
        return -IV_ERROR_ALLOC_MEM;
    }
    memset(buf, 0, sizeEK + 1);

    //////////////////////////////
    // Pointerarray + Strukturen der Zutrittslisten uebertragen

    // Eingabekette der Zutrittskontrolle
    if ((res = getFlash(deviceIDRemote, address, buf, sizeEK)) != IV_ERROR_SUCCESS)
    {
        delete[] buf; 
        buf = NULL;
        return res;
    }
    address += sizeEK;

    // Pointerarray lesen
    if ((res = getFlash(deviceIDRemote, address, (unsigned char*)m_entrance2Pointer, sizeof(m_entrance2Pointer))) != IV_ERROR_SUCCESS)
    {
        delete[] buf; 
        buf = NULL;
        return res;
    }
    address += sizeof(m_entrance2Pointer);

    ///////////////
    // ZK II

    // Anzahl der Zutrittslisten ermitteln
    for (i=0, countTable=0; countTable < (sizeof(m_entrance2Pointer) / sizeof(unsigned int)); countTable++)
    {
        if (m_entrance2Pointer[countTable] == 0)
        {
            break;
        }

        // Gesamte Feldanzahl fuer Groeszenberechnung sammeln.
        i += m_entrance2Table[countTable].count;
    }
    sizeAccess = sizeof(m_entrance2Pointer);
    sizeAccess += countTable * sizeof(StructTable2);
    sizeAccess += i * sizeof(StructTable2Field);

    // Strukturen lesen
    for (i=0; i < countTable; i++)
    {
        // Tabellenstruktur uebertragen
        if ((res = getFlash(deviceIDRemote, address, (unsigned char*)&m_entrance2Table[i], sizeof(StructTable2))) != IV_ERROR_SUCCESS)
        {
            delete[] buf; 
            buf = NULL;
            return res;
        }
        address += sizeof(StructTable2);

        // Feldstrukturen lesen
        if ((res = getFlash(deviceIDRemote, address, (unsigned char*)m_entrance2TableField[i], m_entrance2Table[i].count * sizeof(StructTable2Field))) != IV_ERROR_SUCCESS)
        {
            delete[] buf; 
            buf = NULL;
            return res;
        }
        address += m_entrance2Table[i].count * sizeof(StructTable2Field);
    }


    ///////////////
    // Grunddaten

    // CRC berechnen
    crcTest = 0xFFFFFFFF;
    crcTest = crc32(m_file.m_eep, EEP_SIZE_SU, crcTest, FALSE);
    crcTest = crc32(m_file.m_configMem, sizeof(StructConfig), crcTest, FALSE);
    crcTest = crc32(dummyConfig, sizeof(dummyConfig), crcTest, FALSE);
    crcTest = crc32(m_file.m_flash, sizeSetup, crcTest, FALSE);
    crcTest = crc32(buf, sizeEK, crcTest, FALSE);

    // Speicher wieder freigeben.
    delete[] buf; 
    buf = NULL;

    // CRC der Zutrittslisten ermitteln
    crcTest = crc32((unsigned char*)m_entrance2Pointer, sizeof(m_entrance2Pointer), crcTest, FALSE);

    for (i=0; i < countTable; i++)
    {
        crcTest = crc32((unsigned char*)&m_entrance2Table[i], sizeof(StructTable2), crcTest, FALSE);

        for (countField=0; countField < m_entrance2Table[i].count; countField++)
        {
            // Beim letzten Feld der letzten Tabelle musz der CRC abgeschlossen werden.
            crcTest = crc32((unsigned char*)&m_entrance2TableField[i][countField], sizeof(StructTable2Field), crcTest, ((i == (countTable - 1)) && (countField == (m_entrance2Table[i].count - 1))));
        }
    }

    //////////////////////////////
    // Ende

    if (crc != crcTest)
    {
        trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
        return -IV_ERROR_CRC_SEGMENT;
    }

    // Segmentend: Setup uebertragen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP);

    //////////////////////////////
    // Begin
    address = size = crc = 0;
    // Segmentbegin: Setup uebertragen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED, address, size, crc);

    //////////////////////////////
    // Extended - Speicher Lesen

    if ((res = getFlash(deviceIDRemote, address, m_file.m_flashExtended, size)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += size;

    ///////////////
    // Extended - Speicher pruefen

    // Extended - Speicher
    crcTest = 0xFFFFFFFF;
    crcTest = crc32(m_file.m_flashExtended, size, crcTest, TRUE);

    if (crc != crcTest)
    {
        trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
        return -IV_ERROR_CRC_SEGMENT;
    }

    //////////////////////////////
    // Ende

    // Segmentend: Setup uebertragen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED);


    // Die Eingabekettenzeiger anpassen, da diese fuer die FW vor dem Einspielen angepasst wurden.
    // Zeigeroffsets anpassen (addieren von Offset)
    AdjustExtendedMemPointer();

    crc = 0;
    // Segmentend: Bedienung stoppen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG);

    return res;
}

int Channel4::readSetupData4_2(short deviceIDRemote)
{
    int res = IV_ERROR_SUCCESS;
    unsigned char dummyConfig[0x1C]; // Groesse nicht veraendern !!!
    // Berechnet sich aus 0x80 - sizeof(StructConfig)

    unsigned int sizeSetup = 0, sizeHeader = 0, sizeExtended = 0;

    //
    unsigned int address, size, crc, crcTest = 0;
    address = size = crc = 0;

    // Segmentbegin: Bedienung stoppen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        if ((res & 0xFF) == 1)
        {
            // Funktionsfehler Typ: IV_VALUE_SEGMENT_GET_UPDATE_BG unbekannt.
            // Wenn dass der Fall ist, wird das Auslesen noch nicht unterstuetzt.
            res = -IV_ERROR_FUNCTION_UNAVAILABLE;
        }
        else
        {
            return res;
        }
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc);

    //////////////////////////////
    // Begin

    // Segmentbegin: Setup uebertragen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc);

    // Gelieferte zur Mindestgroesse pruefen
    if (size < (EEP_SIZE_SU + sizeof(StructConfig) + sizeof(dummyConfig)))
    {
        return -IV_ERROR_PROCESSING;
    }

    //////////////////////////////
    // Grunddaten

    // EEPROM uebertragen
    if ((res = getFlash(deviceIDRemote, address, m_file.m_eep, EEP_SIZE_SU)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += EEP_SIZE_SU;
    // CONFIG uebertragen (100 Byte)
    if ((res = getFlash(deviceIDRemote, address, m_file.m_configMem, sizeof(StructConfig))) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeof(StructConfig);
    // CONFIG dummybytes (28 Byte)
    if ((res = getFlash(deviceIDRemote, address, dummyConfig, sizeof(dummyConfig))) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeof(dummyConfig);

    // Groesse der Flashdaten fuer Setup aus EepMem entnehmen
    ChangeOrderLong(&m_file.m_eep[P_FLASH_SIZE_SU]);
    sizeSetup = *(unsigned int*)&m_file.m_eep[P_FLASH_SIZE_SU];
    ChangeOrderLong(&m_file.m_eep[P_FLASH_SIZE_SU]);

    // FLASH uebertragen
    if ((res = getFlash(deviceIDRemote, address, m_file.m_flash, sizeSetup)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeSetup;

    // Groesse der Headerdaten aus EepMem entnehmen.
    sizeHeader = *(unsigned int*)&m_file.m_eep[P_HEADER_SIZE];
    // Headerdaten uebertragen
    if ((res = getFlash(deviceIDRemote, address, m_file.m_header, sizeHeader)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Pruefen ob die Headerangaben passen.
    if (!m_file.IsHeader())
        return -IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION;

    CDFSoftwarebuildVersion minVersion(m_file.GetHeader().minLibraryVersion);
    // Pruefen ob die Mindestversion der benoetigten DLL erreicht bleibt.
    if (libraryVersion < minVersion) {

        // Mindestversion DLL wird nicht eingehalten.
        return -IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION;
    }

    ///////////////
    // Grunddaten
    if (size != (EEP_SIZE_SU + sizeof(StructConfig) + sizeof(dummyConfig) + sizeSetup + sizeHeader)) {

        trace("%s(%i):ERROR(1): %s [ Size von Segment falsch: %u, %u ]\n", __FILE__, __LINE__, __func__, size, 
            (EEP_SIZE_SU + sizeof(StructConfig) + sizeof(dummyConfig) + sizeSetup + sizeHeader));
        return -IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION;;
    }

    // CRC berechnen
    crcTest = 0xFFFFFFFF;
    crcTest = crc32(m_file.m_eep, EEP_SIZE_SU, crcTest, FALSE);
    crcTest = crc32(m_file.m_configMem, sizeof(StructConfig), crcTest, FALSE);
    crcTest = crc32(dummyConfig, sizeof(dummyConfig), crcTest, FALSE);
    crcTest = crc32(m_file.m_flash, sizeSetup, crcTest, FALSE);
    crcTest = crc32(m_file.m_header, sizeHeader, crcTest, TRUE);

    //////////////////////////////
    // Ende

    if (crc != crcTest)
    {
        trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
        return -IV_ERROR_CRC_SEGMENT;
    }

    // Segmentend: Setup uebertragen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP);

    //////////////////////////////
    // Begin
    address = size = crc = 0;
    // Segmentbegin: Setup uebertragen
    if ((res = beginGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED, address, size, crc);

    //////////////////////////////
    // Extended - Speicher Lesen

    // Groesse des Extended Speichers aus eeprom ermitteln.
    sizeExtended = *(unsigned int*)&m_file.m_eep[P_EXTENDED_SIZE];

    if ((res = getFlash(deviceIDRemote, address, m_file.m_flashExtended, sizeExtended)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    address += sizeExtended;

    ///////////////
    // Extended - Speicher pruefen
    if (size != sizeExtended) {

        trace("%s(%i):ERROR(1): %s [ Size von Segment falsch: %u, %u ]\n", __FILE__, __LINE__, __func__, size, sizeExtended);
        return -IV_ERROR_SETUPFILE_MIN_LIBRARY_VERSION;
    }

    // Extended - Speicher
    crcTest = 0xFFFFFFFF;
    crcTest = crc32(m_file.m_flashExtended, size, crcTest, TRUE);

    if (crc != crcTest)
    {
        trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
        return -IV_ERROR_CRC_SEGMENT;
    }

    //////////////////////////////
    // Ende

    // Segmentend: Setup uebertragen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP_EXTENDED);


    // Die Eingabekettenzeiger anpassen, da diese fuer die FW vor dem Einspielen angepasst wurden.
    // Zeigeroffsets anpassen (addieren von Offset)
    AdjustExtendedMemPointer();

    crc = 0;
    // Segmentend: Bedienung stoppen
    if ((res = endGetSegment(deviceIDRemote, IV_VALUE_SEGMENT_GET_UPDATE_BG)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG);

    return res;
}

int Channel4::writeListWithTimeboySlot( short deviceIdRemote )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = writeListData( deviceIdRemote );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::writeListData( short deviceIdRemote )
{
    int i;
    // Listengesamtgroesse berechnen und ggf. sortieren.
    unsigned int listSize = MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned int);
    unsigned int listPointer[MAX_DATENSATZ_BESCHREIBUNG];
    for (i=0; i < MAX_DATENSATZ_BESCHREIBUNG; i++)
    {
        if (m_listSize[i])
        {			
            listPointer[i] = listSize + FLASH_LIST_START;
            ChangeOrderLong((unsigned char*)&listPointer[i]);

            listSize += m_listSize[i];
        }
        else
        {
            listPointer[i] = 0;
        }
    }

    if (listSize == (MAX_DATENSATZ_BESCHREIBUNG * sizeof(unsigned int)))
    {
        return -IV_ERROR_LIST_PROCESSING;
        return 0; // keine Listen vorhanden
    }

    int res = writeListData4( deviceIdRemote );

    if ((res > 0) && (((res >> 8) & 0xFF) == 3))
        return -IV_ERROR_NOT_ENOUGH_FLASH; // Es steht nicht genuegend freier Flash Speicher zur Verfuegung.

    return res;
}

// Fuer neue Listentechnik
int Channel4::writeListData4(short deviceIDRemote)
{
    unsigned char tableNum;
    int res = IV_ERROR_SUCCESS;

    StructTable2 table;
    char name[32];

    //
    unsigned short segmentUpdateVariation = IV_VALUE_SEGMENT_UPDATE_BG;
    unsigned int address, size, crc;
    address = size = crc = 0;

    // Systemstopp Listen uebertragen
    while ((res = beginSetSegment(deviceIDRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        // Wenn das Hintergrunduebertragen nicht unterstuetzt wird, dann die Standardmethode verwenden.
        if (((res & 0xFF) == 1) && (segmentUpdateVariation == IV_VALUE_SEGMENT_UPDATE_BG))
        {
            segmentUpdateVariation = IV_VALUE_SEGMENT_UPDATE;
            continue;
        }

        return res;
    }

    // Pruefen ob Streaming moeglich.
    clearStreamTransfer();
    // Pruefen ob die Uebertragung per Stream moeglich ist.
    if ((res = writeStream(deviceIDRemote, update_bg, NULL, 0)) == IV_ERROR_SUCCESS) 
        setStreamTransfer(true);
    else
        setStreamTransfer(false);

    // Listendaten Tabellenweise uebertragen
    for (tableNum=0; tableNum < MAX_DATENSATZ_BESCHREIBUNG; tableNum++)
    {
        // Nicht importierte Listen uebergehen
        if (m_listSize[tableNum] == 0) continue;

        // Bei Streaming werden die inneren Bloecke wegelassen.
        if (!isStreamTransfer()) {

            // Maximale Segmentanzahl nicht ueberschreiten
            if (tableNum && !(tableNum % 16))
            {
                crc = 0;
                // Systemstopp Listen uebertragen
                if ((res = endSetSegment(deviceIDRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
                {
                    return res;
                }

                // Systemstopp Listen uebertragen
                if ((res = beginSetSegment(deviceIDRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
                {
                    return res;
                }
            }
        }
        memset(name, 0, sizeof(name));
        sprintf(name, "LIST.%i", tableNum);
        // Zur Liste Informationen ermitteln
        if ((res = getTable(deviceIDRemote, name, table)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        strcpy(name, table.name);
        sprintf(table.name, "LIST.%s", name);

        // Muss die Liste auf Sortierung geprueft und ggf. sortiert werden.
        if (m_listSortCheck[tableNum] == false)
        {
            // Listen ohne Datensaetze nicht sortieren.
            // ACHTUNG: Ansonsten Division durch 0 bei Pruefung der Datensatzgroesse !!!
            if ( *reinterpret_cast<unsigned int*>(&m_pucListMem[tableNum][0]) )
            {
                int fieldOffs = 0;

                if (table.flags & TF_SORT && table.sort < table.count)
                {
                    char name[64];
                    int fieldNum;
                    StructTable2Field field;
                    memset(&field, 0, sizeof(StructTable2Field));
                    m_ucListRecordSize[tableNum] = 0;
                    // Alle Felder fuer Offset lesen
                    for (fieldNum=0; fieldNum < table.count; fieldNum++)
                    {
                        if (fieldNum < (table.sort + 1))
                        {
                            // Datensatzoffset fuer Sortierfeld aufsummieren.
                            fieldOffs += field.size;
                        }

                        sprintf(name, "LIST.%i.%i", tableNum, fieldNum);

                        if ((res = getField(deviceIDRemote, name, field)) != IV_ERROR_SUCCESS)
                        {
                            if (res < 0)
                            {
                                // Kommunikationsfehler
                                return res;
                            }
                            else
                            {
                                // Funktionsfehler
                                return res;
                            }
                        }

                        // Datensatzgroesse fuer Swap-Routine berechnen
                        m_ucListRecordSize[tableNum] += field.size;
                    }
                
                    // Die Listensatzgröße muss mit der importierten übereinstimmen.
                    if ( ((m_listSize[tableNum] - sizeof(unsigned int)) / *(unsigned int*)&m_pucListMem[tableNum][0]) != m_ucListRecordSize[tableNum] )
                    {
                        trace("%s(%i):ERROR(1): %s [ Listensatzgroesse (%i) stimmt nicht mit der im Setup definierten ueberein. ]\n", __FILE__, __LINE__, __func__, tableNum);
                        return -IV_ERROR_LIST_PROCESSING;
                    }
                
                    // Liste sortieren.
                    SortList(m_pucListMem[tableNum], m_ucListRecordSize[tableNum], fieldOffs);

                    // Bei allen Listen ist 0 die sortierte Spalte
                    // Flag dafuer dass die Liste sortiert wurde setzen.
                    table.flags |= TF_SORTED;
                }
            }

            // Sortiert oder zumindes geprueft.
            m_listSortCheck[tableNum] = true;
        }

        // CRC berechnen
        crc = 0xFFFFFFFF;
        crc = crc32((unsigned char*)&table, sizeof(StructTable2), crc, FALSE);
        crc = crc32(m_pucListMem[tableNum], m_listSize[tableNum], crc);

        // Groesze berechnen
        size = sizeof(StructTable2) + m_listSize[tableNum];

        // Bei Streaming werden die inneren Bloecke wegelassen.
        if (!isStreamTransfer()) {

            // Systemstopp Liste uebertragen
            if ((res = beginSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_LIST, address, size, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }

        // Daten uebertragen
        if ((res = setFlash(deviceIDRemote, address, (unsigned char*)&table, sizeof(StructTable2))) != IV_ERROR_SUCCESS)
        {
            // Speicherplatz reicht nicht aus.
            if (res == 3) { res = -IV_ERROR_NOT_ENOUGH_FLASH; }
            return res;
        }
        // Daten uebertragen
        address += sizeof(StructTable2);
        if ((res = setFlash(deviceIDRemote, address, m_pucListMem[tableNum], m_listSize[tableNum])) != IV_ERROR_SUCCESS)
        {
            // Speicherplatz reicht nicht aus.
            if (res == 3) { res = -IV_ERROR_NOT_ENOUGH_FLASH; }
            return res;
        }

        // Im Modus Streaming ?, dann gepufferte Daten senden.
        if (isStreamTransfer() && m_streamBuffer.size()) {
            if ((res = writeStream(deviceIDRemote, setup_list, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
                // Gepufferte Daten loeschen.
                clearStreamTransfer();
                return res;
            }
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
        }

        // Bei Streaming werden die inneren Bloecke wegelassen.
        if (!isStreamTransfer()) {

            // Systemstopp Liste uebertragen
            if ((res = endSetSegment(deviceIDRemote, IV_VALUE_SEGMENT_LIST, crc)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
    }

    crc = 0;
    // Systemstopp Liste uebertragen
    if ((res = endSetSegment(deviceIDRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    return res;
}

int Channel4::writeEntrance2ListWithTimeboySlot( short deviceIdRemote, unsigned char listNum )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        // Listendaten zum AEIV uebertragen
        res = writeEntrance2ListData4( deviceIdRemote, listNum );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

// ListenBuffer in AE laden
int Channel4::writeEntrance2ListData4(short deviceIdRemote, unsigned char listNum)
{
    // Alle Listen laden?
    if (listNum == 0xFF)
    {
        // Mindestens eine Liste geladen?
        int i;
        for (i=0; i < 8; i++)
        {
            if (m_ulEntrance2ListSize[i] != 0)
            {
                break;
            }
        }
        if (i == 8)
        {
            return -IV_ERROR_LIST_PROCESSING; // keine Listen vorhanden
        }
    }
    else
    {
        if (m_ulEntrance2ListSize[listNum] == 0)
        {
            return -IV_ERROR_LIST_PROCESSING; // keine Listen vorhanden
        }
    }

    unsigned short segmentUpdateVariation = IV_VALUE_SEGMENT_UPDATE_BG;
    int res = IV_ERROR_SUCCESS;
    unsigned int address, size, crc;
    address = size = crc = 0;

    // Systemstopp Setup uebertragen
    while ((res = beginSetSegment(deviceIdRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        // Wenn das Hintergrunduebertragen nicht unterstuetzt wird, dann die Standardmethode verwenden.
        if (((res & 0xFF) == 1) && (segmentUpdateVariation == IV_VALUE_SEGMENT_UPDATE_BG))
        {
            segmentUpdateVariation = IV_VALUE_SEGMENT_UPDATE;
            continue;
        }

        return res;
    }

    // Pruefen ob Streaming moeglich.
    clearStreamTransfer();
    // Pruefen ob die Uebertragung per Stream moeglich ist.
    if ((res = writeStream(deviceIdRemote, update_bg, NULL, 0)) == IV_ERROR_SUCCESS) 
        setStreamTransfer(true);
    else
        setStreamTransfer(false);

    switch (listNum)
    {
    case 0xFF:
    case 0x00:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[0] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 0)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        if (listNum != 0xFF) break;
    case 0x01:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[1] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 1)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        if (listNum != 0xFF) break;
    case 0x02:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[2] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 2)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        if (listNum != 0xFF) break;
    case 0x03:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[3] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 3)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        if (listNum != 0xFF) break;
    case 0x04:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[4] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 4)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        if (listNum != 0xFF) break;
    case 0x05:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[5] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 5)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
        if (listNum != 0xFF) break;
    case 0x06:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[6] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 6)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
    case 0x07:
        // nur geladene Liste bei 0xFF uebertragen
        if (m_ulEntrance2ListSize[7] != 0)
        {
            // Liste uebertragen
            if ((res = writeEntrance2List4(deviceIdRemote, 7)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
    }

    crc = 0;
    // Systemstopp Setup uebertragen
    if ((res = endSetSegment(deviceIdRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    if ((res > 0) && (((res >> 8) & 0xFF) == 3))
    {
        return -IV_ERROR_NOT_ENOUGH_FLASH; // Es steht nicht genuegend freier Flash Speicher zur Verfuegung.
    }

    return res;
}

// ListenBuffer in AE laden
int Channel4::writeEntrance2List4(short deviceIDRemote, unsigned char listNum)
{
    int res = IV_ERROR_SUCCESS;
    StructTable2 table;
    char name[32];
    memset(name, 0, sizeof(name));
    sprintf(name, "ACCESS.%i", listNum);
    // Zur Liste Informationen ermitteln
    if ((res = getTable(deviceIDRemote, name, table)) != IV_ERROR_SUCCESS)
    {
        return res;
    }
    strcpy(name, table.name);
    sprintf(table.name, "ACCESS.%s", name);

    // Muss die Liste auf Sortierung geprueft und ggf. sortiert werden.
    if (m_entrance2SortCheck[listNum] == false)
    {		
        int fieldOffs = 0;

        if (table.flags & TF_SORT && table.sort < table.count)
        {			
            //char name[64];
            int fieldNum;
            StructTable2Field field;
            memset(&field, 0, sizeof(StructTable2Field));
            // Alle Felder fuer Offset lesen
            for (fieldNum=0; fieldNum < table.count; fieldNum++)
            {				
                if (fieldNum < (table.sort + 1))
                {					
                    // Datensatzoffset fuer Sortierfeld aufsummieren.
                    fieldOffs += field.size;
                }
                else // Da die Datensatzgroesse bekannt ist, brauchen die restlichen Felder nicht ermittelt zu werden. 
                { 
                    break; 
                }

                field = m_entrance2TableField[listNum][fieldNum];

                // Datensatzgroesse wird fuer Swap-Routine benoetigt!
            }

            // Liste sortieren.
            SortList(m_pucEntrance2ListMem[listNum], m_ucEntrance2RecordSize[listNum], fieldOffs);

            // Bei allen Listen ist 0 die sortierte Spalte
            // Flag dafuer dass die Liste sortiert wurde setzen.
            table.flags |= TF_SORTED;
        }

        // Sortiert oder zumindes geprueft.
        m_entrance2SortCheck[listNum] = true;
    }


    //
    unsigned short art = IV_VALUE_SEGMENT_LIST_ACCESS;
    unsigned int address, size, crc;
    address = size = crc = 0;
    /*
    // Systemstopp Setup uebertragen
    if ((res = SegmentBegin(deviceIDRemote, SS_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
    return res;
    }
    */
    // CRC berechnen
    crc = 0xFFFFFFFF;
    crc = crc32((unsigned char*)&table, sizeof(StructTable2), crc, FALSE);
    crc = crc32(m_pucEntrance2ListMem[listNum], m_ulEntrance2ListSize[listNum], crc);

    // Groesze berechnen
    size = sizeof(StructTable2) + m_ulEntrance2ListSize[listNum];

    // Bei Streaming werden die inneren Bloecke wegelassen.
    if (!isStreamTransfer()) {
        
        // Systemstopp List uebertragen
        while ((res = beginSetSegment(deviceIDRemote, art, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            // Typ unbekannt?, dann als "normale" Liste versuchen..
            if (((res & 0xFF) == 1) && (art == IV_VALUE_SEGMENT_LIST_ACCESS))
            {
                art = IV_VALUE_SEGMENT_LIST;
                continue;
            }

            return res;
        }
    }

    // Daten uebertragen
    if ((res = setFlash(deviceIDRemote, address, (unsigned char*)&table, sizeof(StructTable2))) != IV_ERROR_SUCCESS)
    {
        // Speicherplatz reicht nicht aus.
        if (res == 3) { res = -IV_ERROR_NOT_ENOUGH_FLASH; }
        return res;
    }
    // Daten uebertragen
    address += sizeof(StructTable2);
    if ((res = setFlash(deviceIDRemote, address, m_pucEntrance2ListMem[listNum], m_ulEntrance2ListSize[listNum])) != IV_ERROR_SUCCESS)
    {
        // Speicherplatz reicht nicht aus.
        if (res == 3) { res = -IV_ERROR_NOT_ENOUGH_FLASH; }
        return res;
    }

    // Im Modus Streaming ?, dann gepufferte Daten senden.
    if (isStreamTransfer() && m_streamBuffer.size()) {
        if ((res = writeStream(deviceIDRemote, list_access, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
            return res;
        }
        // Gepufferte Daten loeschen.
        clearStreamTransfer();
    }

    // Bei Streaming werden die inneren Bloecke wegelassen.
    if (!isStreamTransfer()) {

        // Systemstopp Setup uebertragen
        if ((res = endSetSegment(deviceIDRemote, art, crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
    }
    /*
    crc = 0;
    // Systemstopp Setup uebertragen
    if ((res = SegmentEnd(deviceIDRemote, SS_UPDATE, crc)) != IV_ERROR_SUCCESS) {

    return res;
    }
    */
    return res;
}

// ListenBuffer in AE laden
int Channel4::writeTimeboyListData4(short deviceIDRemote, unsigned char id, unsigned char num, int reserve)
{
    // Alle Listen laden?
    if (id == 0xFF)
    {
        int groupID, listNum, found = 0;
        for (groupID=0; groupID < MAX_DATENSATZ_BESCHREIBUNG; groupID++)
        {			
            if (num == 0xFF)
            {
                // Mindestens eine Liste geladen?
                for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
                {
                    if (m_ulTimeboyListSize[groupID][listNum] != 0)
                    {
                        found = 1;
                        break;
                    }
                }
                if (found)
                {
                    break;
                }
            }
            else
            {
                // Mindestens eine Liste geladen?
                if (m_ulTimeboyListSize[groupID][num] == 0)
                {
                    found = 1;
                    break;
                }
            }
        }
        if (found == 0)
        {
            return -IV_ERROR_LIST_PROCESSING; // keine Listen vorhanden
        }
    }
    else
    {
        if (num == 0xFF)
        {
            int listNum, found = 0;
            // Mindestens eine Liste geladen?
            for (listNum=0; listNum < MAX_DATENSATZ_BESCHREIBUNG; listNum++)
            {
                if (m_ulTimeboyListSize[id][listNum] != 0)
                {
                    found = 1;
                    break;
                }
            }
            if (found == 0)
            {
                return -IV_ERROR_LIST_PROCESSING; // keine Listen vorhanden
            }
        }
        else
        {
            if (m_ulTimeboyListSize[id][num] == 0)
            {
                return -IV_ERROR_LIST_PROCESSING; // keine Listen vorhanden
            }
        }
    }

    unsigned short segmentUpdateVariation = IV_VALUE_SEGMENT_UPDATE_BG;
    int res = IV_ERROR_SUCCESS;
    unsigned int address, size, crc;
    address = size = crc = 0;

    // Systemstopp Listen uebertragen
    while ((res = beginSetSegment(deviceIDRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        // Wenn das Hintergrunduebertragen nicht unterstuetzt wird, dann die Standardmethode verwenden.
        if (((res & 0xFF) == 1) && (segmentUpdateVariation == IV_VALUE_SEGMENT_UPDATE_BG))
        {
            segmentUpdateVariation = IV_VALUE_SEGMENT_UPDATE;
            continue;
        }

        return res;
    }

    // Pruefen ob Streaming moeglich.
    clearStreamTransfer();
    // Pruefen ob die Uebertragung per Stream moeglich ist.
    if ((res = writeStream(deviceIDRemote, update_bg, NULL, 0)) == IV_ERROR_SUCCESS) 
        setStreamTransfer(true);
    else
        setStreamTransfer(false);

    // Alle ids abarbeiten?
    if (id == 0xFF)
    {
        int tableNum, tableCount = 0;
        int groupID;
        for (groupID=0; groupID < MAX_DATENSATZ_BESCHREIBUNG; groupID++)
        {
            if (num == 0xFF)
            {
                for (tableNum=0; tableNum < MAX_DATENSATZ_BESCHREIBUNG; tableNum++)
                {
                    // nur geladene Liste bei 0xFF uebertragen
                    if (m_ulTimeboyListSize[groupID][tableNum] != 0)
                    {
                        // Maximale Segmentanzahl nicht ueberschreiten
                        if (tableCount && !(tableCount % 16))
                        {
                            crc = 0;
                            // Systemstopp Listen uebertragen
                            if ((res = endSetSegment(deviceIDRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
                            {
                                return res;
                            }

                            // Systemstopp Listen uebertragen
                            if ((res = beginSetSegment(deviceIDRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
                            {
                                return res;
                            }
                        }

                        // Liste uebertragen
                        if ((res = writeTimeboyList4(deviceIDRemote, groupID, tableNum, reserve)) != IV_ERROR_SUCCESS)
                        {
                            return res;
                        }

                        tableCount++;
                    }
                }
            }
            else
            {
                // nur geladene Liste bei 0xFF uebertragen
                if (m_ulTimeboyListSize[groupID][num] != 0)
                {
                    // Maximale Segmentanzahl nicht ueberschreiten
                    if (tableCount && !(tableCount % 16))
                    {
                        crc = 0;
                        // Systemstopp Listen uebertragen
                        if ((res = endSetSegment(deviceIDRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
                        {
                            return res;
                        }

                        // Systemstopp Listen uebertragen
                        if ((res = beginSetSegment(deviceIDRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
                        {
                            return res;
                        }
                    }

                    // Liste uebertragen
                    if ((res = writeTimeboyList4(deviceIDRemote, groupID, num, reserve)) != IV_ERROR_SUCCESS)
                    {
                        return res;
                    }

                    tableCount++;
                }
            }
        }
    }
    else
    {
        if (num == 0xFF)
        {
            int tableNum, tableCount = 0;
            for (tableNum=0; tableNum < MAX_DATENSATZ_BESCHREIBUNG; tableNum++)
            {
                // nur geladene Liste bei 0xFF uebertragen
                if (m_ulTimeboyListSize[id][tableNum] != 0)
                {
                    // Maximale Segmentanzahl nicht ueberschreiten
                    if (tableCount && !(tableCount % 16))
                    {
                        crc = 0;
                        // Systemstopp Listen uebertragen
                        if ((res = endSetSegment(deviceIDRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
                        {
                            return res;
                        }

                        // Systemstopp Listen uebertragen
                        if ((res = beginSetSegment(deviceIDRemote, segmentUpdateVariation, address, size, crc)) != IV_ERROR_SUCCESS)
                        {
                            return res;
                        }
                    }

                    // Liste uebertragen
                    if ((res = writeTimeboyList4(deviceIDRemote, id, tableNum, reserve)) != IV_ERROR_SUCCESS)
                    {
                        return res;
                    }

                    tableCount++;
                }
            }
        }
        else
        {
            // Liste uebertragen
            if ((res = writeTimeboyList4(deviceIDRemote, id, num, reserve)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
    }

    crc = 0;
    // Systemstopp Listen uebertragen
    if ((res = endSetSegment(deviceIDRemote, segmentUpdateVariation, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    return res;
}

int Channel4::writeTimeboyListWithTimeboySlot( short deviceIdRemote, unsigned char id, unsigned char num, int reserve )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        // Listendaten zum AEIV uebertragen
        res = writeTimeboyListData4( deviceIdRemote, id, num, reserve );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

// ListenBuffer in AE laden
int Channel4::writeTimeboyList4(short deviceIDRemote, unsigned char id, unsigned char num, int reserve)
{
    (void)reserve;
    int res = IV_ERROR_SUCCESS;
    StructTable2 table;
    memset(&table, 0, sizeof(StructTable2));
    table.num = num;
    sprintf(table.name, "TBLIST.%i.%i", id, num);
    table.flags = m_timeboyFlags[id][num];

    //
    unsigned short art = IV_VALUE_SEGMENT_LIST_TIMEBOY;
    unsigned int address, size, crc;
    address = size = crc = 0;
    /*
    // Systemstopp Setup uebertragen
    if ((res = SegmentBegin(deviceIDRemote, SS_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
    return res;
    }
    */
    // CRC berechnen
    crc = 0xFFFFFFFF;
    crc = crc32((unsigned char*)&table, sizeof(StructTable2), crc, FALSE);
    crc = crc32(m_pucTimeboyListMem[id][num], m_ulTimeboyListSize[id][num], crc);

    // Groesze berechnen
    size = sizeof(StructTable2) + m_ulTimeboyListSize[id][num];

    // Bei Streaming werden die inneren Bloecke wegelassen.
    if (!isStreamTransfer()) {

        // Systemstopp List uebertragen
        while ((res = beginSetSegment(deviceIDRemote, art, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            // Typ unbekannt?, dann als "normale" Liste versuchen..
            if (((res & 0xFF) == 1) && (art == IV_VALUE_SEGMENT_LIST_ACCESS))
            {
                art = IV_VALUE_SEGMENT_LIST;
                continue;
            }

            return res;
        }
    }

    // Daten uebertragen
    if ((res = setFlash(deviceIDRemote, address, (unsigned char*)&table, sizeof(StructTable2))) != IV_ERROR_SUCCESS)
    {
        // Speicherplatz reicht nicht aus.
        if (res == 3) { res = -IV_ERROR_NOT_ENOUGH_FLASH; }
        return res;
    }
    // Daten uebertragen
    address += sizeof(StructTable2);
    if ((res = setFlash(deviceIDRemote, address, m_pucTimeboyListMem[id][num], m_ulTimeboyListSize[id][num])) != IV_ERROR_SUCCESS)
    {
        // Speicherplatz reicht nicht aus.
        if (res == 3) { res = -IV_ERROR_NOT_ENOUGH_FLASH; }
        return res;
    }

    // Im Modus Streaming ?, dann gepufferte Daten senden.
    if (isStreamTransfer() && m_streamBuffer.size()) {
        if ((res = writeStream(deviceIDRemote, list_timeboy, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
            return res;
        }
        // Gepufferte Daten loeschen.
        clearStreamTransfer();
    }

    // Bei Streaming werden die inneren Bloecke wegelassen.
    if (!isStreamTransfer()) {

        // Systemstopp Setup uebertragen
        if ((res = endSetSegment(deviceIDRemote, art, crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
    }

    if ((res > 0) && (((res >> 8) & 0xFF) == 3))
        return -IV_ERROR_NOT_ENOUGH_FLASH; // Es steht nicht genuegend freier Flash Speicher zur Verfuegung.

    /*
    crc = 0;
    // Systemstopp Setup uebertragen
    if ((res = SegmentEnd(deviceIdRemote, SS_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
    return res;
    }
    */
    return res;
}

int Channel4::writeDeviceWithTimeboySlot( short deviceIdRemote, char *filePath )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = writeDevice( deviceIdRemote, filePath );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::writeDevice( short deviceIdRemote, char *szFileName )
{
    // Hardware- und Softwareinfo der ausgewaehlten Datei.
    StructHardwareInfo hwInfo;

    int iRes = IV_ERROR_SUCCESS;

    // Puffer fuer die Hardwareinfo in Textform
    _info.clear();
    // Strukturen loeschen.
    memset(&hwInfo, 0, sizeof(StructHardwareInfo));


    //#########################################################################
    // Pruefen ob gewaehlte Datei in das Geraet eingespielt werden kann.
    // Dateityp ermitteln
    struct stat st;
    if (stat(szFileName, &st) == -1) 
    {	
        return -IV_ERROR_FILE_OPEN;
        return 0;
    }

    // Angabe ist eine Datei
    if (S_ISREG(st.st_mode)) 
    {	
        // Nur ".hex" gueltig.
        size_t pos = strlen(szFileName) - 4;
#ifdef _MSC_VER
        if (szFileName[pos] != '.' || 
            ((!(szFileName[pos + 1] == 'h' || szFileName[pos + 1] == 'H') ||
            !(szFileName[pos + 2] == 'e' || szFileName[pos + 2] == 'E') ||
            !(szFileName[pos + 3] == 'x' || szFileName[pos + 3] == 'X')) &&
            (!(szFileName[pos + 1] == 'd' || szFileName[pos + 1] == 'D') ||
            !(szFileName[pos + 2] == 'f' || szFileName[pos + 2] == 'F') ||
            !(szFileName[pos + 3] == 'z' || szFileName[pos + 3] == 'Z')) &&
            (!(szFileName[pos + 1] == 'z' || szFileName[pos + 1] == 'Z') ||
            !(szFileName[pos + 2] == 'i' || szFileName[pos + 2] == 'I') ||
            !(szFileName[pos + 3] == 'p' || szFileName[pos + 3] == 'P'))) ) 
        {	
            return -IV_ERROR_FILE_EXTENSION;
            return 0;
        }
#else 
        if (!S_ISREG(st.st_mode) || szFileName[pos] != '.' || 
            !(szFileName[pos + 1] == 'h' || szFileName[pos + 1] == 'H') ||
            !(szFileName[pos + 2] == 'e' || szFileName[pos + 2] == 'E') ||
            !(szFileName[pos + 3] == 'x' || szFileName[pos + 3] == 'X') ) 
        {
            return -IV_ERROR_FILE_EXTENSION;
            return 0;
        }
#endif	
        // Handelt es sich um eine Hexdatei
        if (szFileName[pos + 1] == 'h' || szFileName[pos + 1] == 'H') 
        {	
            // Dateidaten laden
            m_fileDevice.reset();
            if (!m_fileDevice.load(szFileName)) 
            {	
                int err = IV_ERROR_SUCCESS;
                if (m_fileDevice.errorState() == FileIntelHex::Version || m_fileDevice.errorState() == FileIntelHex::DeviceUnknown) 
                {
                    err = -IV_ERROR_VERSION_DEVICEFILE;
                } 
                else if (m_fileDevice.errorState() == FileIntelHex::FileOpen) 
                {
                    err = -IV_ERROR_FILE_OPEN;
                } 
                else 
                {
                    err = -IV_ERROR_DEVICEFILE_LOAD;
                }

                m_fileDevice.reset();
                return err;
            }

            // Ist die Softwareinformation in der Geraetedatei verfuegbar?
            if (m_fileDevice.GetHeaderSoftwareInfo().size) 
            {	
                // Auslesen der Hardwareinfo
                iRes = Channel4ModuleTable::getHardwareInfo(deviceIdRemote, hwInfo);
                if (iRes != IV_ERROR_SUCCESS)
                {
                    return iRes;

                } // Hardwareinfo

                // Liegt keine Hardwareinfo vor?
                // Verwendung der ZK-Option ueber Vorhandensein der Readerliste ermitteln.
                if (hwInfo.version == 0) 
                {	
                    int handle;
                    if (tableOpen(deviceIdRemote, (char*)"ACCESS.READER", 0, handle, (char*)"") == IV_ERROR_SUCCESS &&
                        tableClose(deviceIdRemote, handle) == IV_ERROR_SUCCESS) 
                    {	
                        // ZK-Option in Verwendung.
                        hwInfo.administrationFlags |= HIAF_HAS_ZK;
                    }
                }

                // Pruefen ob die Datei in die vorliegende Hardware eingespielt werden kann.
                if (0 == m_fileDevice.CompareInfos(hwInfo, m_fileDevice.GetHeaderSoftwareInfo())) 
                {	
                    // Hardwareinfo in Text aufloesen
                    getInfos(hwInfo, _info);
                    m_fileDevice.setErrorTextExtended( m_fileDevice.errorText() );
                    return -IV_ERROR_DEVICEFILE_MATCH;
                    return 0;
                }

            } 
            else 
            {	
                unsigned char deviceType = 255, boardType = 255;
                // Geraetetyp ermitteln
                if ((iRes = getTypeDevice(deviceIdRemote, deviceType)) != IV_ERROR_SUCCESS || (iRes = getTypeBoard(deviceIdRemote, boardType)) != IV_ERROR_SUCCESS) 
                {	
                    if (iRes == -IV_ERROR_FUNCTION_UNAVAILABLE)
                    {
                        // Der Geraetetyp wird manuell gesetzt.
                        deviceType = 0;
                    }
                    else
                    {
                        return iRes;
                    }
                }

                //#######################################################################
                // Wenn die SoftwareInfo fehlt kann die Datei keine Mulitple sein!!!

                switch (deviceType) 
                {
                case IV_VALUE_TYPE_DEVICE_PZE: // PZE-MasterIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_PZE) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das PZE-MasterIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_ZK: // ZK-MasterIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_ZK) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das ZK-MasterIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_BDE: // BDE-MasterIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_BDE) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das BDE-MasterIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_MOBIL: // Mobil-MasterIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_MOBIL) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Mobil-MasterIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_MDE: // MDE-BoxIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_MDE) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das MDE-BoxIV
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_AE: // AE-MasterIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_AE) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das AE-MasterIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_TIMEBOY: // TimeboyIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_TIMEBOY) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das TimeboyIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_EXKLUSIV: // Exklusiv-LineIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_EXKLUSIV) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Exklusiv-LineIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_FLEX: // Flex-MasterIV
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_FLEX) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Flex-MasterIV.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE: // Timeboy-Mobil-PZE
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_TIMEBOY_MOBIL_PZE) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Timeboy-Mobil-PZE.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_TERMINAL_2_8: // Terminal 2.8
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_TERMINAL_2_8) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Terminal 2.8.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_TERMINAL_4_3: // Terminal 4.3
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_TERMINAL_4_3) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Terminal 4.3.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7: // IPC-Extension
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_IPC_EXTENSION_VARIO_5_7) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das IPC-Extension.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10: // IPC-Extension
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_IPC_EXTENSION_VARIO_10) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das IPC-Extension.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7: // IPC-Extension
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_IPC_EXTENSION_Q7) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das IPC-Extension.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_IO_BOX: // IO-Box
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_IO_BOX) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das IO-Box.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_ZK_BOX: // ZK-Box
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_ZK_BOX) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das ZK-Box.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                case IV_VALUE_TYPE_DEVICE_MOBIL_BOX: // Mobil-Box
                    if (m_fileDevice.GetHeaderVersionFile() != IV_DFFILEDEVICE_VERSION_MOBIL_BOX) 
                    {
                        // Die gewaehlte Geraetedatei ist NICHT fuer das Mobil-Box.
                        return -IV_ERROR_DEVICEFILE_WRONG;
                    }
                    break;

                default:
                    if ( m_fileDevice.GetHeaderVersionFile() == IV_DFFILEDEVICE_VERSION_SOFTWARE_INFO 
                        && (m_fileDevice.GetHeaderSoftwareInfo().deviceExtended & (1 << deviceType)) != 0 )
                    {
                        // Die Daten sind für den vorliegenden Gerätetyp geeignet.
                    }
                    else
                    {
                        if ( m_fileDevice.GetHeaderVersionFile() == IV_DFFILEDEVICE_VERSION_SOFTWARE_INFO )
                            // Die gewaehlte Geraetedatei ist NICHT fuer diese Gerätevariante.
                                return -IV_ERROR_DEVICEFILE_WRONG;
                        else
                            // Die Setupanwendung ist nicht aktuell genug fuer diesen Geraetetyp.
                            return -IV_ERROR_DEVICE_UNKNOWN;
                    }
                }

                string version;
                if (boardType < IV_VALUE_TYPE_BOARD_3_0)
                {
                    // Handelt es sich um einen Boardtyp < 3 muss die Firmwareversion 04.00.xx oder 04.01.xx sein.
                    if (m_fileDevice.GetHeaderVersionFirmware(version)) 
                    {
                        if ( version.compare( 0, 6, "04.00." ) != 0 && version.compare( 0, 6, "04.01." ) != 0 )
                        {
                            return -IV_ERROR_DEVICEFILE_MATCH;
                        }
                    }
                }
                else
                {
                    // Handelt es sich um einen Boardtyp >= 3 darf die Firmwareversion nicht 04.00.xx oder 04.01.xx sein.
                    if (m_fileDevice.GetHeaderVersionFirmware(version)) 
                    {
                        if (version.compare( 0, 6, "04.00." ) == 0 || version.compare( 0, 6, "04.01." ) == 0 )
                        {
                            return -IV_ERROR_DEVICEFILE_MATCH;
                        }
                    }
                }

            }

        } 
        else 
        { // Es handelt sich um eine Zipdatei

            // Auslesen der Hardwareinfo
            if ((iRes = Channel4ModuleTable::getHardwareInfo(deviceIdRemote, hwInfo)) != IV_ERROR_SUCCESS) 
            {
                return iRes;
            }

            // Liegt keine Hardwareinfo vor?
            // Verwendung der ZK-Option ueber Vorhandensein der Readerliste ermitteln.
            if (hwInfo.version == 0) 
            {
                int handle;
                if (tableOpen(deviceIdRemote, (char*)"ACCESS.READER", 0, handle, (char*)"") == IV_ERROR_SUCCESS &&
                    tableClose(deviceIdRemote, handle) == IV_ERROR_SUCCESS) 
                {
                    // ZK-Option in Verwendung.
                    hwInfo.administrationFlags |= HIAF_HAS_ZK;
                }
            }

            // Pruefen ob die Datei in die vorliegende Hardware eingespielt werden kann.
            if (0 == m_fileDevice.SearchInfosZip(szFileName, hwInfo)) 
            {
                int err = IV_ERROR_SUCCESS;
                // Hardwareinfo in Text aufloesen
                getInfos(hwInfo, _info);
                if (m_fileDevice.errorState() == FileIntelHex::Version || m_fileDevice.errorState() == FileIntelHex::DeviceUnknown) 
                {
                    err = -IV_ERROR_VERSION_DEVICEFILE;
                } 
                else if (m_fileDevice.errorState() == FileIntelHex::NoFileMatch) 
                {
                    err = -IV_ERROR_DEVICEFILE_MATCH;
                } 
                else 
                {
                    err = -IV_ERROR_DEVICEFILE_LOAD;
                }
                return err;
            }
        }	
    } 
    else if (S_ISDIR(st.st_mode)) 
    {		
#ifdef _MSC_VER
        // Auswahl aus Verzeichnis nur unter Linux gueltig!
        return -IV_ERROR_FUNCTION_PARAMETER;
#else
        // Auslesen der Hardwareinfo
        if ((iRes = Channel4ModuleTable::getHardwareInfo(deviceIdRemote, hwInfo)) != IV_ERROR_SUCCESS) 
        {
            return iRes;
        }

        // Liegt keine Hardwareinfo vor?
        // Verwendung der ZK-Option ueber Vorhandensein der Readerliste ermitteln.
        if (hwInfo.version == 0) 
        {
            int handle;
            if (tableOpen(deviceIdRemote, (char*)"ACCESS.READER", 0, handle, (char*)"") == IV_ERROR_SUCCESS &&
                tableClose(deviceIdRemote, handle) == IV_ERROR_SUCCESS) 
            {
                // ZK-Option in Verwendung.
                hwInfo.administrationFlags |= HIAF_HAS_ZK;
            }
        }

        // Pruefen ob eine passende Datei in die vorliegende Hardware eingespielt werden kann.
        if (0 == m_fileDevice.SearchInfosDir(szFileName, hwInfo)) 
        {
            int err = IV_ERROR_SUCCESS;
            // Hardwareinfo in Text aufloesen
            getInfos(hwInfo, _info);
            if (m_fileDevice.errorState() == FileIntelHex::Version || m_fileDevice.errorState() == FileIntelHex::DeviceUnknown)
            {
                err = -IV_ERROR_VERSION_DEVICEFILE;
            } 
            else if (m_fileDevice.errorState() == FileIntelHex::NoFileMatch)
            {
                err = -IV_ERROR_DEVICEFILE_MATCH;
            } 
            else 
            {
                err = -IV_ERROR_DEVICEFILE_LOAD;
            }
            return err;
        }
#endif
    } 
    else 
    {
        return -IV_ERROR_FILE_OPEN;
    }		

    //######################################################################
    // 1. Ermittlung ob auf dem Geraet eine Applicationfirmware laeuft.
    // 2. Wenn Punkt 1 == TRUE, dann wird geprueft ob Datensaetze verfuegbar sind.
    // 2. Wenn Punkt 2 == TRUE, wird die Aktion abgebrochen.
    char version[64];
    int modus = 0;
    // Geraetemodus (Application/Bootloader) ermitteln
    if ((iRes = Channel4ModuleTable::getVersion(deviceIdRemote, version)) != IV_ERROR_SUCCESS) 
    {
        if (IV_RESULT_MODULE(iRes) == 2)
        {
            // Bootloadermodus
            modus = 1;
        }
        else
        {
            return iRes;
        }
    }

    // 2013_04_17:
    // Es muss sichergestellt werden, dass eine Firmwareversion < 04.02.04.20 nur auf Timeboy Geraete mit Ladeschaltung aufgespielt werden kann.
    // Wenn dieses geschieht, dann geht der Timeboy aus und rebooted ständig.
    if ( m_fileDevice.GetHeaderVersionFirmware() >= "04.02.01.00" && m_fileDevice.GetHeaderVersionFirmware() < "04.02.04.20" )
    {
        // Im Bootloadermodus koennen wir das Flag der Ladeschaltung nicht auswerten, dadurch lassen wir das aufspielen zu.
        if ( modus == 0 && CDFSoftwarebuildVersion( version ) >= "04.02.04.20" )
        {
            if ( (hwInfo.typeDevice == IV_VALUE_TYPE_DEVICE_TIMEBOY || hwInfo.typeDevice == IV_VALUE_TYPE_DEVICE_TIMEBOY_MOBIL_PZE)
                && !hwInfo.componentCharge ) // Bei Timeboy's ohne Ladeschaltung ist das aufspielen unzulaessig.
            {
                return -IV_ERROR_DEVICEFILE_MATCH;
            }
        }
    }

    iRes = 0;
    // Abfrage nur wenn Geraet im Applicationmodus
    // Abfrage ob Daten vorhanden
    if (!modus && (iRes = checkData(deviceIdRemote)) == -1) 
    {
        return -IV_ERROR_UNREACHABLE;
    } 
    else if (iRes) 
    {
        return -IV_ERROR_DATA_AVAILABLE;
    }

    // Befehlsausfuehrung starten.
    iRes = writeDevice4( deviceIdRemote );

    if (iRes == IV_ERROR_SUCCESS)
    {
        // Ggf. unbekannte Dockingstation
        resetTimeboyStation();
    }

    return iRes;
}

int Channel4::writeDevice4( short deviceIdRemote )
{
    // Zeichensatz- und Grafikdaten
    FileDisplay *pFileDisplay = m_fileDevice.GetFileDisplay();
    FileDeviceText *pFileDeviceText = m_fileDevice.GetFileDeviceText();
    int res = IV_ERROR_SUCCESS;
    unsigned int address = 0;
	std::vector<unsigned char> buffer;
    unsigned char *buf;
    unsigned int size = 0;
    unsigned int crc = 0;
    char version[64];

    memset(version, 0, sizeof(version));
    // Bootversion lesen 
    if ((res = getVersionBootloader(deviceIdRemote, version)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Pruefen ob Streaming moeglich.
    clearStreamTransfer();
    // Pruefen ob die Uebertragung per Stream moeglich ist.
    if ((res = writeStream(deviceIdRemote, update, NULL, 0)) == IV_ERROR_SUCCESS) 
        setStreamTransfer(true);
    else
        setStreamTransfer(false);

    // Sind Textdaten vorhanden?
    if (pFileDeviceText->GetSizeData())
    {
        // Texte als ersten Block uebertragen
        // TEXTE

        address = 0;
        buf = pFileDeviceText->m_memData;
        size = (unsigned int)pFileDeviceText->m_sizeData;
        crc = crc32(buf, size);

        // Bei Streaming werden die inneren Bloecke wegelassen.
        if (!isStreamTransfer()) {
            // Systemstopp setzen
            res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_TEXT, address, size, crc);
            if (res != IV_ERROR_SUCCESS && (res & 0xFF) != 1)
            {
                return res;
            }
        }

        // Funktionsfehler nicht Blocktyp unbekannt?
        if ((res & 0xFF) != 1)
        {
            // Textdaten uebertragen
            if ((res = setFlash(deviceIdRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Im Modus Streaming ?, dann gepufferte Daten senden.
            if (isStreamTransfer() && m_streamBuffer.size()) {
                if ((res = writeStream(deviceIdRemote, text, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
                    // Gepufferte Daten loeschen.
                    clearStreamTransfer();
                    return res;
                }
                // Gepufferte Daten loeschen.
                clearStreamTransfer();
            }

            // Bei Streaming werden die inneren Bloecke wegelassen.
            if (!isStreamTransfer()) {
                // Systemstopp aufheben
                if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_TEXT, crc)) != IV_ERROR_SUCCESS)
                {
                    return res;
                }
            }
        }
        else
        {
            // Texte als Zeichensatz mit uebertragen.
            int err;
            if (!(err = pFileDisplay->AppendSegmentText(buf, size)))
            {
                if (FileIntelHex::AllocMem == pFileDisplay->errorState())
                {
                    return -IV_ERROR_ALLOC_MEM;
                }

                return -IV_ERROR_FILE_PROCESSING;
            }

            // Ruecksetzen des Flags fuer "gueltige Default-Textdaten vorhanden"
            unsigned char v = 0;
            if ((res = setEEPROM(deviceIdRemote, 0x0ECB, &v, 1)) != IV_ERROR_SUCCESS)
            {
                return res;
            }
        }
    }

    // Sind Displaydaten vorhanden?
    if (pFileDisplay->GetSizeData())
    {
        // ZEICHENSAETZE
        if (pFileDisplay->m_sizeFont)
        {
            address = 0;
            buf = pFileDisplay->m_memFont;
            size = pFileDisplay->m_sizeFont;
            crc = crc32(buf, size);

            // Bei Streaming werden die inneren Bloecke wegelassen.
            if (!isStreamTransfer()) {
                // Systemstopp setzen
                if ((res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_FONTS, address, size, crc)) != IV_ERROR_SUCCESS)
                {
                    return res;
                }
            }

            // Zeichensatzdaten uebertragen
            if ((res = setFlash(deviceIdRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Im Modus Streaming ?, dann gepufferte Daten senden.
            if (isStreamTransfer() && m_streamBuffer.size()) {
                if ((res = writeStream(deviceIdRemote, fonts, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
                    // Gepufferte Daten loeschen.
                    clearStreamTransfer();
                    return res;
                }
                // Gepufferte Daten loeschen.
                clearStreamTransfer();
            }

            // Bei Streaming werden die inneren Bloecke wegelassen.
            if (!isStreamTransfer()) {
                // Systemstopp aufheben
                if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_FONTS, crc)) != IV_ERROR_SUCCESS)
                {
                    return res;
                }
            }
        }

        // GRAFIKEN
        if (pFileDisplay->m_sizeGrafic)
        {
            address = 0;
            buf = pFileDisplay->m_memGrafic;
            size = pFileDisplay->m_sizeGrafic;
            crc = crc32(buf, size);

            // Bei Streaming werden die inneren Bloecke wegelassen.
            if (!isStreamTransfer()) {
                // Systemstopp setzen
                if ((res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_GRAFICS, address, size, crc)) != IV_ERROR_SUCCESS)
                {
                    return res;
                }
            }

            // Grafikdaten uebertragen
            if ((res = setFlash(deviceIdRemote, address, buf, size)) != IV_ERROR_SUCCESS)
            {
                return res;
            }

            // Im Modus Streaming ?, dann gepufferte Daten senden.
            if (isStreamTransfer() && m_streamBuffer.size()) {
                if ((res = writeStream(deviceIdRemote, graphics, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
                    // Gepufferte Daten loeschen.
                    clearStreamTransfer();
                    return res;
                }
                // Gepufferte Daten loeschen.
                clearStreamTransfer();
            }

            // Bei Streaming werden die inneren Bloecke wegelassen.
            if (!isStreamTransfer()) {
                // Systemstopp aufheben
                if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_GRAFICS, crc)) != IV_ERROR_SUCCESS) {

                    return res;
                }
            }
        }
    }

    // FIRMWARE

    address = 0;
    buffer = m_fileDevice.firmwareContent();
    crc = crc32( buffer.data(), buffer.size() );

    // Bei Streaming werden die inneren Bloecke wegelassen.
    if (!isStreamTransfer()) {
        // Systemstopp setzen
        if ((res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_FIRMWARE, address, buffer.size(), crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
    }

    // Firmwaredaten uebertragen
    if ((res = setFlash(deviceIdRemote, address, buffer.data(), buffer.size())) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Im Modus Streaming ?, dann gepufferte Daten senden.
    if (isStreamTransfer() && m_streamBuffer.size()) {
        if ((res = writeStream(deviceIdRemote, firmware, m_streamBuffer.data(), m_streamBuffer.size())) != IV_ERROR_SUCCESS) {
            // Gepufferte Daten loeschen.
            clearStreamTransfer();
            return res;
        }
        // Gepufferte Daten loeschen.
        clearStreamTransfer();
    }

    // Bei Streaming werden die inneren Bloecke wegelassen.
    if (!isStreamTransfer()) {
        // Systemstopp aufheben
        if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_FIRMWARE, crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
    }

    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS)
    {
        return res;
    }

    // Neuaushandlung bei verschluesselter Kommunikation
    getCryptSession().ResetSessionKey();

    // Firmwareversion des Geraetes.
    memset(_versionFirmware, 0, sizeof(_versionFirmware));

    return res;
}

int Channel4::writeModuleWithTimeboySlot(short deviceIdRemote, int type, char *szFileName)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = writeModule( deviceIdRemote, type, szFileName );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::writeModule(short deviceIdRemote, int type, char *szFileName)
{
    int res;
    unsigned int address = 0;
    unsigned char *buf = NULL;
    unsigned int size = 0;
    unsigned int crc = 0;
    //	char version[64];
    unsigned char val;
    bool evoLine;

    //	memset(version, 0, sizeof(version));
    // Bootversion lesen 
    //	if ((res = getVersionBootloader(deviceIdRemote, version)) != IV_ERROR_SUCCESS) {

    //		return res;
    //	}

    // Dateigroesse ermitteln
    // Datei oeffnen
    struct stat st;
    FILE *fp = NULL;

    if (stat(szFileName, &st) != 0 || st.st_size == 0 || (fp = fopen(szFileName, "rb")) == NULL) {

        res = -IV_ERROR_FILE_OPEN;
        goto error_label;
    }

    // Dateigroesse muss groesser 4kByte sein.
    if (st.st_size < 4096)
    {
        res = -IV_ERROR_FILE_FORMAT;
        goto error_label;
    }

    // Speicher reservieren
    buf = new unsigned char[st.st_size + 1];
    if (buf == NULL) {

        res = -IV_ERROR_FILE_OPEN;
        goto error_label;
    }

    // Daten laden
    if ((int)fread(buf, sizeof(unsigned char), st.st_size, fp) != st.st_size) {

        res = -IV_ERROR_FILE_READ;
        goto error_label;
    }

    // Zugriff ueber Struktur initialisieren
    _uploadModuleSignatur = (SUploadModuleSignatur*)&buf[st.st_size - sizeof(SUploadModuleSignatur)];
    // Gesamt-CRC pruefen
    crc = crc32(buf, st.st_size - sizeof(unsigned int));
    if (crc != _uploadModuleSignatur->crc32)
        //if (crc = crc32(buf, st.st_size))
    {
        res = -IV_ERROR_FILE_CRC;
        goto error_label;
    }

    // Pruefung auf unterstutzte Version
    if (_uploadModuleSignatur->version != 1)
    {
        res = -IV_ERROR_FILE_FORMAT;
        goto error_label;
    }

    unsigned char typeDevice;
    unsigned char typeBoard;

    // Gerätetyp abrufen.
    if ((res =  getTypeDevice(deviceIdRemote, typeDevice)) != IV_ERROR_SUCCESS)
    {
        if (res > 0) {

            // Wenn der Modulfehler -Funktion unbekannt- ist,
            // dann ist die Firmwareversion nicht kompatible zum Befehl.
            if (((res >> 8) & 0xFF) == 2) {

                // Hintergrundladen noch nicht implementiert
                res = -IV_ERROR_VERSION_FIRMWARE;
            } else if ((res & 0xFF) == 28) {

                // Passwort f?r Kommunikation wird ben?tigt.
                res = -IV_ERROR_NEED_PASSWORD;
            } else if ((res & 0xFF) == 29) {

                // Nach 3 maliger Passwortfalscheingabe ist Ger?t f?r 60 Sekunden gesperrt.
                res = -IV_ERROR_PASSWORD_LOCK;
            } else {

                // Funktionsfehler
                res = -IV_ERROR_PROCESSING;
            }
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }
    // Boardversion lesen.
    if ((res =  getTypeBoard(deviceIdRemote, typeBoard)) != IV_ERROR_SUCCESS)
    {
        if (res > 0) {

            // Wenn der Modulfehler -Funktion unbekannt- ist,
            // dann ist die Firmwareversion nicht kompatible zum Befehl.
            if (((res >> 8) & 0xFF) == 2) {

                // Hintergrundladen noch nicht implementiert
                res = -IV_ERROR_VERSION_FIRMWARE;
            } else if ((res & 0xFF) == 28) {

                // Passwort f?r Kommunikation wird ben?tigt.
                res = -IV_ERROR_NEED_PASSWORD;
            } else if ((res & 0xFF) == 29) {

                // Nach 3 maliger Passwortfalscheingabe ist Ger?t f?r 60 Sekunden gesperrt.
                res = -IV_ERROR_PASSWORD_LOCK;
            } else {

                // Funktionsfehler
                res = -IV_ERROR_PROCESSING;
            }
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }
    // Liegt ein Gerät von den Folgenden Typen vor, dann werden die weiteren Pruefungen unterlassen.
    // Auch wenn die Boardversion bei den "älteren" Gerätetypen >= 4.0 ist.
    // Es ist ab diesen mit der Softwareversion 04.03.xx ein anderes Hardware-Softwareinfo-Konzept notwendig.
    // IV_VALUE_TYPE_DEVICE_TERMINAL_2_8 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TERMINAL_4_3 und Board 4.1
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_5_7 und Board 1.2
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_VARIO_10 und Board 1.3
    // IV_VALUE_TYPE_DEVICE_IPC_EXTENSION_Q7 und Board 1.1
    // IV_VALUE_TYPE_DEVICE_IP_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_ZK_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_MOBIL_BOX und Board 1.2
    // IV_VALUE_TYPE_DEVICE_TIMEBOY_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_MOBIL_TBDOCK und Board 2.3
    // IV_VALUE_TYPE_DEVICE_ZK_KNOTEN und Board 1.0
    // IV_VALUE_TYPE_DEVICE_ZK_TERMINAL und Board 1.0
    evoLine = typeDevice == 0x0a 
        || typeDevice == 0x0b 
        || typeDevice == 0x0c 
        || typeDevice == 0x0d 
        || typeDevice == 0x0e
        || typeDevice == 0x0F
        || typeDevice == 0x10
        || typeDevice == 0x11
        || typeDevice == 0x12
        || typeDevice == 0x13
        || typeDevice == 0x14
        || typeDevice == 0x15;
    if ( !evoLine && typeBoard < 0x40 )
    {
        // Type des Moduls lesen
        if ((res = getTypeFingerprint(deviceIdRemote, val)) != IV_ERROR_SUCCESS) {

            if (res > 0) {

                // Wenn der Modulfehler -Funktion unbekannt- ist,
                // dann ist die Firmwareversion nicht kompatible zum Befehl.
                if (((res >> 8) & 0xFF) == 2) {

                    // Hintergrundladen noch nicht implementiert
                    res = -IV_ERROR_VERSION_FIRMWARE;
                } else if ((res & 0xFF) == 28) {

                    // Passwort f?r Kommunikation wird ben?tigt.
                    res = -IV_ERROR_NEED_PASSWORD;
                } else if ((res & 0xFF) == 29) {

                    // Nach 3 maliger Passwortfalscheingabe ist Ger?t f?r 60 Sekunden gesperrt.
                    res = -IV_ERROR_PASSWORD_LOCK;
                } else {

                    // Funktionsfehler
                    res = -IV_ERROR_PROCESSING;
                }
            }

            // Fehlerhafte durchfuehrung zurueckmelden.
            goto error_label;
        }

        // Pruefung auf zu aktualisierendes Modul
        if (_uploadModuleSignatur->type != val)
        {
            // In der Datei liegt ein Update fuer eine andere Modulversion vor
            res = -IV_ERROR_FILE_DATA_FORMAT;
            goto error_label;
        }
    }


    // Ggf. gesetztes Streaming deaktivieren.
    setStreamTransfer(false);

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_UPDATE, address, size, crc)) != IV_ERROR_SUCCESS) {

        if (res > 0) {

            // Wenn der Modulfehler -Funktion unbekannt- ist,
            // dann ist die Firmwareversion nicht kompatible zum Befehl.
            if (((res >> 8) & 0xFF) == 2) {

                // Hintergrundladen noch nicht implementiert
                res = -IV_ERROR_VERSION_FIRMWARE;
            } else if ((res & 0xFF) == 28) {

                // Passwort f?r Kommunikation wird ben?tigt.
                res = -IV_ERROR_NEED_PASSWORD;
            } else if ((res & 0xFF) == 29) {

                // Nach 3 maliger Passwortfalscheingabe ist Ger?t f?r 60 Sekunden gesperrt.
                res = -IV_ERROR_PASSWORD_LOCK;
            } else {

                // Funktionsfehler
                res = -IV_ERROR_PROCESSING;
            }
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }


    // FIRMWARE des Biokey laden

    address = 0;
    size   = st.st_size - sizeof(SUploadModuleSignatur);
    crc = crc32(buf, size);

    // Systemstopp setzen
    if ((res = beginSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_FINGERPRINT, address, size, crc)) != IV_ERROR_SUCCESS) {

        if (res > 0) {

            // Wenn der Modulfehler -Funktion unbekannt- ist,
            // dann ist die Firmwareversion nicht kompatible zum Befehl.
            if (((res >> 8) & 0xFF) == 2) {

                // Fingerprintmodul noch nicht implementiert
                res = -IV_ERROR_VERSION_FIRMWARE;
            } else if ((res & 0xFF) == 28) {

                // Passwort f?r Kommunikation wird ben?tigt.
                res = -IV_ERROR_NEED_PASSWORD;
            } else if ((res & 0xFF) == 29) {

                // Nach 3 maliger Passwortfalscheingabe ist Ger?t f?r 60 Sekunden gesperrt.
                res = -IV_ERROR_PASSWORD_LOCK;
            } else {

                // Funktionsfehler
                res = -IV_ERROR_PROCESSING;
            }
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }

    // Firmwaredaten uebertragen
    if ((res = setFlash(deviceIdRemote, address, buf, size)) != IV_ERROR_SUCCESS) {

        if (res > 0) {

            // Funktionsfehler
            res = -IV_ERROR_PROCESSING;
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }

    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_FINGERPRINT, crc)) != IV_ERROR_SUCCESS) {

        if (res > 0) {

            // Wenn der Modulfehler -Funktion unbekannt- ist,
            // dann ist die Firmwareversion nicht kompatible zum Befehl.
            if (((res >> 8) & 0xFF) == 2) {

                // Hintergrundladen noch nicht implementiert
                res = -IV_ERROR_VERSION_FIRMWARE;
            } else if ((res & 0xFF) == 28) {

                // Passwort f?r Kommunikation wird ben?tigt.
                res = -IV_ERROR_NEED_PASSWORD;
            } else if ((res & 0xFF) == 29) {

                // Nach 3 maliger Passwortfalscheingabe ist Ger?t f?r 60 Sekunden gesperrt.
                res = -IV_ERROR_PASSWORD_LOCK;
            } else {

                // Funktionsfehler
                res = -IV_ERROR_PROCESSING;
            }
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }


    crc = 0;
    // Systemstopp aufheben
    if ((res = endSetSegment(deviceIdRemote, IV_VALUE_SEGMENT_UPDATE, crc)) != IV_ERROR_SUCCESS) {

        if (res > 0) {

            // Funktionsfehler
            res = -IV_ERROR_PROCESSING;
        }

        // Fehlerhafte durchfuehrung zurueckmelden.
        goto error_label;
    }

    // Ausfuehrung erfolgreich abgeschlossen

error_label:

    if (buf) delete[] buf;
    if (fp) fclose(fp);

    return res;
}

int Channel4::writeMemToFile( short deviceIdRemote, int type, int address, int size, char *fileName )
{
    StructMemFileHeader header;
    int res = IV_ERROR_SUCCESS, chunk;
    char buf[0x400];
    int baseAddress = 0, offsAddress = 0;

    FILE *fp = fopen(fileName, "wb");
    if (fp == NULL)
    {
        return -IV_ERROR_FILE_OPEN;
    }

    // Dateiheader aufsetzen
    memset(&header, 0, sizeof(header));
    header.structSize = sizeof(header);
    header.type = type;
    header.baseAddress = baseAddress;
    header.memAddress = address;
    header.memSize = size;
    // Dateiheader schreiben
    if (fwrite(&header, sizeof(header), 1, fp) != sizeof(header))
    {
        return -IV_ERROR_FAIL;
    }

    switch (type)
    {
    case 0:
        break;

    case 1:
        if ((res = getEEPROM(deviceIdRemote, address, (unsigned char*)buf, size)) != IV_ERROR_SUCCESS)
        {
            goto label_out;
        }
        break;
    case 2:
        if ((res = getFlash(deviceIdRemote, address, (unsigned char*)buf, size)) != IV_ERROR_SUCCESS)
        {
            goto label_out;
        }
        break;

    case 3:
        break;
    }

label_out:
    if (fp) fclose(fp);
    return res;
}

int Channel4::writeFileToMem( short deviceIdRemote, char *fileName )
{
    StructMemFileHeader header;
    int res = IV_ERROR_SUCCESS, chunk = 0;
    int offsAddress = 0;
    char buf[0x400];

    FILE *fp = fopen(fileName, "rb");
    if (fp == NULL)
    {
        return -IV_ERROR_FILE_OPEN;
    }

    // Dateiheader lesen
    if (fread(&header, sizeof(header), 1, fp) != sizeof(header))
    {	
        res = -IV_ERROR_FAIL;
        goto label_out;
    }
    switch (header.type)
    {
    case 0://MB_ACCESS_SRAM:
        break;

    case 1://MB_ACCESS_EEPROM:
        if ((res = setEEPROM(deviceIdRemote, header.memAddress, (unsigned char*)buf, chunk)) != IV_ERROR_SUCCESS)
        {
            goto label_out;
        }
        break;

    case 2://MB_ACCESS_FLASH:
        if ((res = setFlash(deviceIdRemote, header.memAddress, (unsigned char*)buf, chunk)) != IV_ERROR_SUCCESS)
        {
            goto label_out;
        }
        break;

    case 3://MB_ACCESS_PROGRAM:
        break;

    case 5://MB_ACCESS_FLASH_PAGE:
        break;

    case 6://MB_ACCESS_MMC:
        break;
    }

label_out:
    if (fp) fclose(fp);
    return res;
}

// Abfrage ob Datensaetze im AEIII+ vorliegen.
// Erst ab Datensatzweisem lesen.
int Channel4::checkData( short deviceIdRemote )
{
    int iRes = IV_ERROR_SUCCESS;
    unsigned char buf[0x255];
    int count = 0;
    unsigned char status = 0;

    // Befehlsausfuehrung starten.
    iRes = getRecord( deviceIdRemote, buf, count, status );

    if (iRes == IV_ERROR_SUCCESS)
    {
        // OK
        iRes = status;
    }
    else
    {
        iRes = -1;
    }

    return iRes;
}

// ########################################################
// Überlagerungen aus Channel4ModuleApplication
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int Channel4::getRecord( short deviceIdRemote, unsigned char *buf, int &count, unsigned char &status )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1)) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getRecord( deviceIdRemote, buf, count, status );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setRecord( short deviceIdRemote, unsigned char &status )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        // Datensatz quittieren
        res = Channel4ModuleTable::setRecord( deviceIdRemote, status );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::clearRecords( short deviceIdRemote )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::clearRecords( deviceIdRemote );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::restoreRecords( short deviceIdRemote )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::restoreRecords( deviceIdRemote );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::showInfo( short deviceIdRemote, unsigned char maschine, unsigned char time, unsigned char font, unsigned char audio, short flags, char *text )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::showInfo( deviceIdRemote, maschine, time, font, audio, flags, text );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::showMessage( short deviceIdRemote, unsigned char time, unsigned char action, unsigned char audio, char *text )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::showMessage( deviceIdRemote, time, action, audio, text );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setVariable( short deviceIdRemote, char *name, char *val )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::setVariable( deviceIdRemote, name, val );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getVariable( short deviceIdRemote, char *name, char *val )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getVariable( deviceIdRemote, name, val );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getVersionFirmwareFromFile( char *fileName, char *versionFirmware )
{
    // Lokale Membervariable reseten
    m_fileDevice.reset();
	string version;

    if ( !m_fileDevice.GetVersionFirmware(fileName, version) )
    {
        switch ( m_fileDevice.errorState() )
        {
		case FileDeviceZip::FileOpen:
            return -IV_ERROR_FILE_OPEN;

		case FileDeviceZip::HeadCrc:
            return -IV_ERROR_CRC_PACKET;

		case FileDeviceZip::Version:
        case FileDeviceZip::DeviceUnknown:
        case FileDeviceZip::FirmwareVersionLength:
        case FileDeviceZip::FunctionUnavailable:
            return -IV_ERROR_FUNCTION_UNAVAILABLE;
        }

        return -IV_ERROR_FUNCTION_UNAVAILABLE;
    }

	strcpy( versionFirmware, version.data() );
    return IV_ERROR_SUCCESS;
}

int Channel4::SetTypeFont( short deviceIdRemote, unsigned char type )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::SetTypeFont( deviceIdRemote, type );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

// ########################################################
// Überlagerungen aus Channel4ModuleSystem
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

int Channel4::systemReadRecord( short deviceIdRemote, unsigned char *buf, int &count, unsigned char &status )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::systemReadRecord( deviceIdRemote, buf, count, status );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::systemQuitRecord( short deviceIdRemote, unsigned char &status )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::systemQuitRecord( deviceIdRemote, status );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::systemRestoreRecords( short deviceIdRemote )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::systemRestoreRecords( deviceIdRemote );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getRTC( short deviceIdRemote, char *time )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getRTC( deviceIdRemote, time );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setRTC( short deviceIdRemote, char *time )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::setRTC( deviceIdRemote, time );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getVersion( short deviceIdRemote, char *version )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getVersion( deviceIdRemote, version );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::systemReset( short deviceIdRemote, unsigned short modus )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::systemReset( deviceIdRemote, modus );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getSerial(short deviceIdRemote, unsigned int &serial)
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getSerial( deviceIdRemote, serial );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setRelay( short deviceIdRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char status, unsigned short time )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::setRelay( deviceIdRemote, name, busNum, relayNum, status, time );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getRelay( short deviceIdRemote, char *name, unsigned char busNum, unsigned char relayNum, unsigned char &status, unsigned short &time )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getRelay( deviceIdRemote, name, busNum, relayNum, status, time );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setSystemVariable( short deviceIdRemote, char *name, char *val )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::setSystemVariable( deviceIdRemote, name, val );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::getSystemVariable( short deviceId, const char *name, char *val )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceId, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::getSystemVariable( deviceId, name, val );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceId, 0, res );
    }

    return res;
}

int Channel4::pressVirtualKey( short deviceIdRemote, unsigned char vk, unsigned char flags )
{
    int res;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ((res = setTimeboySlotState( deviceIdRemote, 1 )) == IV_ERROR_SUCCESS )
    {
        // Befehlsausfuehrung starten.
        res = Channel4ModuleTable::pressVirtualKey( deviceIdRemote, vk, flags );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceIdRemote, 0, res );
    }

    return res;
}

int Channel4::setLEDState( short deviceId, unsigned slot, unsigned char id, unsigned char state )
{
    return Channel4ModuleSystem::setLEDState( deviceId, slot, id, state );
}

int Channel4::getLEDState( short deviceId, unsigned slot, unsigned char id, unsigned char &state )
{
    return Channel4ModuleSystem::getLEDState( deviceId, slot, id, state );
}

// ##################################################################
/**
* @brief Ermittelt eine Sammlung von Informationen zum Gerät.
*
*/
int Channel4::getDeviceInfo( short deviceId, DeviceInfo &deviceInfo )
{
    int res = IV_ERROR_SUCCESS;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceId, 1 )) == IV_ERROR_SUCCESS )
    {
        unsigned int flags = 0;
        unsigned int serial;
        unsigned char type;
        unsigned char available;
        bool haveFunction;
		std::array<char, IV_PACKET_LENGTH_DATA> buffer;

        // Initalisieren der DeviceInfo.
        deviceInfo.clear();

		buffer.fill( '\0' );
        // Abfrage der Bootloaderversion
        if ( (res = getVersionBootloader( deviceId, buffer.data() )) != IV_ERROR_SUCCESS )
            return res;

        // Der Infoklasse übergeben.
        deviceInfo.setVersionBootloader( buffer.data() );

        // HIP-Datei lesen (importiert ggf. "alte" Hardwareinfo in die Datei).
        HardwareInfo hardwareInfo;
        if ( (res = getHardwareInfo( deviceId, hardwareInfo )) != IV_ERROR_SUCCESS )
        {
            switch ( res ) 
            {
            case 28: // Funktionsfehler: Setup-Passwort wird gefordert.
                break;

            case 29: // Funktionsfehler: Setup-Passwort wird gefordert, Gerät wegen zu vielen Fehlanfragen gesperrt.
                break;

            default:
                return res;
            }
        }

        // der Infoklasse übergeben.
        deviceInfo.setHardwareInfo( hardwareInfo );
        hardwareInfo.clear();

		buffer.fill( '\0' );
        // Abfrage der Firmwareversion
        if ( (res = getVersion( deviceId, buffer.data() )) != IV_ERROR_SUCCESS )
        {
            // War die Funktion unbekannt?
            if ( res == -IV_ERROR_FUNCTION_UNAVAILABLE )
            {
				buffer.fill( '\0' );
                // Firmwareversion nicht verfuegbar.
                // Geraet befindet sich im Bootloadermodus.
                deviceInfo.setBootloaderMode( true );
            }
            else
            {
                return res;
            }
        }
		else
		{
			// Der Infoklasse übergeben.
			deviceInfo.setVersionFirmware( buffer.data() );
		}

        SoftwareInfo softwareInfo;
        if ( (res = getSoftwareInfo( deviceId, softwareInfo )) != IV_ERROR_SUCCESS )
            return res;

        // Der Infoklasse übergeben.
        deviceInfo.setSoftwareInfo( softwareInfo );
        softwareInfo.clear();

        FirmwareOptions firmwareOptions;
        if ( (res = getFirmwareOptions( deviceId, firmwareOptions )) != IV_ERROR_SUCCESS )
            return res;

        // Der Infoklasse üebrgeben.
        deviceInfo.setFirmwareOptions( firmwareOptions );
        firmwareOptions.clear();

        // Abfrage der Seriennummer.
        if ( (res = getSerial( deviceId, serial )) != IV_ERROR_SUCCESS )
            return res;

        // Der Infoklasse übergeben.
        deviceInfo.setSerialNumber( serial );

		buffer.fill( '\0' );
        // Abfrage des Setuppasswortkeys
        if ( (res = passwordKey( deviceId, buffer.data() )) != IV_ERROR_SUCCESS )
        {
            // War die Funktion unbekannt?
            if ( res == -IV_ERROR_FUNCTION_UNAVAILABLE )
            {
				buffer.fill( '\0' );
                // Passwordkey nicht verfuegbar.
            }
            else
            {
                return res;
            }
        }

        // Der Infoklasse übergeben.
        deviceInfo.setPasswordKey( buffer.data() );

        // Zusätzliche Informationen zu verbauten Modulen verfügbar?
        std::vector<std::string> names;

        if ( deviceInfo.hardwareInfo().moduleBiokey().size() )
            names.push_back( "FINGERPRINT.MODULE" );

        // Die Info liegt lediglich bei folgenden USB-Hosts vor.
        if ( deviceInfo.hardwareInfo().moduleUSBHostCommunication( boost::assign::list_of( EnumModuleUSBHostCommunication::Vinculum ) ).size() )
            names.push_back( "USBHOST.MODULE" );

        if ( deviceInfo.hardwareInfo().moduleGPS().size() )
            names.push_back( "GPS.MODULE" );

        if ( deviceInfo.hardwareInfo().moduleLANCommunication().size() || deviceInfo.hardwareInfo().moduleWLANCommunication().size() )
            names.push_back( "COM.TCPIP" );

        for ( const std::string &name : names )
        {     
			buffer.fill( '\0' );
            int size = buffer.size();
            if ( (res = getInfo( deviceId, name.c_str(), "", buffer.data(), size )) != IV_ERROR_SUCCESS )
            {
                // War die Funktion im Modul unbekannt?
                if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
                {
                    res = IV_ERROR_SUCCESS;
                    // Modulinfo nicht verfuegbar.
					buffer.fill( '\0' );
                    std::strcpy( buffer.data(), "NA" );
                }
                else if ( (res > 0) && (((res & 0xFF) == 1) || ((res & 0xFF) == 2)) )
                {
                    res = IV_ERROR_SUCCESS;
                    // Systemvariable ist nicht verfügbar.
                    buffer.fill( '\0' );
                    std::strcpy( buffer.data(), "NA" );
                }
                else if ( (res > 0) && (((res & 0xFF) == 6) || ((res & 0xFF) == 7) || ((res & 0xFF) == 8)) )
                {
                    if ( (res = getSystemVariable( deviceId, "com.mac", buffer.data() )) != IV_ERROR_SUCCESS || std::strlen( buffer.data() ) != 12 )
                    {                    
                        res = IV_ERROR_SUCCESS;
                        // Abfrage steht nicht zur Verfügung.
                        buffer.fill( '\0' );
                        std::strcpy( buffer.data(), "NA" );
                    }
                    else
                    {
                        // Bindestriche einfügen
                        std::string mac( buffer.data() );
                        int i = mac.size() - 2;
                        for ( ; i >= 2; i -= 2 )
                            mac.insert( i, "-" );
                        
                        std::strcpy( buffer.data(), mac.c_str() );
                    }
                }
                else if ( res == 28 || res == 29 )
                {
                    // 28 == Funktionsfehler: Setup-Passwort wird gefordert.
                    // 29 == Funktionsfehler: Setup-Passwort wird gefordert, Gerät wegen zu vielen Fehlanfragen gesperrt.

                    res = IV_ERROR_SUCCESS;
                    // Abfrage steht nicht zur Verfügung.
                    buffer.fill( '\0' );
                    std::strcpy( buffer.data(), "NA" );

                    break;
                }
                else
                {
                    return res;
                }
            }

            if ( name == "FINGERPRINT.MODULE" )
                deviceInfo.setVersionFingerprint( buffer.data() );

            else if ( name == "USBHOST.MODULE" )
                deviceInfo.setVersionUSBHost( buffer.data() );

            else if ( name == "GPS.MODULE" )
                deviceInfo.setVersionGPS( buffer.data() );

            else if ( name == "COM.TCPIP" )
                deviceInfo.setMacAddress( buffer.data() );
        }

        // Zusätzlich folgende Systemvariablen abrufen.
        names.clear();

        // Zutrittskontrolle nur bei verbautem RS485 aktiv.
        if ( deviceInfo.hardwareInfo().moduleRS485Communication().size() )
            names.push_back( "ACCESS.MASTERID" );

        if ( deviceInfo.hardwareInfo().deviceType() == HardwareInfo::TimeboyIV || deviceInfo.hardwareInfo().deviceType() == HardwareInfo::TimeboyMobilPZE )
            names.push_back( "COM.GROUPID" );

        for ( const std::string &name : names )
        {     
            buffer.fill( '\0' );
            int size = buffer.size();
            if ( (res = getSystemVariable( deviceId, name.c_str(), buffer.data() )) != IV_ERROR_SUCCESS )
            {
                // War die Funktion im Modul unbekannt?
                if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
                {
                    res = IV_ERROR_SUCCESS;
                    // Modulinfo nicht verfuegbar.
                    buffer.fill( '\0' );
                    std::strcpy( buffer.data(), "NA" );
                }
                else if ( (res > 0) && (((res & 0xFF) == 1) || ((res & 0xFF) == 2)) )
                {
                    res = IV_ERROR_SUCCESS;
                    // Major- oder Minorname nicht verfügbar.
                    buffer.fill( '\0' );
                    std::strcpy( buffer.data(), "NA" );
                }
                else if ( res == 28 || res == 29 )
                {
                    // 28 == Funktionsfehler: Setup-Passwort wird gefordert.
                    // 29 == Funktionsfehler: Setup-Passwort wird gefordert, Gerät wegen zu vielen Fehlanfragen gesperrt.

                    res = IV_ERROR_SUCCESS;
                    // Abfrage steht nicht zur Verfügung.
                    buffer.fill( '\0' );
                    std::strcpy( buffer.data(), "NA" );

                    break;
                }
                else
                {
                    return res;
                }
            }

            if ( name == "ACCESS.MASTERID" )
                deviceInfo.setAccessControlMasterId( buffer.data() );

            else if ( name == "COM.GROUPID" )
                deviceInfo.setTimeboyGroupId( buffer.data() );
        }

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceId, 0, res );
    }

    return res;
}

int Channel4::getHardwareInfo( short deviceId, HardwareInfo &hardwareInfo )
{
    int res = IV_ERROR_SUCCESS;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceId, 1 )) == IV_ERROR_SUCCESS )
    {
        unsigned char val = 0;

        // Initalisieren.
        hardwareInfo.clear();

        // Lesen der Hardwareconfig-Nr
        if ( (res = getHWConfig( deviceId, val )) != IV_ERROR_SUCCESS )
            return res;

        // Ab Version 3 kommt eine geänderte HWI zum Einsatz.
        if ( val < 3 )
        {
            unsigned char typeDevice;
            unsigned char typeBoard;
            // Lesen des Gerätetypen
            if ( (res = getTypeDevice( deviceId, typeDevice )) != IV_ERROR_SUCCESS )
                return res;

            // Lesen der Boardversion
            if ( (res = getTypeBoard( deviceId, typeBoard )) != IV_ERROR_SUCCESS )
                return res;

            std::array<unsigned char, sizeof(StructHardwareInfo)> dshwi;
            dshwi.fill( '\0' );
            StructHardwareInfo &shw = reinterpret_cast<StructHardwareInfo &>( dshwi );
            shw.version = val;
            shw.typeDevice = typeDevice;
            shw.typeBoard = typeBoard;

            // Prüfen ob es sich um ein MasterIV handelt.
            if ( hardwareInfo.isMasterIV( shw ) )
            {
                // Lesen der Hardwareinfo
                if ( (res = Channel4ModuleTable::getHardwareInfo( deviceId, shw )) != IV_ERROR_SUCCESS )
                    return res;

                // Vorgängerversion importieren
                if ( !hardwareInfo.importStructHardwareInfo( shw ) )
                    return -IV_ERROR_FUNCTION_UNAVAILABLE;

                return res;
            }
            // else
            // Es handelt sich um eine Zwischenvariante bei der Gerätetyp und Boardrevision noch von der
            // Vorgängerversion genutzt wurden.
        }

        //
        unsigned int address, size, crc;
        address = size = crc = 0;

        // Segmentbegin: Bedienung stoppen
        if ((res = beginGetSegment(deviceId, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            if ((res & 0xFF) == 1)
            {
                // Funktionsfehler Typ: IV_VALUE_SEGMENT_GET_UPDATE_BG unbekannt.
                // Wenn dass der Fall ist, wird das Auslesen noch nicht unterstuetzt.
                res = -IV_ERROR_FUNCTION_UNAVAILABLE;
            }
            else
            {
                return res;
            }
        }
        trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG, address, size, crc);

        // Daten aus dem Gerät in die Datei der HardwareInfo lesen.
        FileHardwareInfo &fileHardwareInfo = hardwareInfo.fileHardwareInfo();
        fileHardwareInfo.reset();

        //////////////////////////////
        // Begin

        // Segmentbegin: Config uebertragen
        if ((res = beginGetSegment(deviceId, IV_VALUE_SEGMENT_HARDWARECONFIGS, address, size, crc)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        trace("%s(%i):INFO(1): %s [ beginGetSegment(%i): %i, %i, 0x%08X ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP, address, size, crc);

        if ( size == 0 )
        {
            // Es liegen keine Daten vor.
            res = -IV_ERROR_NO_DATA_AVAILABLE;
            goto label_no_data;
        }
        //////////////////////////////
        // Daten
        hardwareInfo.fileHardwareInfo().setDataSize( size );

        // HardwareInfo lesen
        if ((res = getFlash(deviceId, address, fileHardwareInfo.data(), fileHardwareInfo.dataSize())) != IV_ERROR_SUCCESS)
        {
            return res;
        }

        //////////////////////////////
        // Ende
label_no_data:
        // Segmentend: Setup uebertragen
        if ((res = endGetSegment(deviceId, IV_VALUE_SEGMENT_HARDWARECONFIGS)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_SETUP);

        // Segmentend: Bedienung stoppen
        if ((res = endGetSegment(deviceId, IV_VALUE_SEGMENT_GET_UPDATE_BG)) != IV_ERROR_SUCCESS)
        {
            return res;
        }
        trace("%s(%i):INFO(1): %s [ endGetSegment(%i) ]\n", __FILE__, __LINE__, __func__, IV_VALUE_SEGMENT_GET_UPDATE_BG);

        if ( size )
        {
            // Datenanzahl in Datei uebernehmen.
            unsigned int crcTest = GetCRC32(fileHardwareInfo.data(), fileHardwareInfo.dataSize());
            if (crc != crcTest)
            {
                trace("%s(%i):ERROR(1): %s [ CRC von Segment falsch: 0x%08X, 0x%08X ]\n", __FILE__, __LINE__, __func__, crc, crcTest);
                return -IV_ERROR_CRC_SEGMENT;
            }

            // Baum aus den Daten erzeugen.
            if ( !fileHardwareInfo.readSerialize() )
            {
                trace("%s(%i):ERROR(1): %s [ Lesen der Datei gescheitert. ]\n", __FILE__, __LINE__, __func__);
                return -IV_ERROR_FILE_WRITE;
            }
        }

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceId, 0, res );
    }

    return res;
}

// ##################################################################
/**
* @brief Ermittelt eine Sammlung von Informationen zum Gerät.
*
*/
int Channel4::getSoftwareInfo( short deviceId, SoftwareInfo &softwareInfo )
{
    int res = IV_ERROR_SUCCESS;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceId, 1 )) == IV_ERROR_SUCCESS )
    {
        StructSoftwareInfo swInfo;
        int size = sizeof(StructSoftwareInfo);
        memset( &swInfo, 0, sizeof(StructSoftwareInfo) );

        // Initalisieren
        softwareInfo.clear();

        // Abfrage der Softwareinfo
        if ( (res = Channel4ModuleTable::getSoftwareInfo( deviceId, (unsigned char*)&swInfo, size )) != IV_ERROR_SUCCESS )
        {
            // War die Funktion im Modul unbekannt?
            if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
            {
                // Softwareinformation nicht verfuegbar.
            }
            else
            {
                return res;
            }
        }

        // Der Infoklasse übergeben.
        softwareInfo.setStructSoftwareInfo( swInfo );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceId, 0, res );
    }

    return res;
}

// ##################################################################
/**
* @brief Ermittelt eine Sammlung von Informationen zum Gerät.
*
*/
int Channel4::getFirmwareOptions( short deviceId, FirmwareOptions &firmwareOptions )
{
    int res = IV_ERROR_SUCCESS;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceId, 1 )) == IV_ERROR_SUCCESS )
    {
        bool haveFunction;
        unsigned int flags = 0;
        unsigned char type;
        unsigned char available;

        // Initalisieren
        firmwareOptions.clear();

        // Zusaetzlich das Flag fuer "Setup und Listen beibehalten" auswerten.
        if ( (res = getSystemFlags( deviceId, flags )) != IV_ERROR_SUCCESS )
        {
            // War die Funktion im Modul unbekannt?
            if ( (res > 0) && (((res >> 8) & 0xFF) == 2) )
            {
                // Versuchen direkt an der Adresse zu lesen und zu setzen
                if ( getEEPROM( deviceId, 0x0ECC, (unsigned char*)&flags, sizeof(unsigned int) ) == IV_ERROR_SUCCESS )
                {
                    // Der Infoklasse übergeben.
                    firmwareOptions.setSystemFlags( flags );
                }
            }
        }
        else
        {
            // Der Infoklasse übergeben.
            firmwareOptions.setSystemFlags( flags );
        }

#undef CHECK_COULD_HAVE_FUNCTION
#define CHECK_COULD_HAVE_FUNCTION( x ) \
        haveFunction = true; \
        if ( (res = x) != IV_ERROR_SUCCESS ) \
        { \
            if ( (res > 0)  && (((res >> 8) & 0xFF) == 2) ) \
            { \
                res = IV_ERROR_SUCCESS; \
                /* Funktion kann ggf. aufgrund eines älteren Firmwarestandes undefiniert sein. */ \
                haveFunction = false; \
            } \
            else \
            { \
                /* Davon ausgehen, dass das Geraet unerreichbar ist.*/ \
                return res; \
            } \
        }

        // Typ ermitteln
        CHECK_COULD_HAVE_FUNCTION( getTypeExternalReader( deviceId, type ) );
        if ( haveFunction )
            firmwareOptions.setTypeExternalReader( type );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionIOModule( deviceId, available ) );
        if ( haveFunction )
            firmwareOptions.setOptionIOModule( available );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionTBAdaption( deviceId, available ) );
        if ( haveFunction )
            firmwareOptions.setOptionTBAdaption( available );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionComServer(deviceId, available) );
        if ( haveFunction )
            firmwareOptions.setOptionComServer( available );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionTPWrite( deviceId, available ) );
        if ( haveFunction )
            firmwareOptions.setOptionTPWrite( available );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionHTTP( deviceId, available ) );
        if ( haveFunction )
            firmwareOptions.setOptionHTTP( available );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionLanguage(deviceId, available) );
        if ( haveFunction )
            firmwareOptions.setOptionLanguage( available );

        // Option ermitteln
        CHECK_COULD_HAVE_FUNCTION( getOptionBiokeyUpdate( deviceId, available ) );
        if ( haveFunction )
            firmwareOptions.setOptionBiokeyUpdate( available );

        unsigned short mask, options;
        // SetupOption: Signalverarbeitung
        CHECK_COULD_HAVE_FUNCTION( getSetupOptions( deviceId, mask, options ) );
        if ( haveFunction )
            firmwareOptions.setSetupOptions( mask, options );

        unsigned char po_mask, po_options;
        // ErweiterteOption: Data on Card
        CHECK_COULD_HAVE_FUNCTION( getPayableOptions( deviceId, po_mask, po_options ) );
        if ( haveFunction )
            firmwareOptions.setPayableOptions( po_mask, po_options );

        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceId, 0, res );
    }

#undef CHECK_COULD_HAVE_FUNCTION

    return res;
}

// ##################################################################
/**
* @brief Ermittelt eine Sammlung von Informationen zum Gerät.
*
*/
int Channel4::setFirmwareOptions( short deviceId, const FirmwareOptions &firmwareOptions )
{
    int res = IV_ERROR_SUCCESS;

    // Ggf. Beginn der Kommunikation ueber Mehrfachdocking starten.
    if ( (res = setTimeboySlotState( deviceId, 1 )) == IV_ERROR_SUCCESS )
    {



        // Versuchen die rote LED abzuschalten.
        setTimeboySlotState( deviceId, 0, res );
    }

    return res;
}

int Channel4::startTransparencyMode( unsigned char deviceAddress, unsigned short id, unsigned short flags, unsigned int sizeReadBuffer, unsigned int sizeWriteBuffer, int timeout )
{
    return Channel4ModuleSystem::startTransparencyMode( deviceAddress, id, flags, sizeReadBuffer, sizeWriteBuffer, timeout );
}

int Channel4::stopTransparencyMode( unsigned char deviceAddress )
{
    return Channel4ModuleSystem::stopTransparencyMode( deviceAddress );
}

bool Channel4::isTransparencyMode()
{
    return Channel4ModuleSystem::isTransparencyMode();
}
